@@ -32,15 +32,27 @@ import (
 	golog "log"
 	"os"
 	"os/exec"
+	"path/filepath"
+	"regexp"
 	"strconv"
 	"strings"
 	"time"
 
-	"github.com/apache/trafficcontrol/cache-config/t3c-apply/config"
-	"github.com/apache/trafficcontrol/cache-config/t3cutil"
-	"github.com/apache/trafficcontrol/lib/go-atscfg"
-	"github.com/apache/trafficcontrol/lib/go-log"
-	"github.com/apache/trafficcontrol/lib/go-tc"
+	"github.com/apache/trafficcontrol/v8/cache-config/t3c-apply/config"
+	"github.com/apache/trafficcontrol/v8/cache-config/t3cutil"
+	"github.com/apache/trafficcontrol/v8/lib/go-atscfg"
+	"github.com/apache/trafficcontrol/v8/lib/go-log"
+	"github.com/apache/trafficcontrol/v8/lib/go-tc"
+)
+
+const (
+	t3cgen       = `t3c generate`
+	t3cupd       = `t3c update`
+	t3cdiff      = `t3c-diff`
+	t3cchkrefs   = `t3c check refs`
+	t3cchkreload = `t3c check reload`
+	t3creq       = `t3c request`
+	t3cpreproc   = `t3c preprocess`
 )
 
 type ServerAndConfigs struct {
@@ -48,6 +60,9 @@ type ServerAndConfigs struct {
 	ConfigFiles json.RawMessage
 }
 
+var stripDate = regexp.MustCompile(`\[\w{3}\s{1,2}\d{1,2}\s\d{2}:\d{2}:\d{2}\.\d{3}\]\s`)
+var t3cpath string = filepath.Join(t3cutil.InstallDir(), `t3c`)
+
 // generate runs t3c-generate and returns the result.
 func generate(cfg config.Cfg) ([]t3cutil.ATSConfigFile, error) {
 	configData, err := requestConfig(cfg)
@@ -55,7 +70,9 @@ func generate(cfg config.Cfg) ([]t3cutil.ATSConfigFile, error) {
 		return nil, errors.New("requesting: " + err.Error())
 	}
 	args := []string{
+		`generate`,
 		"--dir=" + cfg.TsConfigDir,
+		"--cache=" + cfg.CacheType,
 	}
 
 	if cfg.LogLocationErr == log.LogLocationNull {
@@ -80,18 +97,22 @@ func generate(cfg config.Cfg) ([]t3cutil.ATSConfigFile, error) {
 	if cfg.Files == t3cutil.ApplyFilesFlagReval {
 		args = append(args, "--revalidate-only")
 	}
+	if cfg.LocalATSVersion != "" {
+		args = append(args, "--ats-version="+cfg.LocalATSVersion)
+	}
 	args = append(args, "--via-string-release="+strconv.FormatBool(!cfg.OmitViaStringRelease))
 	args = append(args, "--no-outgoing-ip="+strconv.FormatBool(cfg.NoOutgoingIP))
 	args = append(args, "--disable-parent-config-comments="+strconv.FormatBool(cfg.DisableParentConfigComments))
 	args = append(args, "--use-strategies="+cfg.UseStrategies.String())
+	args = append(args, "--go-direct="+cfg.GoDirect)
 
-	generatedFiles, stdErr, code := t3cutil.DoInput(configData, config.GenerateCmd, args...)
+	generatedFiles, stdErr, code := t3cutil.DoInput(configData, t3cpath, args...)
 	if code != 0 {
-		logSubAppErr(`t3c-generate stdout`, generatedFiles)
-		logSubAppErr(`t3c-generate stderr`, stdErr)
-		return nil, fmt.Errorf("t3c-generate returned non-zero exit code %v, see log for output", code)
+		logSubAppErr(t3cgen+` stdout`, generatedFiles)
+		logSubAppErr(t3cgen+` stderr`, stdErr)
+		return nil, fmt.Errorf("%s returned non-zero exit code %v, see log for output", t3cgen, code)
 	}
-	logSubApp(`t3c-generate`, stdErr)
+	logSubApp(t3cgen, stdErr)
 
 	preprocessedBytes, err := preprocess(cfg, configData, generatedFiles)
 	if err != nil {
@@ -108,7 +129,7 @@ func generate(cfg config.Cfg) ([]t3cutil.ATSConfigFile, error) {
 
 // preprocess takes the to Data from 't3c-request --get-data=config' and the generated files from 't3c-generate', passes them to `t3c-preprocess`, and returns the result.
 func preprocess(cfg config.Cfg, configData []byte, generatedFiles []byte) ([]byte, error) {
-	args := []string{}
+	args := []string{`preprocess`}
 
 	if cfg.LogLocationErr == log.LogLocationNull {
 		args = append(args, "-s")
@@ -120,7 +141,7 @@ func preprocess(cfg config.Cfg, configData []byte, generatedFiles []byte) ([]byt
 		args = append(args, "-v")
 	}
 
-	cmd := exec.Command(`t3c-preprocess`, args...)
+	cmd := exec.Command(t3cpath, args...)
 	outbuf := bytes.Buffer{}
 	errbuf := bytes.Buffer{}
 	cmd.Stdout = &outbuf
@@ -161,11 +182,11 @@ func preprocess(cfg config.Cfg, configData []byte, generatedFiles []byte) ([]byt
 	stdOut := outbuf.Bytes()
 	stdErr := errbuf.Bytes()
 	if code != 0 {
-		logSubAppErr(`t3c-preprocess stdout`, stdOut)
-		logSubAppErr(`t3c-preprocess stderr`, stdErr)
-		return nil, fmt.Errorf("t3c-preprocess returned non-zero exit code %v, see log for output", code)
+		logSubAppErr(t3cpreproc+` stdout`, stdOut)
+		logSubAppErr(t3cpreproc+` stderr`, stdErr)
+		return nil, fmt.Errorf("%s returned non-zero exit code %v, see log for output", t3cpreproc, code)
 	}
-	logSubApp(`t3c-preprocess`, stdErr)
+	logSubApp(t3cpreproc, stdErr)
 	return stdOut, nil
 }
 
@@ -219,6 +240,7 @@ func getPackages(cfg config.Cfg) ([]Package, error) {
 // Note the statuses are the value to be set, not whether to set the value.
 func sendUpdate(cfg config.Cfg, configApplyTime, revalApplyTime *time.Time, configApplyBool, revalApplyBool *bool) error {
 	args := []string{
+		`update`,
 		"--traffic-ops-timeout-milliseconds=" + strconv.FormatInt(int64(cfg.TOTimeoutMS), 10),
 		"--traffic-ops-user=" + cfg.TOUser,
 		"--traffic-ops-password=" + cfg.TOPass,
@@ -263,14 +285,51 @@ func sendUpdate(cfg config.Cfg, configApplyTime, revalApplyTime *time.Time, conf
 	if _, used := os.LookupEnv("TO_URL"); !used {
 		args = append(args, "--traffic-ops-url="+cfg.TOURL)
 	}
-	stdOut, stdErr, code := t3cutil.Do(`t3c-update`, args...)
+	stdOut, stdErr, code := t3cutil.Do(t3cpath, args...)
 	if code != 0 {
-		logSubAppErr(`t3c-update stdout`, stdOut)
-		logSubAppErr(`t3c-update stderr`, stdErr)
-		return fmt.Errorf("t3c-update returned non-zero exit code %v, see log for output", code)
+		logSubAppErr(t3cupd+` stdout`, stdOut)
+		logSubAppErr(t3cupd+` stderr`, stdErr)
+		return fmt.Errorf("%s returned non-zero exit code %v, see log for output", t3cupd, code)
+	}
+	logSubApp(t3cupd, stdErr)
+	log.Infoln(t3cupd + " succeeded")
+	return nil
+}
+
+// doTail calls t3c-tail, which will read lines from the file at the provided
+// path, and will print lines matching the 'logMatch' regular expression.
+// When a line matching the 'endMatch' regular expression is encountered,
+// t3c-tail will exit - which means it MUST NOT be an empty string or only the
+// first line of the file will ever be read (and possibly printed, if it matches
+// 'logMatch'). In any case, the process will terminate after 'timeoutInMS'
+// milliseconds.
+// Note that apart from an exit code difference on timeout, this is almost
+// exactly equivalent to the bash command:
+//
+//	timeout timeoutInS tail -fn+2 file | grep -m 1 -B "$(wc -l file | cut -d ' ' -f1)" -E endMatch | grep -E logMatch
+//
+// ... where 'timeoutInS' is 1/1000 of 'timeoutInMS' and the string values of
+// arguments are otherwise substituted wherever they are found (GNU coreutils
+// are assumed to be present).
+func doTail(cfg config.Cfg, file string, logMatch string, endMatch string, timeoutInMS int) error {
+	args := []string{
+		"--file=" + filepath.Join(cfg.TsHome, file),
+		"--match=" + logMatch,
+		"--end-match=" + endMatch,
+		"--timeout-ms=" + strconv.Itoa(timeoutInMS),
+	}
+	stdOut, stdErr, code := t3cutil.Do(`t3c-tail`, args...)
+	if code >= 1 {
+		return fmt.Errorf("t3c-tail returned error code %d stdout '%s' stderr '%s'", code, stdOut, stdErr)
+	}
+	logSubApp(`t3c-tail`, stdErr)
+
+	stdOut = bytes.TrimSpace(stdOut)
+	lines := strings.Split(string(stdOut), "\n")
+	for _, line := range lines {
+		line = stripDate.ReplaceAllString(line, "")
+		log.Infoln(line)
 	}
-	logSubApp(`t3c-update`, stdErr)
-	log.Infoln("t3c-update succeeded")
 	return nil
 }
 
@@ -287,11 +346,14 @@ func diff(cfg config.Cfg, newFile []byte, fileLocation string, reportOnly bool,
 		"--file-gid=" + fmt.Sprint(gid),
 	}
 
-	stdOut, stdErr, code := t3cutil.DoInput(newFile, `t3c-diff`, args...)
+	// t3c-diff is called directly for performance reasons.
+	diffpath := t3cpath + `-diff`
+
+	stdOut, stdErr, code := t3cutil.DoInput(newFile, diffpath, args...)
 	if code > 1 {
-		return false, fmt.Errorf("t3c-diff returned error code %v stdout '%v' stderr '%v'", code, string(stdOut), string(stdErr))
+		return false, fmt.Errorf("%s returned error code %v stdout '%v' stderr '%v'", t3cdiff, code, string(stdOut), string(stdErr))
 	}
-	logSubApp(`t3c-diff`, stdErr)
+	logSubApp(t3cdiff, stdErr)
 
 	if code == 0 {
 		diffMsg += fmt.Sprintf("All lines and file permissions match TrOps for config file: %s\n", fileLocation)
@@ -330,9 +392,7 @@ func diff(cfg config.Cfg, newFile []byte, fileLocation string, reportOnly bool,
 // The cfgFile should be the full text of either a plugin.config or remap.config.
 // Returns nil if t3c-check-refs returned no errors found, or the error found if any.
 func checkRefs(cfg config.Cfg, cfgFile []byte, filesAdding []string) error {
-	args := []string{`check`, `refs`,
-		"--files-adding=" + strings.Join(filesAdding, ","),
-	}
+	args := []string{`check`, `refs`, `--files-adding=input`}
 	if cfg.LogLocationErr == log.LogLocationNull {
 		args = append(args, "-s")
 	}
@@ -343,24 +403,36 @@ func checkRefs(cfg config.Cfg, cfgFile []byte, filesAdding []string) error {
 		args = append(args, "-v")
 	}
 
-	stdOut, stdErr, code := t3cutil.DoInput(cfgFile, `t3c`, args...)
+	inputBts, err := json.Marshal(&t3cutil.CheckRefsInputFileAndAdding{File: cfgFile, Adding: filesAdding})
+	if err != nil {
+		return errors.New("marshalling json input: " + err.Error())
+	}
+
+	stdOut, stdErr, code := t3cutil.DoInput(inputBts, t3cpath, args...)
 
 	if code != 0 {
-		logSubAppErr(`t3c-check-refs stdout`, stdOut)
-		logSubAppErr(`t3c-check-refs stderr`, stdErr)
+		logSubAppErr(t3cchkrefs+` stdout`, stdOut)
+		logSubAppErr(t3cchkrefs+` stderr`, stdErr)
 		return fmt.Errorf("%d plugins failed to verify. See log for details.", code)
 	}
-	logSubApp(`t3c-check-refs stdout`, stdOut)
-	logSubApp(`t3c-check-refs stderr`, stdErr)
+	logSubApp(t3cchkrefs+` stdout`, stdOut)
+	logSubApp(t3cchkrefs+` stderr`, stdErr)
 	return nil
 }
 
-//checkCert checks the validity of the ssl certificate.
-func checkCert(c []byte) error {
+// checkCert checks the validity of the ssl certificate.
+func checkCert(c []byte) (error, bool) {
+	fatal := false
 	block, _ := pem.Decode(c)
+	if block == nil {
+		log.Errorln("Bad Certificate:\n'", string(c), "'")
+		fatal = true
+		return errors.New("Error Decoding Certificate"), fatal
+	}
 	cert, err := x509.ParseCertificate(block.Bytes)
 	if err != nil {
-		return errors.New("Error Parsing Certificate: " + err.Error())
+		fatal = true
+		return errors.New("Error Parsing Certificate: " + err.Error()), fatal
 	}
 	if cert.NotAfter.Unix() < time.Now().Unix() {
 		err = errors.New("Certificate expired: " + cert.NotAfter.Format(config.TimeAndDateLayout))
@@ -368,16 +440,17 @@ func checkCert(c []byte) error {
 	} else {
 		log.Infof("Certificate valid until %s ", cert.NotAfter.Format(config.TimeAndDateLayout))
 	}
-	return err
+	return err, fatal
 }
 
 // checkReload is a helper for the sub-command t3c-check-reload.
 func checkReload(changedConfigFiles []string) (t3cutil.ServiceNeeds, error) {
-	log.Infof("t3c-check-reload calling with changedConfigFiles '%v'\n", changedConfigFiles)
+	log.Infof("%s calling with changedConfigFiles '%v'\n", t3cchkreload, changedConfigFiles)
 
 	changedFiles := []byte(strings.Join(changedConfigFiles, ","))
 
-	cmd := exec.Command(`t3c-check-reload`)
+	args := []string{`check`, `reload`}
+	cmd := exec.Command(t3cpath, args...)
 	outBuf := bytes.Buffer{}
 	errBuf := bytes.Buffer{}
 	cmd.Stdout = &outBuf
@@ -415,16 +488,16 @@ func checkReload(changedConfigFiles []string) (t3cutil.ServiceNeeds, error) {
 	stdErr := errBuf.Bytes()
 
 	if code != 0 {
-		logSubAppErr(`t3c-check-reload stdout`, stdOut)
-		logSubAppErr(`t3c-check-reload stderr`, stdErr)
-		return t3cutil.ServiceNeedsInvalid, fmt.Errorf("t3c-check-reload returned error code %d - see log for details.", code)
+		logSubAppErr(t3cchkreload+` stdout`, stdOut)
+		logSubAppErr(t3cchkreload+` stderr`, stdErr)
+		return t3cutil.ServiceNeedsInvalid, fmt.Errorf("%s returned error code %d - see log for details.", t3cchkreload, code)
 	}
 
-	logSubApp(`t3c-check-reload`, stdErr)
+	logSubApp(t3cchkreload, stdErr)
 
 	needs := t3cutil.StrToServiceNeeds(strings.TrimSpace(string(stdOut)))
 	if needs == t3cutil.ServiceNeedsInvalid {
-		return t3cutil.ServiceNeedsInvalid, errors.New("t3c-check-reload returned unknown string '" + string(stdOut) + "'")
+		return t3cutil.ServiceNeedsInvalid, errors.New(t3cchkreload + " returned unknown string '" + string(stdOut) + "'")
 	}
 	return needs, nil
 }
@@ -444,6 +517,7 @@ func requestJSON(cfg config.Cfg, command string, obj interface{}) error {
 // request calls t3c-request with the given command, and returns the stdout bytes.
 func request(cfg config.Cfg, command string) ([]byte, error) {
 	args := []string{
+		`request`,
 		"--traffic-ops-insecure=" + strconv.FormatBool(cfg.TOInsecure),
 		"--traffic-ops-timeout-milliseconds=" + strconv.FormatInt(int64(cfg.TOTimeoutMS), 10),
 		"--cache-host-name=" + cfg.CacheHostName,
@@ -469,14 +543,14 @@ func request(cfg config.Cfg, command string) ([]byte, error) {
 	if _, used := os.LookupEnv("TO_URL"); !used {
 		args = append(args, "--traffic-ops-url="+cfg.TOURL)
 	}
-	stdOut, stdErr, code := t3cutil.Do(`t3c-request`, args...)
+	stdOut, stdErr, code := t3cutil.Do(t3cpath, args...)
 	if code != 0 {
-		logSubAppErr(`t3c-request stdout`, stdOut)
-		logSubAppErr(`t3c-request stderr`, stdErr)
-		return nil, fmt.Errorf("t3c-request returned non-zero exit code %v, see log for output", code)
+		logSubAppErr(t3creq+` stdout`, stdOut)
+		logSubAppErr(t3creq+` stderr`, stdErr)
+		return nil, fmt.Errorf("%s returned non-zero exit code %v, see log for output", t3creq, code)
 	}
 
-	logSubApp(`t3c-request`, stdErr)
+	logSubApp(t3creq, stdErr)
 
 	return stdOut, nil
 }
@@ -532,16 +606,16 @@ func requestConfig(cfg config.Cfg) ([]byte, error) {
 	stdErr := ([]byte)(nil)
 	code := 0
 	if len(cacheBts) > 0 {
-		stdOut, stdErr, code = t3cutil.DoInput(cacheBts, `t3c`, args...)
+		stdOut, stdErr, code = t3cutil.DoInput(cacheBts, t3cpath, args...)
 	} else {
-		stdOut, stdErr, code = t3cutil.Do(`t3c`, args...)
+		stdOut, stdErr, code = t3cutil.Do(t3cpath, args...)
 	}
 	if code != 0 {
-		logSubAppErr(`t3c-request stdout`, stdOut)
-		logSubAppErr(`t3c-request stderr`, stdErr)
-		return nil, fmt.Errorf("t3c-request returned non-zero exit code %v, see log for details", code)
+		logSubAppErr(t3creq+` stdout`, stdOut)
+		logSubAppErr(t3creq+` stderr`, stdErr)
+		return nil, fmt.Errorf("t3c returned non-zero exit code %v, see log for details", code)
 	}
-	logSubApp(`t3c-request`, stdErr)
+	logSubApp(t3creq, stdErr)
 
 	if err := ioutil.WriteFile(t3cutil.ApplyCachePath, stdOut, 0600); err != nil {
 		log.Errorln("writing config data to cache failed: " + err.Error())
