@@ -22,7 +22,6 @@ package torequest
 import (
 	"errors"
 	"fmt"
-	"io/ioutil"
 	"os"
 	"os/user"
 	"path/filepath"
@@ -32,10 +31,10 @@ import (
 	"syscall"
 	"time"
 
-	"github.com/apache/trafficcontrol/cache-config/t3c-apply/config"
-	"github.com/apache/trafficcontrol/cache-config/t3c-apply/util"
-	"github.com/apache/trafficcontrol/cache-config/t3cutil"
-	"github.com/apache/trafficcontrol/lib/go-log"
+	"github.com/apache/trafficcontrol/v8/cache-config/t3c-apply/config"
+	"github.com/apache/trafficcontrol/v8/cache-config/t3c-apply/util"
+	"github.com/apache/trafficcontrol/v8/cache-config/t3cutil"
+	"github.com/apache/trafficcontrol/v8/lib/go-log"
 )
 
 type UpdateStatus int
@@ -47,6 +46,15 @@ const (
 	UpdateTropsFailed     UpdateStatus = 3
 )
 
+const (
+	TailDiagsLogRelative = "/var/log/trafficserver/diags.log"
+	TailRestartTimeOutMS = 60000
+	TailReloadTimeOutMS  = 15000
+	tailMatch            = `ET_(TASK|NET)\s\d{1,}`
+	tailRestartEnd       = "Traffic Server is fully initialized"
+	tailReloadEnd        = "remap.config finished loading"
+)
+
 type Package struct {
 	Name    string `json:"name"`
 	Version string `json:"version"`
@@ -54,7 +62,7 @@ type Package struct {
 
 type TrafficOpsReq struct {
 	Cfg     config.Cfg
-	pkgs    map[string]bool // map of packages which are installed, either already installed or newly installed by this run.
+	Pkgs    map[string]bool // map of packages which are installed, either already installed or newly installed by this run.
 	plugins map[string]bool // map of verified plugins
 
 	installedPkgs map[string]struct{} // map of packages which were installed by us.
@@ -82,6 +90,8 @@ type RestartData struct {
 	TeakdRestart         bool // a restart of teakd is required
 	TrafficServerRestart bool // a trafficserver restart is required
 	RemapConfigReload    bool // remap.config should be reloaded
+	HitchReload          bool // hitch should be reloaded
+	VarnishReload        bool // varnish should be reloaded
 }
 
 type ConfigFile struct {
@@ -93,6 +103,7 @@ type ConfigFile struct {
 	TropsBackup       string // location to backup the TrafficOps Version
 	AuditComplete     bool   // audit is complete
 	AuditFailed       bool   // audit failed
+	AuditError        string // Error generated when AuditFailed is true
 	ChangeApplied     bool   // a change has been applied
 	ChangeNeeded      bool   // change required
 	PreReqFailed      bool   // failed plugin prerequiste check
@@ -183,7 +194,7 @@ func (r *TrafficOpsReq) DumpConfigFiles() {
 func NewTrafficOpsReq(cfg config.Cfg) *TrafficOpsReq {
 	return &TrafficOpsReq{
 		Cfg:           cfg,
-		pkgs:          map[string]bool{},
+		Pkgs:          map[string]bool{},
 		plugins:       map[string]bool{},
 		configFiles:   map[string]*ConfigFile{},
 		installedPkgs: map[string]struct{}{},
@@ -199,22 +210,33 @@ func (r *TrafficOpsReq) checkConfigFile(cfg *ConfigFile, filesAdding []string) e
 	}
 
 	if cfg.Dir == "" {
+		cfg.AuditFailed = true
 		return errors.New("No location information for " + cfg.Name)
 	}
 	// return if audit has already been done.
-	if cfg.AuditComplete == true {
+	if cfg.AuditComplete {
 		return nil
 	}
 
-	if !util.MkDirWithOwner(cfg.Dir, r.Cfg, &cfg.Uid, &cfg.Gid) {
+	if !util.MkDirWithOwner(cfg.Dir, r.Cfg.ReportOnly, &cfg.Uid, &cfg.Gid) {
+		cfg.AuditFailed = true
 		return errors.New("Unable to create the directory '" + cfg.Dir + " for " + "'" + cfg.Name + "'")
 	}
 
 	log.Debugf("======== Start processing config file: %s ========\n", cfg.Name)
 
+	if cfg.Name == "50-ats.rules" {
+		err := r.processUdevRules(cfg)
+		if err != nil {
+			cfg.AuditFailed = true
+			return errors.New("unable to process udev rules in '" + cfg.Name + "': " + err.Error())
+		}
+	}
+
 	if cfg.Name == "remap.config" {
 		err := r.processRemapOverrides(cfg)
 		if err != nil {
+			cfg.AuditFailed = true
 			return err
 		}
 	}
@@ -223,35 +245,32 @@ func (r *TrafficOpsReq) checkConfigFile(cfg *ConfigFile, filesAdding []string) e
 	if cfg.Name == "remap.config" || cfg.Name == "plugin.config" {
 		if err := checkRefs(r.Cfg, cfg.Body, filesAdding); err != nil {
 			r.configFileWarnings[cfg.Name] = append(r.configFileWarnings[cfg.Name], "failed to verify '"+cfg.Name+"': "+err.Error())
+			cfg.AuditFailed = true
 			return errors.New("failed to verify '" + cfg.Name + "': " + err.Error())
 		}
 		log.Infoln("Successfully verified plugins used by '" + cfg.Name + "'")
 	}
 
 	if strings.HasSuffix(cfg.Name, ".cer") {
-		if err := checkCert(cfg.Body); err != nil {
-			r.configFileWarnings[cfg.Name] = append(r.configFileWarnings[cfg.Name], fmt.Sprintln(err))
+		err, fatal := checkCert(cfg.Body)
+		if err != nil {
+			r.configFileWarnings[cfg.Name] = append(r.configFileWarnings[cfg.Name], err.Error())
 		}
-		for _, wrn := range cfg.Warnings {
-			r.configFileWarnings[cfg.Name] = append(r.configFileWarnings[cfg.Name], wrn)
+		r.configFileWarnings[cfg.Name] = append(r.configFileWarnings[cfg.Name], cfg.Warnings...)
+		if fatal {
+			return errors.New(err.Error() + " for: " + cfg.Name)
 		}
 	}
 
 	changeNeeded, err := diff(r.Cfg, cfg.Body, cfg.Path, r.Cfg.ReportOnly, cfg.Perm, cfg.Uid, cfg.Gid)
 
 	if err != nil {
+		cfg.AuditFailed = true
 		return errors.New("getting diff: " + err.Error())
 	}
 	cfg.ChangeNeeded = changeNeeded
 	cfg.AuditComplete = true
 
-	if cfg.Name == "50-ats.rules" {
-		err := r.processUdevRules(cfg)
-		if err != nil {
-			return errors.New("unable to process udev rules in '" + cfg.Name + "': " + err.Error())
-		}
-	}
-
 	log.Infof("======== End processing config file: %s for service: %s ========\n", cfg.Name, cfg.Service)
 	return nil
 }
@@ -290,7 +309,7 @@ func (r *TrafficOpsReq) checkStatusFiles(svrStatus string) error {
 	}
 
 	if !r.Cfg.ReportOnly {
-		if !util.MkDir(config.StatusDir, r.Cfg) {
+		if !util.MkDir(config.StatusDir, r.Cfg.ReportOnly) {
 			return fmt.Errorf("unable to create '%s'\n", config.StatusDir)
 		}
 		fileExists, _ := util.FileExists(statusFile)
@@ -407,7 +426,7 @@ func (r *TrafficOpsReq) processUdevRules(cfg *ConfigFile) error {
 			}
 		}
 	}
-	fs, err := ioutil.ReadDir("/proc/fs/ext4")
+	fs, err := os.ReadDir("/proc/fs/ext4")
 	if err != nil {
 		log.Errorln("unable to read /proc/fs/ext4, cannot audit disks for filesystem usage.")
 	} else {
@@ -503,6 +522,8 @@ func (r *TrafficOpsReq) replaceCfgFile(cfg *ConfigFile) (*FileRestartData, error
 	trafficServerRestart := cfg.Name == "plugin.config"
 	ntpdRestart := cfg.Name == "ntpd.conf"
 	sysCtlReload := cfg.Name == "sysctl.conf"
+	hitchReload := cfg.Name == "hitch.conf"
+	varnishReload := cfg.Name == "default.vcl"
 
 	log.Debugf("Reload state after %s: remap.config: %t reload: %t restart: %t ntpd: %t sysctl: %t", cfg.Name, remapConfigReload, trafficCtlReload, trafficServerRestart, ntpdRestart, sysCtlReload)
 
@@ -515,6 +536,8 @@ func (r *TrafficOpsReq) replaceCfgFile(cfg *ConfigFile) (*FileRestartData, error
 			NtpdRestart:          ntpdRestart,
 			TrafficServerRestart: trafficServerRestart,
 			RemapConfigReload:    remapConfigReload,
+			HitchReload:          hitchReload,
+			VarnishReload:        varnishReload,
 		},
 	}, nil
 }
@@ -574,27 +597,31 @@ func (r *TrafficOpsReq) CheckSystemServices() error {
 // IsPackageInstalled returns true/false if the named rpm package is installed.
 // the prefix before the version is matched.
 func (r *TrafficOpsReq) IsPackageInstalled(name string) bool {
-	for k, v := range r.pkgs {
+	for k, v := range r.Pkgs {
 		if strings.HasPrefix(k, name) {
+			log.Infof("Found in cache for '%s'", k)
 			return v
 		}
 	}
-
+	if !r.Cfg.RpmDBOk {
+		log.Warnf("RPM DB is corrupted cannot run IsPackageInstalled for '%s' and package metadata is unavailable", name)
+		return false
+	}
 	log.Infof("IsPackageInstalled '%v' not found in cache, querying rpm", name)
 	pkgArr, err := util.PackageInfo("pkg-query", name)
 	if err != nil {
 		log.Errorf(`IsPackageInstalled PackageInfo(pkg-query, %v) failed, caching as not installed and returning false! Error: %v\n`, name, err.Error())
-		r.pkgs[name] = false
+		r.Pkgs[name] = false
 		return false
 	}
 	if len(pkgArr) > 0 {
 		pkgAndVersion := pkgArr[0]
 		log.Infof("IsPackageInstalled '%v' found in rpm, adding '%v' to cache", name, pkgAndVersion)
-		r.pkgs[pkgAndVersion] = true
+		r.Pkgs[pkgAndVersion] = true
 		return true
 	}
 	log.Infof("IsPackageInstalled '%v' not found in rpm, adding '%v'=false to cache", name, name)
-	r.pkgs[name] = false
+	r.Pkgs[name] = false
 	return false
 }
 
@@ -725,8 +752,9 @@ func (r *TrafficOpsReq) CheckRevalidateState(sleepOverride bool) (UpdateStatus,
 	return updateStatus, nil
 }
 
-// CheckSYncDSState retrieves and returns the DS Update status from Traffic Ops.
-func (r *TrafficOpsReq) CheckSyncDSState() (UpdateStatus, error) {
+// CheckSyncDSState retrieves and returns the DS Update status from Traffic Ops.
+// The metaData is this run's metadata. It must not be nil, and this function may add to it.
+func (r *TrafficOpsReq) CheckSyncDSState(metaData *t3cutil.ApplyMetaData, cfg config.Cfg) (UpdateStatus, error) {
 	updateStatus := UpdateTropsNotNeeded
 	randDispSec := time.Duration(0)
 	log.Debugln("Checking syncds state.")
@@ -763,7 +791,7 @@ func (r *TrafficOpsReq) CheckSyncDSState() (UpdateStatus, error) {
 			}
 		} else if !r.Cfg.IgnoreUpdateFlag {
 			log.Errorln("no queued update needs to be applied.  Running revalidation before exiting.")
-			r.RevalidateWhileSleeping()
+			r.RevalidateWhileSleeping(metaData, cfg)
 			return UpdateTropsNotNeeded, nil
 		} else {
 			log.Errorln("Traffic Ops is signaling that no update is waiting to be applied.")
@@ -788,18 +816,21 @@ func (r *TrafficOpsReq) CheckReloadRestart(data []FileRestartData) RestartData {
 		rd.TeakdRestart = rd.TeakdRestart || changedFile.TeakdRestart
 		rd.TrafficServerRestart = rd.TrafficServerRestart || changedFile.TrafficServerRestart
 		rd.RemapConfigReload = rd.RemapConfigReload || changedFile.RemapConfigReload
+		rd.HitchReload = rd.HitchReload || changedFile.HitchReload
+		rd.VarnishReload = rd.VarnishReload || changedFile.VarnishReload
 	}
 	return rd
 }
 
 // ProcessConfigFiles processes all config files retrieved from Traffic Ops.
-func (r *TrafficOpsReq) ProcessConfigFiles() (UpdateStatus, error) {
+func (r *TrafficOpsReq) ProcessConfigFiles(metaData *t3cutil.ApplyMetaData) (UpdateStatus, error) {
 	var updateStatus UpdateStatus = UpdateTropsNotNeeded
+	var auditErrors []string
 
 	log.Infoln(" ======== Start processing config files ========")
 
 	filesAdding := []string{} // list of file names being added, needed for verification.
-	for fileName, _ := range r.configFiles {
+	for fileName := range r.configFiles {
 		filesAdding = append(filesAdding, fileName)
 	}
 
@@ -825,6 +856,7 @@ func (r *TrafficOpsReq) ProcessConfigFiles() (UpdateStatus, error) {
 		err := r.checkConfigFile(cfg, filesAdding)
 		if err != nil {
 			log.Errorln(err)
+			r.configFiles[cfg.Name].AuditError = err.Error()
 		}
 	}
 
@@ -832,6 +864,8 @@ func (r *TrafficOpsReq) ProcessConfigFiles() (UpdateStatus, error) {
 	shouldRestartReload := ShouldReloadRestart{[]FileRestartData{}}
 
 	for _, cfg := range r.configFiles {
+		metaData.OwnedFilePaths = append(metaData.OwnedFilePaths, cfg.Path) // all config files are added to OwnedFiles, even if they aren't changed on disk.
+
 		if cfg.ChangeNeeded &&
 			!cfg.ChangeApplied &&
 			cfg.AuditComplete &&
@@ -839,11 +873,11 @@ func (r *TrafficOpsReq) ProcessConfigFiles() (UpdateStatus, error) {
 			!cfg.AuditFailed {
 
 			changesRequired++
-			if cfg.Name == "plugin.config" && r.configFiles["remap.config"].PreReqFailed == true {
+			if cfg.Name == "plugin.config" && r.configFiles["remap.config"].PreReqFailed {
 				updateStatus = UpdateTropsFailed
 				log.Errorln("plugin.config changed however, prereqs failed for remap.config so I am skipping updates for plugin.config")
 				continue
-			} else if cfg.Name == "remap.config" && r.configFiles["plugin.config"].PreReqFailed == true {
+			} else if cfg.Name == "remap.config" && r.configFiles["plugin.config"].PreReqFailed {
 				updateStatus = UpdateTropsFailed
 				log.Errorln("remap.config changed however, prereqs failed for plugin.config so I am skipping updates for remap.config")
 				continue
@@ -858,9 +892,17 @@ func (r *TrafficOpsReq) ProcessConfigFiles() (UpdateStatus, error) {
 				}
 				shouldRestartReload.ReloadRestart = append(shouldRestartReload.ReloadRestart, *reData)
 			}
+		} else if cfg.AuditFailed {
+			auditErrors = append(auditErrors, cfg.AuditError)
+			log.Warnf("audit failed for config file: %v Error: %s", cfg.Name, cfg.AuditError)
+			updateStatus = UpdateTropsFailed
 		}
 	}
 
+	if updateStatus == UpdateTropsFailed {
+		return UpdateTropsFailed, errors.New(strings.Join(auditErrors, "\n"))
+	}
+
 	r.RestartData = r.CheckReloadRestart(shouldRestartReload.ReloadRestart)
 
 	if 0 < len(r.changedFiles) {
@@ -909,7 +951,7 @@ func (r *TrafficOpsReq) ProcessPackages() error {
 		if r.Cfg.InstallPackages {
 			if instpkg == fullPackage {
 				log.Infof("%s Currently installed and not marked for removal\n", reqpkg)
-				r.pkgs[fullPackage] = true
+				r.Pkgs[fullPackage] = true
 				continue
 			} else if instpkg != "" { // the installed package needs upgrading.
 				log.Infof("%s Currently installed and marked for removal\n", instpkg)
@@ -939,11 +981,11 @@ func (r *TrafficOpsReq) ProcessPackages() error {
 			// Only check if packages exist and complain if they are wrong.
 			if instpkg == fullPackage {
 				log.Infof("%s Currently installed.\n", reqpkg)
-				r.pkgs[fullPackage] = true
+				r.Pkgs[fullPackage] = true
 				continue
 			} else if instpkg != "" { // the installed package needs upgrading.
 				log.Errorf("%s Wrong version currently installed.\n", instpkg)
-				r.pkgs[instpkg] = true
+				r.Pkgs[instpkg] = true
 			} else {
 				// the required package needs installing.
 				log.Errorf("%s is Not installed.\n", fullPackage)
@@ -999,7 +1041,7 @@ func (r *TrafficOpsReq) ProcessPackages() error {
 					if err != nil {
 						return errors.New("Unable to install " + pkg + " : " + err.Error())
 					} else if result == true {
-						r.pkgs[pkg] = true
+						r.Pkgs[pkg] = true
 						r.installedPkgs[pkg] = struct{}{}
 						log.Infof("Package %s was installed\n", pkg)
 					}
@@ -1016,7 +1058,48 @@ func (r *TrafficOpsReq) ProcessPackages() error {
 	return nil
 }
 
-func (r *TrafficOpsReq) RevalidateWhileSleeping() (UpdateStatus, error) {
+func pkgMetaDataToMap(pmd []string) map[string]bool {
+	pkgMap := map[string]bool{}
+	for _, pkg := range pmd {
+		pkgMap[pkg] = true
+	}
+	return pkgMap
+}
+
+func pkgMatch(pkgMetaData []string, pk string) bool {
+	for _, pkg := range pkgMetaData {
+		if strings.Contains(pk, pkg) {
+			return true
+		}
+	}
+	return false
+
+}
+
+// ProcessPackagesWithMetaData will attempt to get installed package data from
+// t3c-apply-metadata.json and log the results.
+func (r *TrafficOpsReq) ProcessPackagesWithMetaData(packageMetaData []string) error {
+	pkgs, err := getPackages(r.Cfg)
+	pkgMdataMap := pkgMetaDataToMap(packageMetaData)
+	if err != nil {
+		return fmt.Errorf("getting packages: %w", err)
+	}
+	for _, pkg := range pkgs {
+		fullPackage := pkg.Name + "-" + pkg.Version
+		if pkgMdataMap[fullPackage] {
+			log.Infof("package %s is assumed to be installed according to metadata file", fullPackage)
+			r.Pkgs[fullPackage] = true
+		} else if pkgMatch(packageMetaData, pkg.Name) {
+			log.Infof("package %s is assumed to be installed according to metadata, but doesn't match traffic ops pkg", fullPackage)
+			r.Pkgs[fullPackage] = true
+		} else {
+			log.Infof("package %s does not appear to be installed.", pkg.Name+"-"+pkg.Version)
+		}
+	}
+	return nil
+}
+
+func (r *TrafficOpsReq) RevalidateWhileSleeping(metaData *t3cutil.ApplyMetaData, cfg config.Cfg) (UpdateStatus, error) {
 	updateStatus, err := r.CheckRevalidateState(true)
 	if err != nil {
 		return updateStatus, err
@@ -1032,12 +1115,15 @@ func (r *TrafficOpsReq) RevalidateWhileSleeping() (UpdateStatus, error) {
 			return updateStatus, err
 		}
 
-		updateStatus, err := r.ProcessConfigFiles()
+		updateStatus, err := r.ProcessConfigFiles(metaData)
 		if err != nil {
+			t3cutil.WriteActionLog(t3cutil.ActionLogActionUpdateFilesReval, t3cutil.ActionLogStatusFailure, metaData)
 			return updateStatus, err
+		} else {
+			t3cutil.WriteActionLog(t3cutil.ActionLogActionUpdateFilesReval, t3cutil.ActionLogStatusSuccess, metaData)
 		}
 
-		if err := r.StartServices(&updateStatus); err != nil {
+		if err := r.StartServices(&updateStatus, metaData, cfg); err != nil {
 			return updateStatus, errors.New("failed to start services: " + err.Error())
 		}
 
@@ -1054,7 +1140,7 @@ func (r *TrafficOpsReq) RevalidateWhileSleeping() (UpdateStatus, error) {
 // StartServices reloads, restarts, or starts ATS as necessary,
 // according to the changed config files and run mode.
 // Returns nil on success or any error.
-func (r *TrafficOpsReq) StartServices(syncdsUpdate *UpdateStatus) error {
+func (r *TrafficOpsReq) StartServices(syncdsUpdate *UpdateStatus, metaData *t3cutil.ApplyMetaData, cfg config.Cfg) error {
 	serviceNeeds := t3cutil.ServiceNeedsNothing
 	if r.Cfg.ServiceAction == t3cutil.ApplyServiceActionFlagRestart {
 		serviceNeeds = t3cutil.ServiceNeedsRestart
@@ -1071,18 +1157,22 @@ func (r *TrafficOpsReq) StartServices(syncdsUpdate *UpdateStatus) error {
 	// If check-reload does not know about these and we do, then we should initiate
 	// a reload as well
 	if serviceNeeds != t3cutil.ServiceNeedsRestart && serviceNeeds != t3cutil.ServiceNeedsReload {
-		if r.TrafficCtlReload || r.RemapConfigReload {
+		if r.TrafficCtlReload || r.RemapConfigReload || r.VarnishReload {
 			log.Infof("ATS config files unchanged, we updated files via t3c-apply, ATS needs reload")
 			serviceNeeds = t3cutil.ServiceNeedsReload
 		}
 	}
-
-	if (serviceNeeds == t3cutil.ServiceNeedsRestart || serviceNeeds == t3cutil.ServiceNeedsReload) && !r.IsPackageInstalled("trafficserver") {
-		// TODO try to reload/restart anyway? To allow non-RPM installs?
-		return errors.New("trafficserver needs " + serviceNeeds.String() + " but is not installed.")
+	packageName := "trafficserver"
+	if cfg.CacheType == "varnish" {
+		packageName = "varnish"
 	}
 
-	svcStatus, _, err := util.GetServiceStatus("trafficserver")
+	if (serviceNeeds == t3cutil.ServiceNeedsRestart || serviceNeeds == t3cutil.ServiceNeedsReload) && !r.IsPackageInstalled(packageName) {
+		// TODO try to reload/restart anyway? To allow non-RPM installs?
+		return errors.New(packageName + " needs " + serviceNeeds.String() + " but is not installed.")
+	}
+
+	svcStatus, _, err := util.GetServiceStatus(packageName)
 	if err != nil {
 		return errors.New("getting trafficserver service status: " + err.Error())
 	}
@@ -1099,10 +1189,21 @@ func (r *TrafficOpsReq) StartServices(syncdsUpdate *UpdateStatus) error {
 		if svcStatus != util.SvcRunning {
 			startStr = "start"
 		}
-		if _, err := util.ServiceStart("trafficserver", startStr); err != nil {
+		if _, err := util.ServiceStart(packageName, startStr); err != nil {
+			t3cutil.WriteActionLog(t3cutil.ActionLogActionATSRestart, t3cutil.ActionLogStatusFailure, metaData)
 			return errors.New("failed to restart trafficserver")
 		}
+		t3cutil.WriteActionLog(t3cutil.ActionLogActionATSRestart, t3cutil.ActionLogStatusSuccess, metaData)
 		log.Infoln("trafficserver has been " + startStr + "ed")
+
+		if !r.Cfg.NoConfirmServiceAction {
+			log.Infoln("confirming ATS restart succeeded")
+			if err := doTail(r.Cfg, TailDiagsLogRelative, ".*", tailRestartEnd, TailRestartTimeOutMS); err != nil {
+				log.Errorln("error running tail")
+			}
+		} else {
+			log.Infoln("skipping ATS restart success confirmation")
+		}
 		if *syncdsUpdate == UpdateTropsNeeded {
 			*syncdsUpdate = UpdateTropsSuccessful
 		}
@@ -1115,16 +1216,35 @@ func (r *TrafficOpsReq) StartServices(syncdsUpdate *UpdateStatus) error {
 			log.Errorln("ATS configuration has changed.  The new config will be picked up the next time ATS is started.")
 		} else if serviceNeeds == t3cutil.ServiceNeedsReload {
 			log.Infoln("ATS configuration has changed, Running 'traffic_ctl config reload' now.")
-			if _, _, err := util.ExecCommand(config.TSHome+config.TrafficCtl, "config", "reload"); err != nil {
+			reloadCommand := config.TSHome + config.TrafficCtl
+			reloadArgs := []string{"config", "reload"}
+			if cfg.CacheType == "varnish" {
+				reloadCommand = "/usr/sbin/varnishreload"
+				reloadArgs = []string{}
+			}
+			if _, _, err := util.ExecCommand(reloadCommand, reloadArgs...); err != nil {
+				t3cutil.WriteActionLog(t3cutil.ActionLogActionATSReload, t3cutil.ActionLogStatusFailure, metaData)
+
 				if *syncdsUpdate == UpdateTropsNeeded {
 					*syncdsUpdate = UpdateTropsFailed
 				}
 				return errors.New("ATS configuration has changed and 'traffic_ctl config reload' failed, check ATS logs: " + err.Error())
 			}
+			t3cutil.WriteActionLog(t3cutil.ActionLogActionATSReload, t3cutil.ActionLogStatusSuccess, metaData)
+
 			if *syncdsUpdate == UpdateTropsNeeded {
 				*syncdsUpdate = UpdateTropsSuccessful
 			}
 			log.Infoln("ATS 'traffic_ctl config reload' was successful")
+
+			if !r.Cfg.NoConfirmServiceAction {
+				log.Infoln("confirming ATS reload succeeded")
+				if err := doTail(r.Cfg, TailDiagsLogRelative, tailMatch, tailReloadEnd, TailReloadTimeOutMS); err != nil {
+					log.Errorln("error running tail: ", err)
+				}
+			} else {
+				log.Infoln("skipping ATS reload success confirmation")
+			}
 		}
 		if *syncdsUpdate == UpdateTropsNeeded {
 			*syncdsUpdate = UpdateTropsSuccessful
@@ -1134,6 +1254,12 @@ func (r *TrafficOpsReq) StartServices(syncdsUpdate *UpdateStatus) error {
 	return nil
 }
 
+func (r *TrafficOpsReq) ShowUpdateStatus(flagType []string, start time.Time, curSetting, newSetting bool) {
+	for _, flag := range flagType {
+		log.Infof("%s flag currently set to %v, setting to %v took %v", flag, curSetting, newSetting, time.Since(start).Round(time.Millisecond))
+	}
+}
+
 func (r *TrafficOpsReq) UpdateTrafficOps(syncdsUpdate *UpdateStatus) error {
 	var performUpdate bool
 
@@ -1166,17 +1292,25 @@ func (r *TrafficOpsReq) UpdateTrafficOps(syncdsUpdate *UpdateStatus) error {
 
 	// TODO: The boolean flags/representation can be removed after ATC (v7.0+)
 	if !r.Cfg.ReportOnly && !r.Cfg.NoUnsetUpdateFlag {
+		start := time.Now()
+		apply := []string{}
+		var b bool
 		if r.Cfg.Files == t3cutil.ApplyFilesFlagAll {
-			b := false
+			b = false
+			apply = append(apply, "update")
+			log.Infof("Update flag currently set to %v, setting to %v", serverStatus.UpdatePending, b)
 			err = sendUpdate(r.Cfg, serverStatus.ConfigUpdateTime, nil, &b, nil)
 		} else if r.Cfg.Files == t3cutil.ApplyFilesFlagReval {
-			b := false
+			b = false
+			apply = append(apply, t3cutil.ApplyFilesFlagReval.String())
+			log.Infof("Reval flag currently set to %v, setting to %v", serverStatus.RevalPending, b)
 			err = sendUpdate(r.Cfg, nil, serverStatus.RevalidateUpdateTime, nil, &b)
 		}
 		if err != nil {
 			return errors.New("Traffic Ops Update failed: " + err.Error())
 		}
 		log.Infoln("Traffic Ops has been updated.")
+		r.ShowUpdateStatus(apply, start, serverStatus.UpdatePending, b)
 	}
 	return nil
 }
