@@ -21,16 +21,15 @@ package tcdata
 
 import (
 	"encoding/json"
-	"io/ioutil"
 	"os"
 
-	"github.com/apache/trafficcontrol/lib/go-log"
+	"github.com/apache/trafficcontrol/v8/lib/go-log"
 )
 
 // LoadFixtures ...
 func (r *TCData) LoadFixtures(fixturesPath string) {
 
-	f, err := ioutil.ReadFile(fixturesPath)
+	f, err := os.ReadFile(fixturesPath)
 	if err != nil {
 		log.Errorf("Cannot unmarshal fixtures json %s", err)
 		os.Exit(1)
