@@ -12,15 +12,14 @@
 * limitations under the License.
 */
 import { Injectable } from "@angular/core";
-import type { CanActivate, CanLoad } from "@angular/router";
 
-import { CurrentUserService } from "src/app/shared/currentUser/current-user.service";
+import { CurrentUserService } from "src/app/shared/current-user/current-user.service";
 
 /**
  * AuthenticationGuard ensures that the user is logged in.
  */
 @Injectable()
-export class AuthenticatedGuard implements CanActivate, CanLoad {
+export class AuthenticatedGuard  {
 	constructor(private readonly auth: CurrentUserService) {
 	}
 
