@@ -13,11 +13,13 @@
 * limitations under the License.
 */
 
+import type { Environment } from "./environment.type";
+
 /**
  * environment contains information about the running environment.
  */
-export const environment = {
-	apiVersion: "3.0",
+export const environment: Environment = {
+	apiVersion: "4.0",
 	customModule: false,
-	production: true
+	production: true,
 };
