@@ -22,15 +22,6 @@ import {
 	platformBrowserDynamicTesting
 } from "@angular/platform-browser-dynamic/testing";
 
-declare const require: {
-	/** Not sure what this does */
-	context(path: string, deep?: boolean, filter?: RegExp): {
-		/** Idk what this is */
-		keys(): string[];
-		<T>(id: string): T;
-	};
-};
-
 // First, initialize the Angular testing environment.
 getTestBed().initTestEnvironment(
 	BrowserDynamicTestingModule,
@@ -38,7 +29,3 @@ getTestBed().initTestEnvironment(
 		teardown: { destroyAfterEach: false }
 	}
 );
-// Then we find all the tests.
-const context = require.context("./", true, /\.spec\.ts$/);
-// And load the modules.
-context.keys().map(context);
