@@ -17,10 +17,42 @@ package tc
 
 import (
 	"time"
+
+	"github.com/apache/trafficcontrol/v8/lib/go-util"
 )
 
+// DeliveryServiceServerV5 is the struct used to represent a delivery service server, in the latest minor version for
+// API 5.x.
+type DeliveryServiceServerV5 = DeliveryServiceServerV50
+
+// DeliveryServiceServerV50 is the type of each entry in the `response` array
+// property of responses from Traffic Ops to GET requests made to the
+// /deliveryserviceservers endpoint of its API, for api version 5.0.
+type DeliveryServiceServerV50 struct {
+	Server          *int       `json:"server" db:"server"`
+	DeliveryService *int       `json:"deliveryService" db:"deliveryservice"`
+	LastUpdated     *time.Time `json:"lastUpdated" db:"last_updated"`
+}
+
+// DeliveryServiceServerResponseV5 is the type of a response from Traffic Ops
+// to a GET request to the /deliveryserviceserver endpoint, for the latest minor version of api v5.x.
+type DeliveryServiceServerResponseV5 = DeliveryServiceServerResponseV50
+
+// DeliveryServiceServerResponseV50 is the type of a response from Traffic Ops
+// to a GET request to the /deliveryserviceserver endpoint for API v5.0.
+type DeliveryServiceServerResponseV50 struct {
+	Orderby  string                    `json:"orderby"`
+	Response []DeliveryServiceServerV5 `json:"response"`
+	Size     int                       `json:"size"`
+	Limit    int                       `json:"limit"`
+	Alerts
+}
+
 // DeliveryServiceServerResponse is the type of a response from Traffic Ops
 // to a GET request to the /deliveryserviceserver endpoint.
+//
+// Deprecated: Direct server-to-DS assignments are deprecated in favor of
+// Topology usage.
 type DeliveryServiceServerResponse struct {
 	Orderby  string                  `json:"orderby"`
 	Response []DeliveryServiceServer `json:"response"`
@@ -32,6 +64,9 @@ type DeliveryServiceServerResponse struct {
 // DSSMapResponse is the type of the `response` property of a response from
 // Traffic Ops to a PUT request made to the /deliveryserviceserver endpoint of
 // its API.
+//
+// Deprecated: Direct server-to-DS assignments are deprecated in favor of
+// Topology usage.
 type DSSMapResponse struct {
 	DsId    int   `json:"dsId"`
 	Replace bool  `json:"replace"`
@@ -40,6 +75,9 @@ type DSSMapResponse struct {
 
 // DSSReplaceResponse is the type of a response from Traffic Ops to a PUT
 // request made to the /deliveryserviceserver endpoint of its API.
+//
+// Deprecated: Direct server-to-DS assignments are deprecated in favor of
+// Topology usage.
 type DSSReplaceResponse struct {
 	Alerts
 	Response DSSMapResponse `json:"response"`
@@ -48,6 +86,9 @@ type DSSReplaceResponse struct {
 // DSServersResponse is the type of a response from Traffic Ops to a POST
 // request made to the /deliveryservices/{{XML ID}}/servers endpoint of its
 // API.
+//
+// Deprecated: Direct server-to-DS assignments are deprecated in favor of
+// Topology usage.
 type DSServersResponse struct {
 	Response DeliveryServiceServers `json:"response"`
 	Alerts
@@ -55,6 +96,9 @@ type DSServersResponse struct {
 
 // DeliveryServiceServers structures represent the servers assigned to a
 // Delivery Service.
+//
+// Deprecated: Direct server-to-DS assignments are deprecated in favor of
+// Topology usage.
 type DeliveryServiceServers struct {
 	ServerNames []string `json:"serverNames"`
 	XmlId       string   `json:"xmlId"`
@@ -63,6 +107,9 @@ type DeliveryServiceServers struct {
 // DeliveryServiceServer is the type of each entry in the `response` array
 // property of responses from Traffic Ops to GET requests made to the
 // /deliveryserviceservers endpoint of its API.
+//
+// Deprecated: Direct server-to-DS assignments are deprecated in favor of
+// Topology usage.
 type DeliveryServiceServer struct {
 	Server          *int       `json:"server" db:"server"`
 	DeliveryService *int       `json:"deliveryService" db:"deliveryservice"`
@@ -89,6 +136,9 @@ const (
 )
 
 // DSServerBase contains the base information for a Delivery Service Server.
+//
+// Deprecated: Direct server-to-DS assignments are deprecated in favor of
+// Topology usage.
 type DSServerBase struct {
 	Cachegroup                  *string              `json:"cachegroup" db:"cachegroup"`
 	CachegroupID                *int                 `json:"cachegroupId" db:"cachegroup_id"`
@@ -131,6 +181,9 @@ type DSServerBase struct {
 }
 
 // DSServerBaseV4 contains the base information for a Delivery Service Server.
+//
+// Deprecated: Direct server-to-DS assignments are deprecated in favor of
+// Topology usage.
 type DSServerBaseV4 struct {
 	Cachegroup                  *string              `json:"cachegroup" db:"cachegroup"`
 	CachegroupID                *int                 `json:"cachegroupId" db:"cachegroup_id"`
@@ -169,12 +222,18 @@ type DSServerBaseV4 struct {
 }
 
 // DSServerV11 contains the legacy format for a Delivery Service Server.
+//
+// Deprecated: Direct server-to-DS assignments are deprecated in favor of
+// Topology usage.
 type DSServerV11 struct {
 	DSServerBase
 	LegacyInterfaceDetails
 }
 
 // DSServer contains information for a Delivery Service Server.
+//
+// Deprecated: Direct server-to-DS assignments are deprecated in favor of
+// Topology usage.
 type DSServer struct {
 	DSServerBase
 	ServerInterfaces *[]ServerInterfaceInfo `json:"interfaces" db:"interfaces"`
@@ -182,12 +241,18 @@ type DSServer struct {
 
 // DSServerResponseV30 is the type of a response from Traffic Ops to a request
 // for servers assigned to a Delivery Service - in API version 3.0.
+//
+// Deprecated: Direct server-to-DS assignments are deprecated in favor of
+// Topology usage.
 type DSServerResponseV30 struct {
 	Response []DSServer `json:"response"`
 	Alerts
 }
 
 // DSServerV4 contains information for a V4.x Delivery Service Server.
+//
+// Deprecated: Direct server-to-DS assignments are deprecated in favor of
+// Topology usage.
 type DSServerV4 struct {
 	DSServerBaseV4
 	ServerInterfaces *[]ServerInterfaceInfoV40 `json:"interfaces" db:"interfaces"`
@@ -195,6 +260,9 @@ type DSServerV4 struct {
 
 // DSServerResponseV40 is the type of a response from Traffic Ops to a request
 // for servers assigned to a Delivery Service - in API version 4.0.
+//
+// Deprecated: Direct server-to-DS assignments are deprecated in favor of
+// Topology usage.
 type DSServerResponseV40 struct {
 	Response []DSServerV4 `json:"response"`
 	Alerts
@@ -203,8 +271,44 @@ type DSServerResponseV40 struct {
 // DSServerResponseV4 is the type of a response from Traffic Ops to a request
 // for servers assigned to a Delivery Service - in the latest minor version of
 // API version 4.
+//
+// Deprecated: Direct server-to-DS assignments are deprecated in favor of
+// Topology usage.
 type DSServerResponseV4 = DSServerResponseV40
 
+// DSServerV5 is an alias for the latest minor version of the major version 5.
+//
+// Deprecated: Direct server-to-DS assignments are deprecated in favor of
+// Topology usage.
+type DSServerV5 = DSServerV50
+
+// DSServerV50 contains information about a server associated with some Delivery
+// Service Server.
+//
+// Deprecated: Direct server-to-DS assignments are deprecated in favor of
+// Topology usage.
+type DSServerV50 struct {
+	ServerV5
+	DeliveryServices            map[string][]string `json:"deliveryServices,omitempty"`
+	ServerCapabilities          []string            `json:"-" db:"server_capabilities"`
+	DeliveryServiceCapabilities []string            `json:"-" db:"deliveryservice_capabilities"`
+}
+
+// DSServerResponseV5 is an alias for the latest minor version of the major version 5.
+//
+// Deprecated: Direct server-to-DS assignments are deprecated in favor of
+// Topology usage.
+type DSServerResponseV5 = DSServerResponseV50
+
+// DSServerResponseV50 is response from Traffic Ops to a request for servers assigned to a Delivery Service - in  the latest minor version APIv50.
+//
+// Deprecated: Direct server-to-DS assignments are deprecated in favor of
+// Topology usage.
+type DSServerResponseV50 struct {
+	Response []DSServerV50 `json:"response"`
+	Alerts
+}
+
 // ToDSServerBaseV4 upgrades the DSServerBase to the structure used by the
 // latest minor version of version 4 of Traffic Ops's API.
 func (oldBase DSServerBase) ToDSServerBaseV4() DSServerBaseV4 {
@@ -290,3 +394,54 @@ func (baseV4 DSServerBaseV4) ToDSServerBase(routerHostName, routerPort, pDesc *s
 	dsServerBase.RouterPortName = routerPort
 	return dsServerBase
 }
+
+// ToDSServerV5 convert DSServerV4 lastUpdated time format to RFC3339 for DSServerV5
+// and also assign V4 values to V5
+func (server DSServerV4) Upgrade() DSServerV5 {
+	var t time.Time
+	if server.LastUpdated != nil {
+		t = server.LastUpdated.Time
+	}
+
+	var dses map[string][]string
+	if server.DeliveryServices != nil {
+		dses = *server.DeliveryServices
+	}
+
+	return DSServerV5{
+		DeliveryServices: dses,
+		ServerV5: ServerV5{
+			CacheGroup:         util.CoalesceToDefault(server.Cachegroup),
+			CacheGroupID:       util.CoalesceToDefault(server.CachegroupID),
+			CDNID:              util.CoalesceToDefault(server.CDNID),
+			CDN:                util.CoalesceToDefault(server.CDNName),
+			DomainName:         util.CoalesceToDefault(server.DomainName),
+			GUID:               server.GUID,
+			HostName:           util.CoalesceToDefault(server.HostName),
+			HTTPSPort:          server.HTTPSPort,
+			ID:                 util.CoalesceToDefault(server.ID),
+			ILOIPAddress:       server.ILOIPAddress,
+			ILOIPGateway:       server.ILOIPGateway,
+			ILOIPNetmask:       server.ILOIPNetmask,
+			ILOPassword:        server.ILOPassword,
+			ILOUsername:        server.ILOUsername,
+			LastUpdated:        t,
+			MgmtIPAddress:      server.MgmtIPAddress,
+			MgmtIPGateway:      server.MgmtIPGateway,
+			MgmtIPNetmask:      server.MgmtIPNetmask,
+			OfflineReason:      server.OfflineReason,
+			Profiles:           server.ProfileNames,
+			PhysicalLocation:   util.CoalesceToDefault(server.PhysLocation),
+			PhysicalLocationID: util.CoalesceToDefault(server.PhysLocationID),
+			Rack:               server.Rack,
+			Status:             util.CoalesceToDefault(server.Status),
+			StatusID:           util.CoalesceToDefault(server.StatusID),
+			TCPPort:            server.TCPPort,
+			Type:               server.Type,
+			TypeID:             util.CoalesceToDefault(server.TypeID),
+			Interfaces:         *server.ServerInterfaces,
+		},
+		ServerCapabilities:          server.ServerCapabilities,
+		DeliveryServiceCapabilities: server.DeliveryServiceCapabilities,
+	}
+}
