@@ -26,55 +26,33 @@ import (
 	"fmt"
 	"time"
 
-	"github.com/apache/trafficcontrol/lib/go-rfc"
-	"github.com/apache/trafficcontrol/lib/go-util"
+	"github.com/apache/trafficcontrol/v8/lib/go-rfc"
+	"github.com/apache/trafficcontrol/v8/lib/go-util"
 
 	validation "github.com/go-ozzo/ozzo-validation"
 	"github.com/go-ozzo/ozzo-validation/is"
 )
 
-// copyStringIfNotNil makes a deep copy of s - unless it's nil, in which case it
-// just returns nil.
-func copyStringIfNotNil(s *string) *string {
-	if s == nil {
-		return nil
-	}
-	ret := new(string)
-	*ret = *s
-	return ret
-}
-
-// copyIntIfNotNil makes a deep copy of i - unless it's nil, in which case it
-// just returns nil.
-func copyIntIfNotNil(i *int) *int {
-	if i == nil {
-		return nil
-	}
-	ret := new(int)
-	*ret = *i
-	return ret
-}
-
 // Upgrade converts a User to a UserV4 (as seen in API versions 4.x).
 func (u User) Upgrade() UserV4 {
 	var ret UserV4
-	ret.AddressLine1 = copyStringIfNotNil(u.AddressLine1)
-	ret.AddressLine2 = copyStringIfNotNil(u.AddressLine2)
-	ret.City = copyStringIfNotNil(u.City)
-	ret.Company = copyStringIfNotNil(u.Company)
-	ret.Country = copyStringIfNotNil(u.Country)
-	ret.Email = copyStringIfNotNil(u.Email)
-	ret.GID = copyIntIfNotNil(u.GID)
-	ret.ID = copyIntIfNotNil(u.ID)
-	ret.LocalPassword = copyStringIfNotNil(u.LocalPassword)
-	ret.PhoneNumber = copyStringIfNotNil(u.PhoneNumber)
-	ret.PostalCode = copyStringIfNotNil(u.PostalCode)
-	ret.PublicSSHKey = copyStringIfNotNil(u.PublicSSHKey)
-	ret.StateOrProvince = copyStringIfNotNil(u.StateOrProvince)
-	ret.Tenant = copyStringIfNotNil(u.Tenant)
-	ret.Token = copyStringIfNotNil(u.Token)
-	ret.UID = copyIntIfNotNil(u.UID)
-	ret.FullName = copyStringIfNotNil(u.FullName)
+	ret.AddressLine1 = util.CopyIfNotNil(u.AddressLine1)
+	ret.AddressLine2 = util.CopyIfNotNil(u.AddressLine2)
+	ret.City = util.CopyIfNotNil(u.City)
+	ret.Company = util.CopyIfNotNil(u.Company)
+	ret.Country = util.CopyIfNotNil(u.Country)
+	ret.Email = util.CopyIfNotNil(u.Email)
+	ret.GID = util.CopyIfNotNil(u.GID)
+	ret.ID = util.CopyIfNotNil(u.ID)
+	ret.LocalPassword = util.CopyIfNotNil(u.LocalPassword)
+	ret.PhoneNumber = util.CopyIfNotNil(u.PhoneNumber)
+	ret.PostalCode = util.CopyIfNotNil(u.PostalCode)
+	ret.PublicSSHKey = util.CopyIfNotNil(u.PublicSSHKey)
+	ret.StateOrProvince = util.CopyIfNotNil(u.StateOrProvince)
+	ret.Tenant = util.CopyIfNotNil(u.Tenant)
+	ret.Token = util.CopyIfNotNil(u.Token)
+	ret.UID = util.CopyIfNotNil(u.UID)
+	ret.FullName = util.CopyIfNotNil(u.FullName)
 	if u.LastUpdated != nil {
 		ret.LastUpdated = u.LastUpdated.Time
 	}
@@ -113,26 +91,26 @@ func (u UserV4) Downgrade() User {
 	ret.Username = new(string)
 	*ret.Username = u.Username
 
-	ret.AddressLine1 = copyStringIfNotNil(u.AddressLine1)
-	ret.AddressLine2 = copyStringIfNotNil(u.AddressLine2)
-	ret.City = copyStringIfNotNil(u.City)
-	ret.Company = copyStringIfNotNil(u.Company)
-	ret.ConfirmLocalPassword = copyStringIfNotNil(u.LocalPassword)
-	ret.Country = copyStringIfNotNil(u.Country)
-	ret.Email = copyStringIfNotNil(u.Email)
-	ret.GID = copyIntIfNotNil(u.GID)
-	ret.ID = copyIntIfNotNil(u.ID)
-	ret.LocalPassword = copyStringIfNotNil(u.LocalPassword)
-	ret.PhoneNumber = copyStringIfNotNil(u.PhoneNumber)
-	ret.PostalCode = copyStringIfNotNil(u.PostalCode)
-	ret.PublicSSHKey = copyStringIfNotNil(u.PublicSSHKey)
+	ret.AddressLine1 = util.CopyIfNotNil(u.AddressLine1)
+	ret.AddressLine2 = util.CopyIfNotNil(u.AddressLine2)
+	ret.City = util.CopyIfNotNil(u.City)
+	ret.Company = util.CopyIfNotNil(u.Company)
+	ret.ConfirmLocalPassword = util.CopyIfNotNil(u.LocalPassword)
+	ret.Country = util.CopyIfNotNil(u.Country)
+	ret.Email = util.CopyIfNotNil(u.Email)
+	ret.GID = util.CopyIfNotNil(u.GID)
+	ret.ID = util.CopyIfNotNil(u.ID)
+	ret.LocalPassword = util.CopyIfNotNil(u.LocalPassword)
+	ret.PhoneNumber = util.CopyIfNotNil(u.PhoneNumber)
+	ret.PostalCode = util.CopyIfNotNil(u.PostalCode)
+	ret.PublicSSHKey = util.CopyIfNotNil(u.PublicSSHKey)
 	if u.RegistrationSent != nil {
 		ret.RegistrationSent = TimeNoModFromTime(*u.RegistrationSent)
 	}
-	ret.StateOrProvince = copyStringIfNotNil(u.StateOrProvince)
-	ret.Tenant = copyStringIfNotNil(u.Tenant)
-	ret.Token = copyStringIfNotNil(u.Token)
-	ret.UID = copyIntIfNotNil(u.UID)
+	ret.StateOrProvince = util.CopyIfNotNil(u.StateOrProvince)
+	ret.Tenant = util.CopyIfNotNil(u.Tenant)
+	ret.Token = util.CopyIfNotNil(u.Token)
+	ret.UID = util.CopyIfNotNil(u.UID)
 
 	return ret
 }
@@ -149,7 +127,7 @@ type UserToken struct {
 	Token string `json:"t"`
 }
 
-// commonUserFields is unexported, but its contents are still visible when it is embedded
+// commonUserFields contents are still visible when it is embedded
 // LastUpdated is a new field for some structs.
 type commonUserFields struct {
 	AddressLine1    *string    `json:"addressLine1" db:"address_line1"`
@@ -216,23 +194,23 @@ func (u UserV4) ToLegacyCurrentUser(roleID int, localUser bool) UserCurrent {
 	*ret.UserName = u.Username
 	ret.LocalUser = new(bool)
 	*ret.LocalUser = localUser
-	ret.Token = copyStringIfNotNil(u.Token)
-	ret.AddressLine1 = copyStringIfNotNil(u.AddressLine1)
-	ret.AddressLine2 = copyStringIfNotNil(u.AddressLine2)
-	ret.City = copyStringIfNotNil(u.City)
-	ret.Company = copyStringIfNotNil(u.Company)
-	ret.Country = copyStringIfNotNil(u.Country)
-	ret.Email = copyStringIfNotNil(u.Email)
-	ret.GID = copyIntIfNotNil(u.GID)
-	ret.ID = copyIntIfNotNil(u.ID)
-	ret.UID = copyIntIfNotNil(u.UID)
-	ret.PhoneNumber = copyStringIfNotNil(u.PhoneNumber)
-	ret.PostalCode = copyStringIfNotNil(u.PostalCode)
-	ret.PublicSSHKey = copyStringIfNotNil(u.PublicSSHKey)
-	ret.StateOrProvince = copyStringIfNotNil(u.StateOrProvince)
-	ret.Tenant = copyStringIfNotNil(u.Tenant)
-	ret.Token = copyStringIfNotNil(u.Token)
-	ret.UID = copyIntIfNotNil(u.UID)
+	ret.Token = util.CopyIfNotNil(u.Token)
+	ret.AddressLine1 = util.CopyIfNotNil(u.AddressLine1)
+	ret.AddressLine2 = util.CopyIfNotNil(u.AddressLine2)
+	ret.City = util.CopyIfNotNil(u.City)
+	ret.Company = util.CopyIfNotNil(u.Company)
+	ret.Country = util.CopyIfNotNil(u.Country)
+	ret.Email = util.CopyIfNotNil(u.Email)
+	ret.GID = util.CopyIfNotNil(u.GID)
+	ret.ID = util.CopyIfNotNil(u.ID)
+	ret.UID = util.CopyIfNotNil(u.UID)
+	ret.PhoneNumber = util.CopyIfNotNil(u.PhoneNumber)
+	ret.PostalCode = util.CopyIfNotNil(u.PostalCode)
+	ret.PublicSSHKey = util.CopyIfNotNil(u.PublicSSHKey)
+	ret.StateOrProvince = util.CopyIfNotNil(u.StateOrProvince)
+	ret.Tenant = util.CopyIfNotNil(u.Tenant)
+	ret.Token = util.CopyIfNotNil(u.Token)
+	ret.UID = util.CopyIfNotNil(u.UID)
 
 	return ret
 }
@@ -324,25 +302,25 @@ type CurrentUserUpdateRequestUser struct {
 // Fields not present in earlier API versions need to be passed explicitly
 func (u UserCurrent) Upgrade(registrationSent, lastAuthenticated *time.Time, ucdn string, changeLogCount int) UserV4 {
 	var ret UserV4
-	ret.AddressLine1 = copyStringIfNotNil(u.AddressLine1)
-	ret.AddressLine2 = copyStringIfNotNil(u.AddressLine2)
+	ret.AddressLine1 = util.CopyIfNotNil(u.AddressLine1)
+	ret.AddressLine2 = util.CopyIfNotNil(u.AddressLine2)
 	ret.ChangeLogCount = changeLogCount
-	ret.City = copyStringIfNotNil(u.City)
-	ret.Company = copyStringIfNotNil(u.Company)
-	ret.Country = copyStringIfNotNil(u.Country)
-	ret.Email = copyStringIfNotNil(u.Email)
-	ret.GID = copyIntIfNotNil(u.GID)
-	ret.ID = copyIntIfNotNil(u.ID)
+	ret.City = util.CopyIfNotNil(u.City)
+	ret.Company = util.CopyIfNotNil(u.Company)
+	ret.Country = util.CopyIfNotNil(u.Country)
+	ret.Email = util.CopyIfNotNil(u.Email)
+	ret.GID = util.CopyIfNotNil(u.GID)
+	ret.ID = util.CopyIfNotNil(u.ID)
 	ret.LastAuthenticated = lastAuthenticated
-	ret.PhoneNumber = copyStringIfNotNil(u.PhoneNumber)
-	ret.PostalCode = copyStringIfNotNil(u.PostalCode)
-	ret.PublicSSHKey = copyStringIfNotNil(u.PublicSSHKey)
+	ret.PhoneNumber = util.CopyIfNotNil(u.PhoneNumber)
+	ret.PostalCode = util.CopyIfNotNil(u.PostalCode)
+	ret.PublicSSHKey = util.CopyIfNotNil(u.PublicSSHKey)
 	ret.RegistrationSent = registrationSent
-	ret.StateOrProvince = copyStringIfNotNil(u.StateOrProvince)
-	ret.Tenant = copyStringIfNotNil(u.Tenant)
-	ret.Token = copyStringIfNotNil(u.Token)
+	ret.StateOrProvince = util.CopyIfNotNil(u.StateOrProvince)
+	ret.Tenant = util.CopyIfNotNil(u.Tenant)
+	ret.Token = util.CopyIfNotNil(u.Token)
 	ret.UCDN = ucdn
-	ret.UID = copyIntIfNotNil(u.UID)
+	ret.UID = util.CopyIfNotNil(u.UID)
 	ret.FullName = u.FullName
 	if u.LastUpdated != nil {
 		ret.LastUpdated = u.LastUpdated.Time
@@ -493,11 +471,6 @@ func (u *CurrentUserUpdateRequestUser) UnmarshalAndValidate(user *User) error {
 	return util.JoinErrs(errs)
 }
 
-// ------------------- Response structs -------------------- //
-//  Response structs should only be used in the client       //
-//  The client's use of these will eventually be deprecated  //
-// --------------------------------------------------------- //
-
 // UsersResponse can hold a Traffic Ops API response to a request to get a list of users.
 type UsersResponse struct {
 	Response []User `json:"response"`
@@ -585,7 +558,7 @@ type UserRegistrationRequestV40 struct {
 }
 
 // Validate implements the
-// github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/api.ParseValidator
+// github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/api.ParseValidator
 // interface.
 func (urr *UserRegistrationRequestV4) Validate(tx *sql.Tx) error {
 	var errs = []error{}
@@ -607,7 +580,7 @@ func (urr *UserRegistrationRequestV4) Validate(tx *sql.Tx) error {
 }
 
 // Validate implements the
-// github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/api.ParseValidator
+// github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/api.ParseValidator
 // interface.
 func (urr *UserRegistrationRequest) Validate(tx *sql.Tx) error {
 	var errs = []error{}
