@@ -21,6 +21,7 @@ package util
 
 import (
 	"runtime"
+	"time"
 )
 
 // Stacktrace is a helper function which returns the current stacktrace.
@@ -36,3 +37,19 @@ func Stacktrace() []byte {
 		buf = make([]byte, len(buf)*2)
 	}
 }
+
+// SliceToSet converts a slice to a map whose keys are the slice members, that is, a set.
+// Note duplicates will be lost, as is the nature of a set.
+func SliceToSet[T comparable](sl []T) map[T]struct{} {
+	st := map[T]struct{}{}
+	for _, val := range sl {
+		st[val] = struct{}{}
+	}
+	return st
+}
+
+// ConvertTimeFormat converts the input time to the supplied format.
+func ConvertTimeFormat(t time.Time, format string) (*time.Time, error) {
+	formattedTime, err := time.Parse(format, t.Format(format))
+	return &formattedTime, err
+}
