@@ -22,7 +22,7 @@ package datareq
 import (
 	"strconv"
 
-	"github.com/apache/trafficcontrol/traffic_monitor/threadsafe"
+	"github.com/apache/trafficcontrol/v8/traffic_monitor/threadsafe"
 )
 
 func srvAPIBandwidthCapacityKbps(statMaxKbpses threadsafe.CacheKbpses) []byte {
