@@ -24,13 +24,13 @@ import (
 	"testing"
 	"time"
 
-	"github.com/apache/trafficcontrol/traffic_monitor/cache"
-	"github.com/apache/trafficcontrol/traffic_monitor/config"
-	"github.com/apache/trafficcontrol/traffic_monitor/peer"
-	"github.com/apache/trafficcontrol/traffic_monitor/threadsafe"
-	"github.com/apache/trafficcontrol/traffic_monitor/todata"
+	"github.com/apache/trafficcontrol/v8/traffic_monitor/cache"
+	"github.com/apache/trafficcontrol/v8/traffic_monitor/config"
+	"github.com/apache/trafficcontrol/v8/traffic_monitor/peer"
+	"github.com/apache/trafficcontrol/v8/traffic_monitor/threadsafe"
+	"github.com/apache/trafficcontrol/v8/traffic_monitor/todata"
 
-	"github.com/apache/trafficcontrol/lib/go-tc"
+	"github.com/apache/trafficcontrol/v8/lib/go-tc"
 )
 
 // TestNoMonitoredInterfacesGetVitals assures that GetVitals
