@@ -62,12 +62,12 @@ INSERT INTO public.profile ("name", "description", "type", cdn) VALUES ('INFLUXD
 INSERT INTO public.profile ("name", "description", "type", cdn) VALUES ('RIAK_ALL', 'Riak profile for all CDNs', 'RIAK_PROFILE', (SELECT id FROM cdn WHERE "name"='ALL')) ON CONFLICT ("name") DO NOTHING;
 
 -- statuses
-INSERT INTO public.status ("name", "description") VALUES ('OFFLINE', 'Server is Offline. Not active in any configuration.') ON CONFLICT ("name") DO NOTHING;
 INSERT INTO public.status ("name", "description") VALUES ('ONLINE', 'Server is online.') ON CONFLICT ("name") DO NOTHING;
+INSERT INTO public.status ("name", "description") VALUES ('OFFLINE', 'Server is Offline. Not active in any configuration.') ON CONFLICT ("name") DO NOTHING;
 INSERT INTO public.status ("name", "description") VALUES ('REPORTED', 'Server is online and reported in the health protocol.') ON CONFLICT ("name") DO NOTHING;
 INSERT INTO public.status ("name", "description") VALUES ('ADMIN_DOWN', 'Sever is administrative down and does not receive traffic.') ON CONFLICT ("name") DO NOTHING;
-INSERT INTO public.status ("name", "description") VALUES ('CCR_IGNORE', 'Server is ignored by traffic router.') ON CONFLICT ("name") DO NOTHING;
 INSERT INTO public.status ("name", "description") VALUES ('PRE_PROD', 'Pre Production. Not active in any configuration.') ON CONFLICT ("name") DO NOTHING;
+INSERT INTO public.status ("name", "description") VALUES ('CCR_IGNORE', 'Server is ignored by traffic router.') ON CONFLICT ("name") DO NOTHING;
 
 -- tenants
 INSERT INTO public.tenant ("name", active, parent_id) VALUES ('root', true, NULL) ON CONFLICT DO NOTHING;
@@ -98,18 +98,22 @@ WHERE "name" in ('operations', 'read-only', 'portal', 'federation', 'steering')
 ON CONFLICT DO NOTHING;
 
 -- Using role 'read-only'
-INSERT INTO public.role_capability
+INSERT INTO public.role_capability (role_id, cap_name)
 SELECT id, perm
 FROM public.role
 CROSS JOIN ( VALUES
+	('ACME:READ'),
 	('ASN:READ'),
 	('ASYNC-STATUS:READ'),
 	('CACHE-GROUP:READ'),
 	('CAPABILITY:READ'),
 	('CDN-SNAPSHOT:READ'),
 	('CDN:READ'),
+	('CDNI-ADMIN:READ'),
+	('CDNI-CAPACITY:READ'),
 	('COORDINATE:READ'),
 	('DELIVERY-SERVICE:READ'),
+	('DELIVERY-SERVICE-SAFE:UPDATE'),
 	('DIVISION:READ'),
 	('DS-REQUEST:READ'),
 	('DS-SECURITY-KEY:READ'),
@@ -129,6 +133,7 @@ CROSS JOIN ( VALUES
 	('SERVER-CAPABILITY:READ'),
 	('SERVER:READ'),
 	('SERVICE-CATEGORY:READ'),
+	('SSL-KEY-EXPIRATION:READ'),
 	('STATIC-DN:READ'),
 	('STATUS:READ'),
 	('SERVER-CHECK:READ'),
@@ -182,6 +187,10 @@ INSERT INTO public.role_capability
 SELECT id, perm
 FROM public.role
 CROSS JOIN ( VALUES
+	('ACME:CREATE'),
+	('ACME:DELETE'),
+	('ACME:READ'),
+	('ACME:UPDATE'),
 	('ASN:CREATE'),
 	('ASN:DELETE'),
 	('ASN:UPDATE'),
@@ -203,7 +212,9 @@ CROSS JOIN ( VALUES
 	('DIVISION:CREATE'),
 	('DIVISION:DELETE'),
 	('DIVISION:UPDATE'),
+	('DNS-SEC:READ'),
 	('DNS-SEC:UPDATE'),
+	('DNS-SEC:DELETE'),
 	('ISO:GENERATE'),
 	('ORIGIN:CREATE'),
 	('ORIGIN:DELETE'),
@@ -220,6 +231,7 @@ CROSS JOIN ( VALUES
 	('REGION:CREATE'),
 	('REGION:DELETE'),
 	('REGION:UPDATE'),
+	('SECURE-SERVER:READ'),
 	('SERVER-CAPABILITY:CREATE'),
 	('SERVER-CAPABILITY:DELETE'),
 	('SERVER-CAPABILITY:UPDATE'),
