@@ -24,11 +24,11 @@ import (
 	"testing"
 	"time"
 
-	"github.com/apache/trafficcontrol/lib/go-rfc"
-	"github.com/apache/trafficcontrol/lib/go-tc"
-	"github.com/apache/trafficcontrol/traffic_ops/testing/api/assert"
-	"github.com/apache/trafficcontrol/traffic_ops/testing/api/utils"
-	"github.com/apache/trafficcontrol/traffic_ops/toclientlib"
+	"github.com/apache/trafficcontrol/v8/lib/go-rfc"
+	"github.com/apache/trafficcontrol/v8/lib/go-tc"
+	"github.com/apache/trafficcontrol/v8/lib/go-util/assert"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/testing/api/utils"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/toclientlib"
 )
 
 func TestServers(t *testing.T) {
@@ -82,7 +82,7 @@ func TestServers(t *testing.T) {
 				},
 				"OK when VALID PROFILEID parameter": {
 					ClientSession: TOSession,
-					RequestParams: url.Values{"profileId": {strconv.Itoa(GetProfileId(t, "EDGE1"))}},
+					RequestParams: url.Values{"profileId": {strconv.Itoa(GetProfileID(t, "EDGE1")())}},
 					Expectations:  utils.CkRequest(utils.NoError(), utils.HasStatus(http.StatusOK), utils.ResponseLengthGreaterOrEqual(1)),
 				},
 				"OK when VALID STATUS parameter": {
@@ -126,7 +126,7 @@ func TestServers(t *testing.T) {
 			},
 			"PUT": {
 				"OK when VALID request": {
-					EndpointId:    GetServerID(t, "atlanta-edge-03"),
+					EndpointID:    GetServerID(t, "atlanta-edge-03"),
 					ClientSession: TOSession,
 					RequestBody: map[string]interface{}{
 						"id":           GetServerID(t, "atlanta-edge-03")(),
@@ -155,7 +155,7 @@ func TestServers(t *testing.T) {
 							"routerPort":     "9004",
 						}},
 						"physLocationId": GetPhysicalLocationID(t, "Denver")(),
-						"profileId":      GetProfileId(t, "EDGE1"),
+						"profileId":      GetProfileID(t, "EDGE1")(),
 						"rack":           "RR 119.03",
 						"statusId":       GetStatusID(t, "REPORTED")(),
 						"tcpPort":        8080,
@@ -170,7 +170,7 @@ func TestServers(t *testing.T) {
 						})),
 				},
 				"BAD REQUEST when CHANGING XMPPID": {
-					EndpointId:    GetServerID(t, "atlanta-edge-16"),
+					EndpointID:    GetServerID(t, "atlanta-edge-16"),
 					ClientSession: TOSession,
 					RequestBody: generateServer(t, map[string]interface{}{
 						"id":     GetServerID(t, "atlanta-edge-16")(),
@@ -179,7 +179,7 @@ func TestServers(t *testing.T) {
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusBadRequest)),
 				},
 				"CONFLICT when UPDATING SERVER TYPE when ASSIGNED to DS": {
-					EndpointId:    GetServerID(t, "test-ds-server-assignments"),
+					EndpointID:    GetServerID(t, "test-ds-server-assignments"),
 					ClientSession: TOSession,
 					RequestBody: generateServer(t, map[string]interface{}{
 						"id":           GetServerID(t, "test-ds-server-assignments")(),
@@ -189,7 +189,7 @@ func TestServers(t *testing.T) {
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusConflict)),
 				},
 				"CONFLICT when UPDATING SERVER STATUS when its the ONLY EDGE SERVER ASSIGNED": {
-					EndpointId:    GetServerID(t, "test-ds-server-assignments"),
+					EndpointID:    GetServerID(t, "test-ds-server-assignments"),
 					ClientSession: TOSession,
 					RequestBody: generateServer(t, map[string]interface{}{
 						"id":       GetServerID(t, "test-ds-server-assignments")(),
@@ -198,30 +198,30 @@ func TestServers(t *testing.T) {
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusConflict)),
 				},
 				"BAD REQUEST when UPDATING CDN when LAST SERVER IN CACHEGROUP IN TOPOLOGY": {
-					EndpointId:    GetServerID(t, "midInTopologyMidCg01"),
+					EndpointID:    GetServerID(t, "midInTopologyMidCg01"),
 					ClientSession: TOSession,
 					RequestBody: generateServer(t, map[string]interface{}{
 						"id":           GetServerID(t, "midInTopologyMidCg01")(),
 						"cdnId":        GetCDNID(t, "cdn1")(),
-						"profileId":    GetProfileId(t, "MID1"),
+						"profileId":    GetProfileID(t, "MID1")(),
 						"cachegroupId": GetCacheGroupId(t, "topology-mid-cg-01")(),
 					}),
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusBadRequest)),
 				},
 				"BAD REQUEST when UPDATING CACHEGROUP when LAST SERVER IN CACHEGROUP IN TOPOLOGY": {
-					EndpointId:    GetServerID(t, "midInTopologyMidCg01"),
+					EndpointID:    GetServerID(t, "midInTopologyMidCg01"),
 					ClientSession: TOSession,
 					RequestBody: generateServer(t, map[string]interface{}{
 						"id":           GetServerID(t, "midInTopologyMidCg01")(),
 						"hostName":     "midInTopologyMidCg01",
 						"cdnId":        GetCDNID(t, "cdn2")(),
-						"profileId":    GetProfileId(t, "CDN2_MID"),
+						"profileId":    GetProfileID(t, "CDN2_MID")(),
 						"cachegroupId": GetCacheGroupId(t, "topology-mid-cg-02")(),
 					}),
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusBadRequest)),
 				},
 				"BAD REQUEST when IPADDRESS EXISTS with SAME PROFILE": {
-					EndpointId:    GetServerID(t, "atlanta-edge-16"),
+					EndpointID:    GetServerID(t, "atlanta-edge-16"),
 					ClientSession: TOSession,
 					RequestBody: generateServer(t, map[string]interface{}{
 						"profileNames": []string{"EDGE1"},
@@ -237,19 +237,19 @@ func TestServers(t *testing.T) {
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusBadRequest)),
 				},
 				"BAD REQUEST when BLANK HOSTNAME": {
-					EndpointId:    GetServerID(t, "atlanta-edge-16"),
+					EndpointID:    GetServerID(t, "atlanta-edge-16"),
 					ClientSession: TOSession,
 					RequestBody:   generateServer(t, map[string]interface{}{"hostName": ""}),
 					Expectations:  utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusBadRequest)),
 				},
 				"BAD REQUEST when BLANK DOMAINNAME": {
-					EndpointId:    GetServerID(t, "atlanta-edge-16"),
+					EndpointID:    GetServerID(t, "atlanta-edge-16"),
 					ClientSession: TOSession,
 					RequestBody:   generateServer(t, map[string]interface{}{"domainName": ""}),
 					Expectations:  utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusBadRequest)),
 				},
 				"PRECONDITION FAILED when updating with IMS & IUS Headers": {
-					EndpointId:     GetServerID(t, "atlanta-edge-01"),
+					EndpointID:     GetServerID(t, "atlanta-edge-01"),
 					ClientSession:  TOSession,
 					RequestHeaders: http.Header{rfc.IfUnmodifiedSince: {currentTimeRFC}},
 					RequestBody: generateServer(t, map[string]interface{}{
@@ -258,7 +258,7 @@ func TestServers(t *testing.T) {
 					Expectations: utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusPreconditionFailed)),
 				},
 				"PRECONDITION FAILED when updating with IFMATCH ETAG Header": {
-					EndpointId:    GetServerID(t, "atlanta-edge-01"),
+					EndpointID:    GetServerID(t, "atlanta-edge-01"),
 					ClientSession: TOSession,
 					RequestBody: generateServer(t, map[string]interface{}{
 						"id": GetServerID(t, "atlanta-edge-01")(),
@@ -269,12 +269,12 @@ func TestServers(t *testing.T) {
 			},
 			"DELETE": {
 				"BAD REQUEST when LAST SERVER in CACHE GROUP": {
-					EndpointId:    GetServerID(t, "midInTopologyMidCg01"),
+					EndpointID:    GetServerID(t, "midInTopologyMidCg01"),
 					ClientSession: TOSession,
 					Expectations:  utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusBadRequest)),
 				},
 				"CONFLICT when DELETING SERVER when its the ONLY EDGE SERVER ASSIGNED": {
-					EndpointId:    GetServerID(t, "test-ds-server-assignments"),
+					EndpointID:    GetServerID(t, "test-ds-server-assignments"),
 					ClientSession: TOSession,
 					Expectations:  utils.CkRequest(utils.HasError(), utils.HasStatus(http.StatusConflict)),
 				},
@@ -317,14 +317,14 @@ func TestServers(t *testing.T) {
 						})
 					case "PUT":
 						t.Run(name, func(t *testing.T) {
-							alerts, reqInf, err := testCase.ClientSession.UpdateServerByIDWithHdr(testCase.EndpointId(), server, testCase.RequestHeaders)
+							alerts, reqInf, err := testCase.ClientSession.UpdateServerByIDWithHdr(testCase.EndpointID(), server, testCase.RequestHeaders)
 							for _, check := range testCase.Expectations {
 								check(t, reqInf, nil, alerts, err)
 							}
 						})
 					case "DELETE":
 						t.Run(name, func(t *testing.T) {
-							alerts, reqInf, err := testCase.ClientSession.DeleteServerByID(testCase.EndpointId())
+							alerts, reqInf, err := testCase.ClientSession.DeleteServerByID(testCase.EndpointID())
 							for _, check := range testCase.Expectations {
 								check(t, reqInf, nil, alerts, err)
 							}
@@ -458,7 +458,7 @@ func generateServer(t *testing.T, requestServer map[string]interface{}) map[stri
 			"name": "eth0",
 		}},
 		"physLocationId": GetPhysicalLocationID(t, "Denver")(),
-		"profileId":      GetProfileId(t, "EDGE1"),
+		"profileId":      GetProfileID(t, "EDGE1")(),
 		"statusId":       GetStatusID(t, "REPORTED")(),
 		"typeId":         GetTypeId(t, "EDGE"),
 	}
