@@ -18,11 +18,11 @@ import (
 	"testing"
 	"time"
 
-	"github.com/apache/trafficcontrol/lib/go-tc"
-	"github.com/apache/trafficcontrol/lib/go-util"
-	"github.com/apache/trafficcontrol/traffic_ops/testing/api/assert"
-	"github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/deliveryservice"
-	client "github.com/apache/trafficcontrol/traffic_ops/v4-client"
+	"github.com/apache/trafficcontrol/v8/lib/go-tc"
+	"github.com/apache/trafficcontrol/v8/lib/go-util"
+	"github.com/apache/trafficcontrol/v8/lib/go-util/assert"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/deliveryservice"
+	client "github.com/apache/trafficcontrol/v8/traffic_ops/v4-client"
 )
 
 func TestDeliveryServicesKeys(t *testing.T) {
@@ -96,27 +96,27 @@ func cleanUp(t *testing.T, ds tc.DeliveryServiceV4, oldCDNID int, newCDNID int,
 
 // getCustomDS returns a DS that is guaranteed to have non-nil:
 //
-//    Active
-//    CDNID
-//    DSCP
-//    DisplayName
-//    RoutingName
-//    GeoLimit
-//    GeoProvider
-//    IPV6RoutingEnabled
-//    InitialDispersion
-//    LogsEnabled
-//    MissLat
-//    MissLong
-//    MultiSiteOrigin
-//    OrgServerFQDN
-//    Protocol
-//    QStringIgnore
-//    RangeRequestHandling
-//    RegionalGeoBlocking
-//    TenantID
-//    TypeID
-//    XMLID
+//	Active
+//	CDNID
+//	DSCP
+//	DisplayName
+//	RoutingName
+//	GeoLimit
+//	GeoProvider
+//	IPV6RoutingEnabled
+//	InitialDispersion
+//	LogsEnabled
+//	MissLat
+//	MissLong
+//	MultiSiteOrigin
+//	OrgServerFQDN
+//	Protocol
+//	QStringIgnore
+//	RangeRequestHandling
+//	RegionalGeoBlocking
+//	TenantID
+//	TypeID
+//	XMLID
 //
 // BUT, will ALWAYS have nil MaxRequestHeaderBytes.
 // Note that the Tenant is hard-coded to #1.
