@@ -19,18 +19,21 @@ import (
 	"net/http"
 	"testing"
 
-	"github.com/apache/trafficcontrol/traffic_ops/testing/api/utils"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/testing/api/utils"
+	client "github.com/apache/trafficcontrol/v8/traffic_ops/v4-client"
 )
 
 func TestPing(t *testing.T) {
 
-	methodTests := utils.V4TestCase{
+	methodTests := utils.TestCase[client.Session, client.RequestOptions, struct{}]{
 		"GET": {
 			"OK when VALID request": {
-				ClientSession: TOSession, Expectations: utils.CkRequest(utils.NoError(), utils.HasStatus(http.StatusOK)),
+				ClientSession: TOSession,
+				Expectations:  utils.CkRequest(utils.NoError(), utils.HasStatus(http.StatusOK)),
 			},
 			"OK when UNAUTHENTICATED": {
-				ClientSession: NoAuthTOSession, Expectations: utils.CkRequest(utils.NoError(), utils.HasStatus(http.StatusOK)),
+				ClientSession: NoAuthTOSession,
+				Expectations:  utils.CkRequest(utils.NoError(), utils.HasStatus(http.StatusOK)),
 			},
 		},
 	}
