@@ -38,15 +38,15 @@ import (
 	"strings"
 	"time"
 
-	"github.com/apache/trafficcontrol/lib/go-log"
-	"github.com/apache/trafficcontrol/lib/go-rfc"
-	"github.com/apache/trafficcontrol/lib/go-tc"
-	"github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/auth"
-	"github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/config"
-	"github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/tenant"
-	"github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/tocookie"
-	"github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/trafficvault"
-	"github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/trafficvault/backends/disabled"
+	"github.com/apache/trafficcontrol/v8/lib/go-log"
+	"github.com/apache/trafficcontrol/v8/lib/go-rfc"
+	"github.com/apache/trafficcontrol/v8/lib/go-tc"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/auth"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/config"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/tenant"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/tocookie"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/trafficvault"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/trafficvault/backends/disabled"
 
 	influx "github.com/influxdata/influxdb/client/v2"
 	"github.com/jmoiron/sqlx"
@@ -83,10 +83,7 @@ const (
 	TrafficVaultContextKey = "tv"
 )
 
-const (
-	MojoCookie  = "mojoCookie"
-	AccessToken = "access_token"
-)
+const MojoCookie = "mojoCookie"
 
 const influxServersQuery = `
 SELECT (host_name||'.'||domain_name) as fqdn,
@@ -218,6 +215,9 @@ func HandleErr(w http.ResponseWriter, r *http.Request, tx *sql.Tx, statusCode in
 			log.Errorln("rolling back transaction: " + err.Error())
 		}
 	}
+
+	w.Header().Set(rfc.ContentType, rfc.ApplicationJSON)
+	w.WriteHeader(statusCode)
 	handleSimpleErr(w, r, statusCode, userErr, sysErr)
 }
 
@@ -518,6 +518,7 @@ type APIInfo struct {
 	Vault     trafficvault.TrafficVault
 	Config    *config.Config
 	request   *http.Request
+	w         http.ResponseWriter
 }
 
 // NewInfo get and returns the context info needed by handlers. It also returns any user error, any system error, and the status code which should be returned to the client if an error occurred.
@@ -529,26 +530,26 @@ type APIInfo struct {
 // Close() must be called to free resources, and should be called in a defer immediately after NewInfo(), to finish the transaction.
 //
 // Example:
-//  func handler(w http.ResponseWriter, r *http.Request) {
-//    inf, userErr, sysErr, errCode := api.NewInfo(r, nil, nil)
-//    if userErr != nil || sysErr != nil {
-//      api.HandleErr(w, r, inf.Tx.Tx, errCode, userErr, sysErr)
-//      return
-//    }
-//    defer inf.Close()
 //
-//    respObj, err := finalDatabaseOperation(inf.Tx)
-//    if err != nil {
-//      api.HandleErr(w, r, inf.Tx.Tx, http.StatusInternalServerError, nil, errors.New("final db op: " + err.Error()))
-//      return
-//    }
-//    if err := inf.Tx.Tx.Commit(); err != nil {
-//      api.HandleErr(w, r, inf.Tx.Tx, http.StatusInternalServerError, nil, errors.New("committing transaction: " + err.Error()))
-//      return
-//    }
-//    api.WriteResp(w, r, respObj)
-//  }
+//	func handler(w http.ResponseWriter, r *http.Request) {
+//	  inf, userErr, sysErr, errCode := api.NewInfo(r, nil, nil)
+//	  if userErr != nil || sysErr != nil {
+//	    api.HandleErr(w, r, inf.Tx.Tx, errCode, userErr, sysErr)
+//	    return
+//	  }
+//	  defer inf.Close()
 //
+//	  respObj, err := finalDatabaseOperation(inf.Tx)
+//	  if err != nil {
+//	    api.HandleErr(w, r, inf.Tx.Tx, http.StatusInternalServerError, nil, errors.New("final db op: " + err.Error()))
+//	    return
+//	  }
+//	  if err := inf.Tx.Tx.Commit(); err != nil {
+//	    api.HandleErr(w, r, inf.Tx.Tx, http.StatusInternalServerError, nil, errors.New("committing transaction: " + err.Error()))
+//	    return
+//	  }
+//	  api.WriteResp(w, r, respObj)
+//	}
 func NewInfo(r *http.Request, requiredParams []string, intParamNames []string) (*APIInfo, error, error, int) {
 	db, err := GetDB(r.Context())
 	if err != nil {
@@ -625,6 +626,15 @@ func (inf APIInfo) UseIMS() bool {
 	return inf.Config.UseIMS && inf.request.Header.Get(rfc.IfModifiedSince) != ""
 }
 
+// WriteNotModifiedResponse writes a 304 Not Modified response with the given
+// last modification time to the provided response writer. The request must be
+// provided as well, so that it can be marked as handled.
+func WriteNotModifiedResponse(t time.Time, w http.ResponseWriter, r *http.Request) {
+	AddLastModifiedHdr(w, t)
+	w.WriteHeader(http.StatusNotModified)
+	WriteResp(w, r, nil)
+}
+
 // CheckPrecondition checks a request's "preconditions" - its If-Match and
 // If-Unmodified-Since headers versus the last updated time of the requested
 // object(s), and returns (in order), an HTTP response code appropriate for the
@@ -684,13 +694,89 @@ func (inf *APIInfo) Close() {
 	}
 }
 
+// WriteOKResponse writes a 200 OK response with the given object as the
+// 'response' property of the response body.
+//
+// This CANNOT be used by any APIInfo that wasn't constructed for the caller by
+// Wrap - ing a Handler (yet).
+func (inf APIInfo) WriteOKResponse(resp any) (int, error, error) {
+	WriteResp(inf.w, inf.request, resp)
+	return http.StatusOK, nil, nil
+}
+
+// WriteOKResponseWithSummary writes a 200 OK response with the given object as
+// the 'response' property of the response body, and the given count as the
+// `count` property of the response's summary.
+//
+// This CANNOT be used by any APIInfo that wasn't constructed for the caller by
+// Wrap - ing a Handler (yet).
+//
+// Deprecated: Summary sections on responses were intended to cover up for a
+// deficiency in jQuery-based tables on the front-end, so now that we aren't
+// using those anymore it serves no purpose.
+func (inf APIInfo) WriteOKResponseWithSummary(resp any, count uint64) (int, error, error) {
+	WriteRespWithSummary(inf.w, inf.request, resp, count)
+	return http.StatusOK, nil, nil
+}
+
+// WriteNotModifiedResponse writes a 304 Not Modified response with the given
+// time as the last modified time in the headers.
+//
+// This CANNOT be used by any APIInfo that wasn't constructed for the caller by
+// Wrap - ing a Handler (yet).
+func (inf APIInfo) WriteNotModifiedResponse(lastModified time.Time) (int, error, error) {
+	inf.w.Header().Set(rfc.LastModified, FormatLastModified(lastModified))
+	inf.w.WriteHeader(http.StatusNotModified)
+	setRespWritten(inf.request)
+	return http.StatusNotModified, nil, nil
+}
+
+// WriteSuccessResponse writes the given response object as the `response`
+// property of the response body, with the accompanying message as a
+// success-level Alert.
+func (inf APIInfo) WriteSuccessResponse(resp any, message string) (int, error, error) {
+	WriteAlertsObj(inf.w, inf.request, http.StatusOK, tc.CreateAlerts(tc.SuccessLevel, message), resp)
+	return http.StatusOK, nil, nil
+}
+
+// WriteCreatedResponse writes the given response object as the `response`
+// property of the response body of a 201 created response, with the
+// accompanying message as a success-level Alert. It also sets the Location
+// header to the given path. This will be automatically prefaced with the
+// correct path to the API version the client requested.
+func (inf APIInfo) WriteCreatedResponse(resp any, message, path string) (int, error, error) {
+	inf.w.Header().Set(rfc.Location, strings.Join([]string{"/api", inf.Version.String(), strings.TrimPrefix(path, "/")}, "/"))
+	inf.w.WriteHeader(http.StatusCreated)
+	WriteAlertsObj(inf.w, inf.request, http.StatusCreated, tc.CreateAlerts(tc.SuccessLevel, message), resp)
+	return http.StatusCreated, nil, nil
+}
+
+// RequestHeaders returns the headers sent by the client in the API request.
+func (inf APIInfo) RequestHeaders() http.Header {
+	return inf.request.Header
+}
+
+// SetLastModified sets the "last modified" header on the response writer.
+//
+// This CANNOT be used by any APIInfo that wasn't constructed for the caller by
+// Wrap - ing a Handler (yet).
+func (inf APIInfo) SetLastModified(t time.Time) {
+	inf.w.Header().Set(rfc.LastModified, FormatLastModified(t))
+}
+
+// DecodeBody reads the client request's body and attempts to decode it into the
+// provided reference.
+func (inf APIInfo) DecodeBody(ref any) error {
+	return json.NewDecoder(inf.request.Body).Decode(ref)
+}
+
 // SendMail is a convenience method used to call SendMail using an APIInfo structure's configuration.
 func (inf *APIInfo) SendMail(to rfc.EmailAddress, msg []byte) (int, error, error) {
 	return SendMail(to, msg, inf.Config)
 }
 
 // IsResourceAuthorizedToCurrentUser is a convenience method used to call
-// github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/tenant.IsResourceAuthorizedToUserTx
+// github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/tenant.IsResourceAuthorizedToUserTx
 // using an APIInfo structure to provide the current user and database transaction.
 func (inf *APIInfo) IsResourceAuthorizedToCurrentUser(resourceTenantID int) (bool, error) {
 	return tenant.IsResourceAuthorizedToUserTx(resourceTenantID, inf.User, inf.Tx.Tx)
@@ -794,11 +880,28 @@ func (val APIInfoImpl) APIInfo() *APIInfo {
 	return val.ReqInfo
 }
 
+// Version represents an API version.
 type Version struct {
 	Major uint64
 	Minor uint64
 }
 
+// String implements the fmt.Stringer interface.
+func (v *Version) String() string {
+	if v == nil {
+		return "{{null}}"
+	}
+	return strconv.FormatUint(v.Major, 10) + "." + strconv.FormatUint(v.Minor, 10)
+}
+
+func (v *Version) LessThan(otherVersion *Version) bool {
+	return v.Major < otherVersion.Major || (v.Major == otherVersion.Major && v.Minor < otherVersion.Minor)
+}
+
+func (v *Version) GreaterThanOrEqualTo(otherVersion *Version) bool {
+	return !v.LessThan(otherVersion)
+}
+
 // GetRequestedAPIVersion returns a pointer to the requested API Version from the request if it exists or returns nil otherwise.
 func GetRequestedAPIVersion(path string) *Version {
 	pathParts := strings.Split(path, "/")
@@ -982,7 +1085,6 @@ func parseEnumConstraint(err *pq.Error) (error, error, int) {
 //
 // It may be helpful to look at constraints for api_capability, role_capability,
 // and user_role for examples.
-//
 func parseRestrictFKConstraint(err *pq.Error) (error, error, int) {
 	pattern := regexp.MustCompile(`update or delete on table "([a-z_]+)" violates foreign key constraint ".+" on table "([a-z_]+)"`)
 	match := pattern.FindStringSubmatch(err.Message)
@@ -1066,7 +1168,7 @@ func GetUserFromReq(w http.ResponseWriter, r *http.Request, secret string) (auth
 				continue
 			}
 			switch givenCookie.Name {
-			case AccessToken:
+			case rfc.AccessToken:
 				bearerCookie, readToken, err := getCookieFromAccessToken(givenCookie.Value, secret)
 				if err != nil {
 					return auth.CurrentUser{}, errors.New("unauthorized, please log in."), err, http.StatusUnauthorized
@@ -1083,9 +1185,9 @@ func GetUserFromReq(w http.ResponseWriter, r *http.Request, secret string) (auth
 		return auth.CurrentUser{}, errors.New("unauthorized, please log in."), nil, http.StatusUnauthorized
 	}
 
-	oldCookie, err := tocookie.Parse(secret, cookie.Value)
-	if err != nil {
-		return auth.CurrentUser{}, errors.New("unauthorized, please log in."), errors.New("error parsing cookie: " + err.Error()), http.StatusUnauthorized
+	oldCookie, userErr, sysErr := tocookie.Parse(secret, cookie.Value)
+	if oldCookie == nil || userErr != nil || sysErr != nil {
+		return auth.CurrentUser{}, userErr, sysErr, http.StatusUnauthorized
 	}
 
 	username := oldCookie.AuthData
@@ -1130,7 +1232,7 @@ func GetUserFromReq(w http.ResponseWriter, r *http.Request, secret string) (auth
 		}
 
 		http.SetCookie(w, &http.Cookie{
-			Name:     AccessToken,
+			Name:     rfc.AccessToken,
 			Value:    string(jwtSigned),
 			Path:     "/",
 			MaxAge:   newCookie.MaxAge,
