@@ -22,13 +22,13 @@ package crconfig
 import (
 	"database/sql"
 	"errors"
-	"fmt"
+	"math"
 	"strconv"
 	"strings"
 	"time"
 
-	"github.com/apache/trafficcontrol/lib/go-log"
-	"github.com/apache/trafficcontrol/lib/go-tc"
+	"github.com/apache/trafficcontrol/v8/lib/go-log"
+	"github.com/apache/trafficcontrol/v8/lib/go-tc"
 	"github.com/lib/pq"
 )
 
@@ -43,14 +43,25 @@ const DefaultTLDTTLNS = 3600 * time.Second
 const GeoProviderMaxmindStr = "maxmindGeolocationService"
 const GeoProviderNeustarStr = "neustarGeolocationService"
 
-func makeDSes(cdn string, domain string, tx *sql.Tx) (map[string]tc.CRConfigDeliveryService, error) {
+func makeDSes(cdn string, domain string, ttlOverride int, tx *sql.Tx) (map[string]tc.CRConfigDeliveryService, error) {
 	dses := map[string]tc.CRConfigDeliveryService{}
 
 	admin := CDNSOAAdmin
-	expireSecondsStr := strconv.Itoa(int(CDNSOAExpire / time.Second))
-	minimumSecondsStr := strconv.Itoa(int(CDNSOAMinimum / time.Second))
-	refreshSecondsStr := strconv.Itoa(int(CDNSOARefresh / time.Second))
-	retrySecondsStr := strconv.Itoa(int(CDNSOARetry / time.Second))
+	expireSeconds := int(CDNSOAExpire / time.Second)
+	minimumSeconds := int(CDNSOAMinimum / time.Second)
+	refreshSeconds := int(CDNSOARefresh / time.Second)
+	retrySeconds := int(CDNSOARetry / time.Second)
+	if ttlOverride > 0 {
+		expireSeconds = int(math.Min(float64(ttlOverride), float64(expireSeconds)))
+		minimumSeconds = int(math.Min(float64(ttlOverride), float64(minimumSeconds)))
+		refreshSeconds = int(math.Min(float64(ttlOverride), float64(refreshSeconds)))
+		retrySeconds = int(math.Min(float64(ttlOverride), float64(retrySeconds)))
+	}
+
+	expireSecondsStr := strconv.Itoa(expireSeconds)
+	minimumSecondsStr := strconv.Itoa(minimumSeconds)
+	refreshSecondsStr := strconv.Itoa(refreshSeconds)
+	retrySecondsStr := strconv.Itoa(retrySeconds)
 	cdnSOA := &tc.SOA{
 		Admin:          &admin,
 		ExpireSeconds:  &expireSecondsStr,
@@ -76,6 +87,8 @@ func makeDSes(cdn string, domain string, tx *sql.Tx) (map[string]tc.CRConfigDeli
 		return nil, errors.New("getting deliveryservice parameters: " + err.Error())
 	}
 
+	ttlOverrideDuration := time.Duration(ttlOverride) * time.Second
+
 	dsParams, err := getDSParams(serverParams)
 	if err != nil {
 		return nil, errors.New("getting deliveryservice server parameters: " + err.Error())
@@ -118,9 +131,7 @@ SELECT d.anonymous_blocking_enabled,
        d.miss_long,
        p.name AS profile,
        d.protocol,
-       (SELECT ARRAY_AGG(required_capability ORDER BY required_capability)
-                         FROM deliveryservices_required_capability
-                         WHERE deliveryservice_id = d.id) AS required_capabilities,
+       d.required_capabilities,
        d.topology,
        d.tr_request_headers,
        d.tr_response_headers,
@@ -131,10 +142,10 @@ FROM deliveryservice AS d
 INNER JOIN type AS t ON t.id = d.type
 LEFT OUTER JOIN profile AS p ON p.id = d.profile
 WHERE d.cdn_id = (select id FROM cdn WHERE name = $1)
-AND d.active = true
+AND d.active = $2
+AND t.name != $3
 `
-	q += fmt.Sprintf(" and t.name != '%s'", tc.DSTypeAnyMap)
-	rows, err := tx.Query(q, cdn)
+	rows, err := tx.Query(q, cdn, tc.DSActiveStateActive, tc.DSTypeAnyMap)
 	if err != nil {
 		return nil, errors.New("querying deliveryservices: " + err.Error())
 	}
@@ -213,6 +224,10 @@ AND d.active = true
 			return nil, errors.New("scanning deliveryservice: " + err.Error())
 		}
 
+		if ttlOverride > 0 && ds.TTL != nil {
+			*ds.TTL = int(math.Min(float64(ttlOverride), float64(*ds.TTL)))
+		}
+
 		// TODO prevent (lat XOR lon) in the Tx and UI
 		if missLat.Valid && missLon.Valid {
 			ds.MissLocation = &tc.CRConfigLatitudeLongitudeShort{Lat: missLat.Float64, Lon: missLon.Float64}
@@ -313,7 +328,13 @@ AND d.active = true
 				}
 			}
 		}
+		if ttlOverride > 0 && ttlOverrideDuration < nsSeconds {
+			nsSeconds = ttlOverrideDuration
+		}
 		nsSecondsStr := strconv.Itoa(int(nsSeconds / time.Second))
+		if ttlOverride > 0 && ttlOverrideDuration < soaSeconds {
+			soaSeconds = ttlOverrideDuration
+		}
 		soaSecondsStr := strconv.Itoa(int(soaSeconds / time.Second))
 		ttlStr := ""
 		if ds.TTL != nil {
@@ -336,6 +357,9 @@ AND d.active = true
 			}
 			if dnsBypassTTL.Valid {
 				i := int(dnsBypassTTL.Int64)
+				if ttlOverride > 0 {
+					i = int(math.Min(float64(ttlOverride), float64(i)))
+				}
 				bypassDest.TTL = &i
 			}
 			if dnsBypassCName.Valid && dnsBypassCName.String != "" {
@@ -411,6 +435,13 @@ AND d.active = true
 		}
 
 		ds.StaticDNSEntries = staticDNSEntries[tc.DeliveryServiceName(xmlID)]
+		if ttlOverride > 0 {
+			for xmlID := range staticDNSEntries {
+				for index := range staticDNSEntries[xmlID] {
+					staticDNSEntries[xmlID][index].TTL = ttlOverride
+				}
+			}
+		}
 
 		dses[xmlID] = ds
 	}
@@ -431,9 +462,9 @@ from staticdnsentry as e
 inner join deliveryservice as d on d.id = e.deliveryservice
 inner join type as t on t.id = e.type
 where d.cdn_id = (select id from cdn where name = $1)
-and d.active = true
+and d.active = $2
 `
-	rows, err := tx.Query(q, cdn)
+	rows, err := tx.Query(q, cdn, tc.DSActiveStateActive)
 	if err != nil {
 		return nil, errors.New("querying static DNS entries: " + err.Error())
 	}
@@ -466,10 +497,10 @@ inner join deliveryservice as d on d.id = dr.deliveryservice
 inner join type as t on t.id = r.type
 inner join type as dt on dt.id = d.type
 where d.cdn_id = (select id from cdn where name = $1)
-and d.active = true
+and d.active = $2
 order by dr.set_number asc
 `
-	rows, err := tx.Query(q, cdn)
+	rows, err := tx.Query(q, cdn, tc.DSActiveStateActive)
 	if err != nil {
 		return nil, nil, errors.New("querying deliveryservices: " + err.Error())
 	}
@@ -534,8 +565,8 @@ func getDSParams(serverParams map[string]map[string]string) (map[string]string,
 		"tld.soa.minimum":   struct{}{},
 		"tld.soa.refresh":   struct{}{},
 		"tld.soa.retry":     struct{}{},
-		"tld.ttls.SOA":      struct{}{},
-		"tld.ttls.NS":       struct{}{},
+		tc.CrConfigTTLSOA:   struct{}{},
+		tc.CrConfigTTLNS:    struct{}{},
 		"LogRequestHeaders": struct{}{},
 	}
 	dsParams := map[string]string{}
