@@ -28,9 +28,9 @@ import (
 	"strings"
 	"time"
 
-	"github.com/apache/trafficcontrol/lib/go-log"
-	"github.com/apache/trafficcontrol/lib/go-tc"
-	"github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/topology/topology_validation"
+	"github.com/apache/trafficcontrol/v8/lib/go-log"
+	"github.com/apache/trafficcontrol/v8/lib/go-tc"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/topology/topology_validation"
 
 	"github.com/jmoiron/sqlx"
 	"github.com/lib/pq"
@@ -105,7 +105,12 @@ WHERE tm_user.email = $1
 // CheckIfCurrentUserHasCdnLock checks if the current user has the lock on the cdn that the requested operation is to be performed on.
 // This will succeed if the either there is no lock by any user on the CDN, or if the current user has the lock on the CDN.
 func CheckIfCurrentUserHasCdnLock(tx *sql.Tx, cdn, user string) (error, error, int) {
-	query := `SELECT c.username, ARRAY_REMOVE(ARRAY_AGG(u.username), NULL) AS shared_usernames FROM cdn_lock c LEFT JOIN cdn_lock_user u ON c.username = u.owner AND c.cdn = u.cdn WHERE c.cdn=$1 GROUP BY c.username`
+	query := `
+SELECT c.username, ARRAY_REMOVE(ARRAY_AGG(u.username), NULL) AS shared_usernames
+FROM cdn_lock c
+    LEFT JOIN cdn_lock_user u ON c.username = u.owner AND c.cdn = u.cdn
+WHERE c.cdn=$1
+GROUP BY c.username`
 	var userName string
 	var sharedUserNames []string
 	rows, err := tx.Query(query, cdn)
@@ -315,15 +320,15 @@ func CheckIfCurrentUserCanModifyCDNWithID(tx *sql.Tx, cdnID int64, user string)
 // This will succeed if no other user has a hard lock on any of the CDNs that relate to the cachegroup in question.
 func CheckIfCurrentUserCanModifyCachegroup(tx *sql.Tx, cachegroupID int, user string) (error, error, int) {
 	query := `
-SELECT c.username, c.cdn, c.soft, ARRAY_REMOVE(ARRAY_AGG(u.username), NULL) AS shared_usernames 
-FROM cdn_lock c LEFT JOIN cdn_lock_user u 
-    ON c.username = u.owner 
-           AND c.cdn = u.cdn 
+SELECT c.username, c.cdn, c.soft, ARRAY_REMOVE(ARRAY_AGG(u.username), NULL) AS shared_usernames
+FROM cdn_lock c LEFT JOIN cdn_lock_user u
+    ON c.username = u.owner
+           AND c.cdn = u.cdn
 WHERE c.cdn IN (
-    SELECT name FROM cdn 
+    SELECT name FROM cdn
     WHERE id IN (
-        SELECT cdn_id FROM server 
-        WHERE cachegroup = ($1))) 
+        SELECT cdn_id FROM server
+        WHERE cachegroup = ($1)))
 GROUP BY c.username, c.cdn, c.soft`
 	var userName string
 	var cdn string
@@ -357,14 +362,14 @@ GROUP BY c.username, c.cdn, c.soft`
 // CheckIfCurrentUserCanModifyCachegroups checks if the current user has the lock on the cdns that are associated with the provided cachegroup IDs.
 // This will succeed if no other user has a hard lock on any of the CDNs that relate to the cachegroups in question.
 func CheckIfCurrentUserCanModifyCachegroups(tx *sql.Tx, cachegroupIDs []int, user string) (error, error, int) {
-	query := `SELECT c.username, c.cdn, c.soft, ARRAY_REMOVE(ARRAY_AGG(u.username), NULL) AS shared_usernames FROM cdn_lock c 
-    LEFT JOIN cdn_lock_user u 
-        ON c.username = u.owner 
-               AND c.cdn = u.cdn 
+	query := `SELECT c.username, c.cdn, c.soft, ARRAY_REMOVE(ARRAY_AGG(u.username), NULL) AS shared_usernames FROM cdn_lock c
+    LEFT JOIN cdn_lock_user u
+        ON c.username = u.owner
+               AND c.cdn = u.cdn
 WHERE c.cdn IN (
-    SELECT name FROM cdn 
+    SELECT name FROM cdn
     WHERE id IN (
-        SELECT cdn_id FROM server 
+        SELECT cdn_id FROM server
         WHERE cachegroup = ANY($1)))
         GROUP BY c.username, c.cdn, c.soft`
 	var userName string
@@ -409,7 +414,7 @@ func parseCriteriaAndQueryValues(queryParamsToSQLCols map[string]WhereColumnInfo
 				err = colInfo.Checker(urlValue)
 			}
 			if err != nil {
-				errs = append(errs, errors.New(key+" "+err.Error()))
+				errs = append(errs, fmt.Errorf("%s %w", key, err))
 			} else {
 				criteria = colInfo.Column + "=:" + key
 				criteriaArgs = append(criteriaArgs, criteria)
@@ -441,15 +446,14 @@ func AddTenancyCheck(where string, queryValues map[string]interface{}, tenantCol
 //
 // Example:
 //
-//  tx, err := db.Begin()
-//  txCommit := false
-//  defer dbhelpers.CommitIf(tx, &txCommit)
-//  if err := tx.Exec("select ..."); err != nil {
-//    return errors.New("executing: " + err.Error())
-//  }
-//  txCommit = true
-//  return nil
-//
+//	tx, err := db.Begin()
+//	txCommit := false
+//	defer dbhelpers.CommitIf(tx, &txCommit)
+//	if err := tx.Exec("select ..."); err != nil {
+//	  return errors.New("executing: " + err.Error())
+//	}
+//	txCommit = true
+//	return nil
 func CommitIf(tx *sql.Tx, doCommit *bool) {
 	if *doCommit {
 		tx.Commit()
@@ -613,12 +617,16 @@ INNER JOIN type AS dt ON dt.id = ds.type
 INNER JOIN profile AS p ON p.id = s.profile
 INNER JOIN status AS st ON st.id = s.status
 WHERE ds.cdn_id = (SELECT id FROM cdn WHERE name = $1)
-AND ds.active = true
-AND dt.name != '` + tc.DSTypeAnyMap.String() + `'
+AND ds.active = $2
+AND dt.name != $3
 AND p.routing_disabled = false
-AND (st.name = '` + tc.CacheStatusOnline.String() + `' OR st.name = '` + tc.CacheStatusReported.String() + `' OR st.name = '` + tc.CacheStatusAdminDown.String() + `')
+AND (
+	st.name = $4
+	OR st.name = $5
+	OR st.name = $6
+)
 `
-	rows, err := tx.Query(q, cdn)
+	rows, err := tx.Query(q, cdn, tc.DSActiveStateActive, tc.DSTypeAnyMap, tc.CacheStatusOnline, tc.CacheStatusReported, tc.CacheStatusAdminDown)
 	if err != nil {
 		return nil, errors.New("querying server deliveryservice names by CDN: " + err.Error())
 	}
@@ -842,13 +850,11 @@ func GetRequiredCapabilitiesOfDeliveryServices(ids []int, tx *sql.Tx) (map[int][
 	dsCaps := make(map[int][]string, len(ids))
 	q := `
 SELECT
-  d.id,
-  ARRAY_REMOVE(ARRAY_AGG(dsrc.required_capability ORDER BY dsrc.required_capability), NULL) AS required_capabilities
-FROM deliveryservice d
-LEFT JOIN deliveryservices_required_capability dsrc on d.id = dsrc.deliveryservice_id
-WHERE
-  d.id = ANY($1)
-GROUP BY d.id
+  ds.id,
+  ARRAY_REMOVE((ds.required_capabilities), NULL) AS required_capabilities
+FROM deliveryservice ds
+WHERE ds.id = ANY($1)
+GROUP BY ds.id, ds.required_capabilities
 `
 	rows, err := tx.Query(q, pq.Array(&queryIDs))
 	if err != nil {
@@ -919,23 +925,14 @@ func ScanCachegroupsServerCapabilities(rows *sql.Rows) (map[string][]int, map[in
 // GetDSRequiredCapabilitiesFromID returns the server's capabilities.
 func GetDSRequiredCapabilitiesFromID(id int, tx *sql.Tx) ([]string, error) {
 	q := `
-	SELECT required_capability
-	FROM deliveryservices_required_capability
-	WHERE deliveryservice_id = $1
-	ORDER BY required_capability`
-	rows, err := tx.Query(q, id)
-	if err != nil {
-		return nil, errors.New("querying deliveryservice required capabilities from id: " + err.Error())
-	}
-	defer rows.Close()
+	SELECT required_capabilities
+	FROM deliveryservice
+	WHERE id = $1
+	ORDER BY required_capabilities`
 
 	caps := []string{}
-	for rows.Next() {
-		var cap string
-		if err := rows.Scan(&cap); err != nil {
-			return nil, errors.New("scanning capability: " + err.Error())
-		}
-		caps = append(caps, cap)
+	if err := tx.QueryRow(q, id).Scan(pq.Array(&caps)); err != nil {
+		return nil, errors.New("getting/ scanning capability: " + err.Error())
 	}
 	return caps, nil
 }
@@ -1317,26 +1314,24 @@ func TopologyExists(tx *sql.Tx, name string) (bool, error) {
 // one of the Topology's Cache Groups is empty with respect to the Delivery
 // Service's CDN. Note that this can panic if ds does not have a properly set
 // CDNID.
-func CheckTopology(tx *sqlx.Tx, ds tc.DeliveryServiceV4) (int, error, error) {
-	statusCode, userErr, sysErr := http.StatusOK, error(nil), error(nil)
-
+func CheckTopology(tx *sqlx.Tx, ds tc.DeliveryServiceV5) (int, error, error) {
 	if ds.Topology == nil {
-		return statusCode, userErr, sysErr
+		return http.StatusOK, nil, nil
 	}
 
 	cacheGroupIDs, _, err := GetTopologyCachegroups(tx.Tx, *ds.Topology)
 	if err != nil {
-		return http.StatusInternalServerError, nil, fmt.Errorf("getting topology cachegroups: %v", err)
+		return http.StatusInternalServerError, nil, fmt.Errorf("getting topology cachegroups: %w", err)
 	}
 	if len(cacheGroupIDs) == 0 {
 		return http.StatusBadRequest, fmt.Errorf("no such Topology '%s'", *ds.Topology), nil
 	}
 
-	if err = topology_validation.CheckForEmptyCacheGroups(tx, cacheGroupIDs, []int{*ds.CDNID}, true, []int{}); err != nil {
-		return http.StatusBadRequest, fmt.Errorf("empty cachegroups in Topology %s found for CDN %d: %w", *ds.Topology, *ds.CDNID, err), nil
+	if err = topology_validation.CheckForEmptyCacheGroups(tx, cacheGroupIDs, []int{ds.CDNID}, true, []int{}); err != nil {
+		return http.StatusBadRequest, fmt.Errorf("empty cachegroups in Topology %s found for CDN %d: %w", *ds.Topology, ds.CDNID, err), nil
 	}
 
-	return statusCode, userErr, sysErr
+	return http.StatusOK, nil, nil
 }
 
 // GetTopologyCachegroups returns an array of cachegroup IDs and an array of cachegroup
@@ -1540,7 +1535,7 @@ func GetDeliveryServiceTypeAndCDNName(dsID int, tx *sql.Tx) (tc.DSType, string,
 	return dsType, cdnName, true, nil
 }
 
-// GetDeliveryServiceTypeAndTopology returns the type of the deliveryservice and the name of its topology.
+// GetDeliveryServiceTypeRequiredCapabilitiesAndTopology returns the type of the deliveryservice and the name of its topology.
 func GetDeliveryServiceTypeRequiredCapabilitiesAndTopology(dsID int, tx *sql.Tx) (tc.DSType, []string, *string, bool, error) {
 	var dsType tc.DSType
 	var reqCap []string
@@ -1548,13 +1543,12 @@ func GetDeliveryServiceTypeRequiredCapabilitiesAndTopology(dsID int, tx *sql.Tx)
 	q := `
 SELECT
   t.name,
-  ARRAY_REMOVE(ARRAY_AGG(dsrc.required_capability ORDER BY dsrc.required_capability), NULL) AS required_capabilities,
+  ARRAY_REMOVE(ds.required_capabilities, NULL) AS required_capabilities,
   ds.topology
 FROM deliveryservice AS ds
-LEFT JOIN deliveryservices_required_capability AS dsrc ON dsrc.deliveryservice_id = ds.id
 JOIN type t ON ds.type = t.id
 WHERE ds.id = $1
-GROUP BY t.name, ds.topology
+GROUP BY t.name, ds.topology, ds.required_capabilities
 `
 	if err := tx.QueryRow(q, dsID).Scan(&dsType, pq.Array(&reqCap), &topology); err != nil {
 		if err == sql.ErrNoRows {
@@ -1981,6 +1975,30 @@ WHERE server.id = $2;`
 	return nil
 }
 
+// SetUpdateFailedForServer sets the update failed flag for the server.
+func SetUpdateFailedForServer(tx *sql.Tx, serverID int64, failed bool) error {
+	query := `
+UPDATE public.server
+SET config_update_failed = $1
+WHERE server.id = $2`
+	if _, err := tx.Exec(query, failed, serverID); err != nil {
+		return fmt.Errorf("setting config update failed for ServerID %d with value %v: %w", serverID, failed, err)
+	}
+	return nil
+}
+
+// SetRevalFailedForServer sets the reval failed flag for the server.
+func SetRevalFailedForServer(tx *sql.Tx, serverID int64, failed bool) error {
+	query := `
+UPDATE public.server
+SET revalidate_update_failed = $1
+WHERE server.id = $2`
+	if _, err := tx.Exec(query, failed, serverID); err != nil {
+		return fmt.Errorf("setting reval update failed for ServerID %d with value %v: %w", serverID, failed, err)
+	}
+	return nil
+}
+
 // QueueRevalForServer sets the revalidate update time for the server to now.
 func QueueRevalForServer(tx *sql.Tx, serverID int64) error {
 	query := `
@@ -2178,3 +2196,180 @@ func GetProfileIDDesc(tx *sql.Tx, name string) (id int, desc string) {
 	}
 	return
 }
+
+// GetSCInfo confirms whether the server capability exists, and an error (if one occurs).
+func GetSCInfo(tx *sql.Tx, name string) (bool, error) {
+	var count int
+	if err := tx.QueryRow("SELECT count(name) FROM server_capability AS sc WHERE sc.name=$1", name).Scan(&count); err != nil {
+		return false, fmt.Errorf("error getting server capability info: %w", err)
+	}
+	if count == 0 {
+		return false, nil
+	}
+	if count != 1 {
+		return false, fmt.Errorf("getting server capability info - expected row count: 1, actual: %d", count)
+	}
+	return true, nil
+}
+
+// ServiceCategoryExists confirms whether the service category exists, and an error (if one occurs).
+func ServiceCategoryExists(tx *sql.Tx, name string) (bool, error) {
+	var count int
+	if err := tx.QueryRow("SELECT count(name) FROM service_category AS sc WHERE sc.name=$1", name).Scan(&count); err != nil {
+		return false, fmt.Errorf("error getting service category info: %w", err)
+	}
+	if count == 0 {
+		return false, nil
+	}
+	if count != 1 {
+		return false, fmt.Errorf("getting service category info - expected row count: 1, actual: %d", count)
+	}
+	return true, nil
+}
+
+// ASNExists confirms whether the asn exists, and an error (if one occurs).
+func ASNExists(tx *sql.Tx, id string) (bool, error) {
+	var count int
+	if err := tx.QueryRow("SELECT count(asn) FROM asn WHERE id=$1", id).Scan(&count); err != nil {
+		return false, fmt.Errorf("error getting asn info: %w", err)
+	}
+	if count == 0 {
+		return false, nil
+	}
+	if count != 1 {
+		return false, fmt.Errorf("getting asn info - expected row count: 1, actual: %d", count)
+	}
+	return true, nil
+}
+
+// CacheGroupExists confirms whether the cache group exists, and an error (if one occurs).
+func CacheGroupExists(tx *sql.Tx, name string) (bool, error) {
+	var count int
+	if err := tx.QueryRow("SELECT count(name) FROM cachegroup AS cg WHERE cg.name=$1", name).Scan(&count); err != nil {
+		return false, fmt.Errorf("error getting cache group info: %w", err)
+	}
+	if count == 0 {
+		return false, nil
+	}
+	if count != 1 {
+		return false, fmt.Errorf("getting cache group info - expected row count: 1, actual: %d", count)
+	}
+	return true, nil
+}
+
+// DivisionExists confirms whether the division exists, and an error (if one occurs).
+func DivisionExists(tx *sql.Tx, id string) (bool, error) {
+	var count int
+	if err := tx.QueryRow("SELECT count(id) FROM division AS div WHERE div.id=$1", id).Scan(&count); err != nil {
+		return false, fmt.Errorf("error getting divisions info: %w", err)
+	}
+	if count == 0 {
+		return false, nil
+	}
+	if count != 1 {
+		return false, fmt.Errorf("getting division info - expected row count: 1, actual: %d", count)
+	}
+	return true, nil
+}
+
+// PhysLocationExists confirms whether the PhysLocation exists, and an error (if one occurs).
+func PhysLocationExists(tx *sql.Tx, id string) (bool, error) {
+	var count int
+	if err := tx.QueryRow("SELECT count(name) FROM phys_location WHERE id=$1", id).Scan(&count); err != nil {
+		return false, fmt.Errorf("error getting PhysLocation info: %w", err)
+	}
+	if count == 0 {
+		return false, nil
+	}
+	if count != 1 {
+		return false, fmt.Errorf("getting PhysLocation info - expected row count: 1, actual: %d", count)
+	}
+	return true, nil
+}
+
+// ParameterExists confirms whether the Parameter exists, and an error (if one occurs).
+func ParameterExists(tx *sql.Tx, id string) (bool, error) {
+	var count int
+	if err := tx.QueryRow("SELECT count(name) FROM parameter WHERE id=$1", id).Scan(&count); err != nil {
+		return false, fmt.Errorf("error getting Parameter info: %w", err)
+	}
+	if count == 0 {
+		return false, nil
+	}
+	if count != 1 {
+		return false, fmt.Errorf("getting Parameter info - expected row count: 1, actual: %d", count)
+	}
+	return true, nil
+}
+
+// ProfileExists confirms whether the profile exists, and an error (if one occurs).
+func ProfileExists(tx *sql.Tx, id string) (bool, error) {
+	var count int
+	if err := tx.QueryRow(`SELECT count(name) FROM profile WHERE id=$1`, id).Scan(&count); err != nil {
+		return false, fmt.Errorf("error getting profile info: %w", err)
+	}
+	if count == 0 {
+		return false, nil
+	}
+	if count != 1 {
+		return false, fmt.Errorf("getting profile info - expected row count: 1, actual: %d", count)
+	}
+	return true, nil
+}
+
+// GetCoordinateID obtains coordinateID, and an error (if one occurs)
+func GetCoordinateID(tx *sql.Tx, id int) (*int, error) {
+	q := `SELECT coordinate FROM cachegroup WHERE id = $1`
+
+	var coordinateID *int
+	if err := tx.QueryRow(q, id).Scan(&coordinateID); err != nil {
+		return nil, err
+	}
+
+	return coordinateID, nil
+}
+
+// DeleteCoordinate deletes coordinate by id, and an error (if one occurs)
+func DeleteCoordinate(tx *sql.Tx, cacheGroupID int, coordinateID int) error {
+	q := `UPDATE cachegroup SET coordinate = NULL WHERE id = $1`
+	result, err := tx.Exec(q, cacheGroupID)
+	if err != nil {
+		return fmt.Errorf("updating cachegroup %d coordinate to null: %s", cacheGroupID, err.Error())
+	}
+	rowsAffected, err := result.RowsAffected()
+	if err != nil {
+		return fmt.Errorf("updating cachegroup %d coordinate to null, getting rows affected: %s", coordinateID, err.Error())
+	}
+	if rowsAffected == 0 {
+		return fmt.Errorf("updating cachegroup %d coordinate to null, zero rows affected", coordinateID)
+	}
+
+	q = `DELETE FROM coordinate WHERE id = $1`
+	result, err = tx.Exec(q, coordinateID)
+	if err != nil {
+		return fmt.Errorf("delete coordinate %d for cachegroup %d: %s", coordinateID, coordinateID, err.Error())
+	}
+	rowsAffected, err = result.RowsAffected()
+	if err != nil {
+		return fmt.Errorf("delete coordinate %d for cachegroup %d, getting rows affected: %s", coordinateID, coordinateID, err.Error())
+	}
+	if rowsAffected == 0 {
+		return fmt.Errorf("delete coordinate %d for cachegroup %d, zero rows affected", coordinateID, coordinateID)
+	}
+	return nil
+}
+
+// ProfileParameterExists confirms whether the ProfileParameter exists, and an error (if one occurs).
+func ProfileParameterExists(tx *sql.Tx, profileID string, parameterID string) (bool, error) {
+	var count int
+	if err := tx.QueryRow("SELECT count(*) FROM profile_parameter WHERE profile=$1 and parameter=$2", profileID, parameterID).Scan(&count); err != nil {
+		return false, fmt.Errorf("error getting profile_parameter info: %w", err)
+	}
+	if count == 0 {
+		return false, nil
+	}
+	if count != 1 {
+		return false, fmt.Errorf("getting profile_parameter info - expected row count: 1, actual: %d", count)
+	}
+	return true, nil
+}
