@@ -9,11 +9,11 @@ import (
 	"testing"
 	"time"
 
-	"github.com/apache/trafficcontrol/lib/go-tc"
-	"github.com/apache/trafficcontrol/lib/go-util"
+	"github.com/apache/trafficcontrol/v8/lib/go-tc"
+	"github.com/apache/trafficcontrol/v8/lib/go-util"
 
-	"github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/api"
-	"github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/parameter"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/api"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/parameter"
 
 	"github.com/jmoiron/sqlx"
 	"github.com/lib/pq"
@@ -271,19 +271,26 @@ func TestGetProfiles(t *testing.T) {
 
 	mock.ExpectBegin()
 	rows := sqlmock.NewRows([]string{"profile", "name", "value"})
+	rows1 := sqlmock.NewRows([]string{"profile", "name", "value"})
 	for _, profile := range profiles {
 		for paramName, paramVal := range profile.Parameters {
-			rows = rows.AddRow(profile.Name, paramName, paramVal)
+			if profile.Name == "routerProfile" {
+				rows = rows.AddRow(profile.Name, paramName, paramVal)
+			} else if profile.Name == "cacheProfile" {
+				rows1 = rows1.AddRow(profile.Name, paramName, paramVal)
+			}
 		}
 	}
 
 	caches := []Cache{cache}
 	routers := []Router{router}
 
-	profileNames := []string{"cacheProfile"}
-
+	profileNames := []string{"routerProfile"}
 	mock.ExpectQuery("SELECT").WithArgs(pq.Array(profileNames), CacheMonitorConfigFile).WillReturnRows(rows)
 
+	profileNames = []string{"cacheProfile"}
+	mock.ExpectQuery("SELECT").WithArgs(pq.Array(profileNames), CacheMonitorConfigFile).WillReturnRows(rows1)
+
 	dbCtx, f := context.WithTimeout(context.TODO(), time.Duration(10)*time.Second)
 	defer f()
 	tx, err := db.BeginTx(dbCtx, nil)
@@ -494,19 +501,26 @@ func TestGetMonitoringJSON(t *testing.T) {
 		}
 
 		rows := sqlmock.NewRows([]string{"profile", "name", "value"})
+		rows1 := sqlmock.NewRows([]string{"profile", "name", "value"})
 		for _, profile := range profiles {
 			for paramName, paramVal := range profile.Parameters {
-				rows = rows.AddRow(profile.Name, paramName, paramVal)
+				if profile.Name == "routerProfile" {
+					rows = rows.AddRow(profile.Name, paramName, paramVal)
+				} else if profile.Name == "cacheProfile" {
+					rows1 = rows1.AddRow(profile.Name, paramName, paramVal)
+				}
 			}
 		}
 
 		// caches := []Cache{cache}
 		// routers := []Router{router}
 
-		profileNames := []string{"cacheProfile"}
-
+		profileNames := []string{"routerProfile"}
 		mock.ExpectQuery("SELECT").WithArgs(pq.Array(profileNames), CacheMonitorConfigFile).WillReturnRows(rows)
 		resp.Response.Profiles = profiles
+
+		profileNames = []string{"cacheProfile"}
+		mock.ExpectQuery("SELECT").WithArgs(pq.Array(profileNames), CacheMonitorConfigFile).WillReturnRows(rows1)
 	}
 	{
 		//
