@@ -17,7 +17,7 @@ package plugin
 import (
 	"encoding/json"
 
-	"github.com/apache/trafficcontrol/lib/go-log"
+	"github.com/apache/trafficcontrol/v8/lib/go-log"
 )
 
 func init() {
