@@ -26,62 +26,62 @@ import (
 	"testing"
 	"time"
 
-	"github.com/apache/trafficcontrol/lib/go-tc"
-	"github.com/apache/trafficcontrol/lib/go-util"
+	"github.com/apache/trafficcontrol/v8/lib/go-tc"
+	"github.com/apache/trafficcontrol/v8/lib/go-util"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/api"
+	"github.com/apache/trafficcontrol/v8/traffic_ops/traffic_ops_golang/auth"
 
-	"github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/api"
-	"github.com/apache/trafficcontrol/traffic_ops/traffic_ops_golang/auth"
 	"github.com/jmoiron/sqlx"
+	"github.com/lib/pq"
 	"gopkg.in/DATA-DOG/go-sqlmock.v1"
 )
 
 type ServerAndInterfaces struct {
-	Server    tc.ServerV40
+	Server    tc.ServerV5
 	Interface tc.ServerInterfaceInfoV40
 }
 
 func getTestServers() []ServerAndInterfaces {
 	servers := []ServerAndInterfaces{}
-	testServer := tc.ServerV40{
-		Cachegroup:        util.StrPtr("Cachegroup"),
-		CachegroupID:      util.IntPtr(1),
-		CDNID:             util.IntPtr(1),
-		CDNName:           util.StrPtr("cdnName"),
-		DomainName:        util.StrPtr("domainName"),
-		GUID:              util.StrPtr("guid"),
-		HostName:          util.StrPtr("server1"),
-		HTTPSPort:         util.IntPtr(443),
-		ID:                util.IntPtr(1),
-		ILOIPAddress:      util.StrPtr("iloIpAddress"),
-		ILOIPGateway:      util.StrPtr("iloIpGateway"),
-		ILOIPNetmask:      util.StrPtr("iloIpNetmask"),
-		ILOPassword:       util.StrPtr("iloPassword"),
-		ILOUsername:       util.StrPtr("iloUsername"),
-		LastUpdated:       &tc.TimeNoMod{Time: time.Now()},
-		MgmtIPAddress:     util.StrPtr("mgmtIpAddress"),
-		MgmtIPGateway:     util.StrPtr("mgmtIpGateway"),
-		MgmtIPNetmask:     util.StrPtr("mgmtIpNetmask"),
-		OfflineReason:     util.StrPtr("offlineReason"),
-		PhysLocation:      util.StrPtr("physLocation"),
-		PhysLocationID:    util.IntPtr(1),
-		ProfileNames:      []string{"profile"},
-		Rack:              util.StrPtr("rack"),
-		RevalPending:      util.BoolPtr(true),
-		Status:            util.StrPtr("status"),
-		StatusID:          util.IntPtr(1),
-		TCPPort:           util.IntPtr(80),
-		Type:              "EDGE",
-		TypeID:            util.IntPtr(1),
-		UpdPending:        util.BoolPtr(true),
-		XMPPID:            util.StrPtr("xmppId"),
-		XMPPPasswd:        util.StrPtr("xmppPasswd"),
-		StatusLastUpdated: &(time.Time{}),
-		ConfigUpdateTime:  &(time.Time{}),
-		ConfigApplyTime:   &(time.Time{}),
-		RevalUpdateTime:   &(time.Time{}),
-		RevalApplyTime:    &(time.Time{}),
+	testServerV40 := tc.ServerV5{
+		CacheGroup:         "Cache Group",
+		CacheGroupID:       1,
+		CDNID:              1,
+		CDN:                "cdnName",
+		DomainName:         "domainName",
+		GUID:               util.Ptr("guid"),
+		HostName:           "server1",
+		HTTPSPort:          util.Ptr(443),
+		ID:                 1,
+		ILOIPAddress:       util.Ptr("iloIpAddress"),
+		ILOIPGateway:       util.Ptr("iloIpGateway"),
+		ILOIPNetmask:       util.Ptr("iloIpNetmask"),
+		ILOPassword:        util.Ptr("iloPassword"),
+		ILOUsername:        util.Ptr("iloUsername"),
+		LastUpdated:        time.Now(),
+		MgmtIPAddress:      util.Ptr("mgmtIpAddress"),
+		MgmtIPGateway:      util.Ptr("mgmtIpGateway"),
+		MgmtIPNetmask:      util.Ptr("mgmtIpNetmask"),
+		OfflineReason:      util.Ptr("offlineReason"),
+		PhysicalLocation:   "physLocation",
+		PhysicalLocationID: 1,
+		Profiles:           []string{"profile"},
+		Rack:               util.Ptr("rack"),
+		Status:             "status",
+		StatusID:           1,
+		TCPPort:            util.Ptr(80),
+		Type:               "EDGE",
+		TypeID:             1,
+		XMPPID:             util.Ptr("xmppId"),
+		XMPPPasswd:         util.Ptr("xmppPasswd"),
+		StatusLastUpdated:  &(time.Time{}),
+		ConfigUpdateTime:   &(time.Time{}),
+		ConfigApplyTime:    &(time.Time{}),
+		ConfigUpdateFailed: false,
+		RevalUpdateTime:    &(time.Time{}),
+		RevalApplyTime:     &(time.Time{}),
+		RevalUpdateFailed:  false,
 	}
-
 	mtu := uint64(9500)
 
 	iface := tc.ServerInterfaceInfoV40{
@@ -102,25 +102,25 @@ func getTestServers() []ServerAndInterfaces {
 		RouterPortName: "routerPortName",
 	}
 
-	servers = append(servers, ServerAndInterfaces{Server: testServer, Interface: iface})
+	servers = append(servers, ServerAndInterfaces{Server: testServerV40, Interface: iface})
 
-	testServer2 := testServer
-	testServer2.Cachegroup = util.StrPtr("cachegroup2")
-	testServer2.HostName = util.StrPtr("server2")
-	testServer2.ID = util.IntPtr(2)
+	testServer2 := testServerV40
+	testServer2.CacheGroup = "cachegroup2"
+	testServer2.HostName = "server2"
+	testServer2.ID = 2
 	servers = append(servers, ServerAndInterfaces{Server: testServer2, Interface: iface})
 
-	testServer3 := testServer
-	testServer3.Cachegroup = util.StrPtr("cachegroup3")
-	testServer3.HostName = util.StrPtr("server3")
-	testServer3.ID = util.IntPtr(3)
+	testServer3 := testServerV40
+	testServer3.CacheGroup = "cachegroup3"
+	testServer3.HostName = "server3"
+	testServer3.ID = 3
 	servers = append(servers, ServerAndInterfaces{Server: testServer3, Interface: iface})
 
 	return servers
 }
 
 // Test to make sure that updating the "cdn" of a server already assigned to a DS fails
-func TestUpdateServer(t *testing.T) {
+func TestCheckTypeChangeSafety(t *testing.T) {
 	mockDB, mock, err := sqlmock.New()
 	if err != nil {
 		t.Fatalf("an error '%s' was not expected when opening a stub database connection", err)
@@ -134,7 +134,7 @@ func TestUpdateServer(t *testing.T) {
 
 	rows := sqlmock.NewRows([]string{"type", "cdn_id"})
 	// note here that the cdnid is 5, which is not the same as the initial cdnid of the fist traffic server
-	rows.AddRow(*testServers[0].Server.TypeID, 5)
+	rows.AddRow(testServers[0].Server.TypeID, 5)
 	// Make it return a list of atleast one associated ds
 	dsrows := sqlmock.NewRows([]string{"array"})
 	dsrows.AddRow("{3}")
@@ -142,11 +142,10 @@ func TestUpdateServer(t *testing.T) {
 	mock.ExpectQuery("SELECT").WillReturnRows(rows)
 	mock.ExpectQuery("SELECT ARRAY").WillReturnRows(dsrows)
 
-	s := tc.ServerV40{
-		CDNID:    testServers[0].Server.CDNID,
-		FqdnTime: time.Time{},
-		TypeID:   testServers[0].Server.TypeID,
-		ID:       testServers[0].Server.ID,
+	s := tc.ServerV5{
+		CDNID:  testServers[0].Server.CDNID,
+		TypeID: testServers[0].Server.TypeID,
+		ID:     testServers[0].Server.ID,
 	}
 
 	userErr, _, errCode := checkTypeChangeSafety(s, db.MustBegin())
@@ -177,9 +176,10 @@ func TestGetServersByCachegroup(t *testing.T) {
 	cols := []string{"cachegroup", "cachegroup_id", "cdn_id", "cdn_name", "domain_name", "guid", "host_name",
 		"https_port", "id", "ilo_ip_address", "ilo_ip_gateway", "ilo_ip_netmask", "ilo_password", "ilo_username",
 		"last_updated", "mgmt_ip_address", "mgmt_ip_gateway", "mgmt_ip_netmask", "offline_reason", "phys_location",
-		"phys_location_id", "profile_name", "rack", "reval_pending", "revalidate_update_time", "revalidate_apply_time",
-		"status", "status_id", "tcp_port", "server_type", "server_type_id", "upd_pending", "config_update_time",
-		"config_apply_time", "xmpp_id", "xmpp_passwd", "status_last_updated"}
+		"phys_location_id", "profile_name", "rack", "revalidate_update_time", "revalidate_apply_time",
+		"revalidate_update_failed", "status", "status_id", "tcp_port", "server_type", "server_type_id",
+		"config_update_time", "config_apply_time", "config_update_failed", "xmpp_id", "xmpp_passwd",
+		"status_last_updated"}
 	interfaceCols := []string{"max_bandwidth", "monitor", "mtu", "name", "server", "router_host_name", "router_port_name"}
 	rows := sqlmock.NewRows(cols)
 	interfaceRows := sqlmock.NewRows(interfaceCols)
@@ -192,40 +192,40 @@ func TestGetServersByCachegroup(t *testing.T) {
 	for _, srv := range testServers {
 		ts := srv.Server
 		rows = rows.AddRow(
-			*ts.Cachegroup,
-			*ts.CachegroupID,
-			*ts.CDNID,
-			*ts.CDNName,
-			*ts.DomainName,
+			ts.CacheGroup,
+			ts.CacheGroupID,
+			ts.CDNID,
+			ts.CDN,
+			ts.DomainName,
 			*ts.GUID,
-			*ts.HostName,
+			ts.HostName,
 			*ts.HTTPSPort,
-			*ts.ID,
+			ts.ID,
 			*ts.ILOIPAddress,
 			*ts.ILOIPGateway,
 			*ts.ILOIPNetmask,
 			*ts.ILOPassword,
 			*ts.ILOUsername,
-			*ts.LastUpdated,
+			ts.LastUpdated,
 			*ts.MgmtIPAddress,
 			*ts.MgmtIPGateway,
 			*ts.MgmtIPNetmask,
 			*ts.OfflineReason,
-			*ts.PhysLocation,
-			*ts.PhysLocationID,
-			fmt.Sprintf("{%s}", strings.Join(ts.ProfileNames, ",")),
+			ts.PhysicalLocation,
+			ts.PhysicalLocationID,
+			fmt.Sprintf("{%s}", strings.Join(ts.Profiles, ",")),
 			*ts.Rack,
-			*ts.RevalPending,
 			*ts.RevalUpdateTime,
 			*ts.RevalApplyTime,
-			*ts.Status,
-			*ts.StatusID,
+			ts.RevalUpdateFailed,
+			ts.Status,
+			ts.StatusID,
 			*ts.TCPPort,
 			ts.Type,
-			*ts.TypeID,
-			*ts.UpdPending,
+			ts.TypeID,
 			*ts.ConfigUpdateTime,
 			*ts.ConfigApplyTime,
+			ts.ConfigUpdateFailed,
 			*ts.XMPPID,
 			*ts.XMPPPasswd,
 			*ts.StatusLastUpdated,
@@ -235,7 +235,7 @@ func TestGetServersByCachegroup(t *testing.T) {
 			srv.Interface.Monitor,
 			srv.Interface.MTU,
 			srv.Interface.Name,
-			*ts.ID,
+			ts.ID,
 			srv.Interface.RouterHostName,
 			srv.Interface.RouterPortName,
 		)
@@ -245,7 +245,7 @@ func TestGetServersByCachegroup(t *testing.T) {
 				ip.Address,
 				ip.Gateway,
 				ip.ServiceAddress,
-				*ts.ID,
+				ts.ID,
 				srv.Interface.Name,
 			)
 		}
@@ -265,7 +265,7 @@ func TestGetServersByCachegroup(t *testing.T) {
 
 	version := api.Version{Major: 4, Minor: 0}
 
-	servers, _, userErr, sysErr, errCode, _ := getServers(nil, v, db.MustBegin(), &user, false, version)
+	servers, _, userErr, sysErr, errCode, _ := getServers(nil, v, db.MustBegin(), &user, false, version, false)
 	if userErr != nil || sysErr != nil {
 		t.Errorf("getServers expected: no errors, actual: %v %v with status: %s", userErr, sysErr, http.StatusText(errCode))
 	}
@@ -288,8 +288,8 @@ func TestGetMidServers(t *testing.T) {
 	testServers := getTestServers()
 	testServers = testServers[0:2]
 
-	testServers[1].Server.Cachegroup = util.StrPtr("parentCacheGroup")
-	testServers[1].Server.CachegroupID = util.IntPtr(2)
+	testServers[1].Server.CacheGroup = "parentCacheGroup"
+	testServers[1].Server.CacheGroupID = 2
 	testServers[1].Server.Type = "MID"
 
 	unfilteredCols := []string{"count"}
@@ -298,9 +298,10 @@ func TestGetMidServers(t *testing.T) {
 	cols := []string{"cachegroup", "cachegroup_id", "cdn_id", "cdn_name", "domain_name", "guid", "host_name",
 		"https_port", "id", "ilo_ip_address", "ilo_ip_gateway", "ilo_ip_netmask", "ilo_password", "ilo_username",
 		"last_updated", "mgmt_ip_address", "mgmt_ip_gateway", "mgmt_ip_netmask", "offline_reason", "phys_location",
-		"phys_location_id", "profile_name", "rack", "reval_pending", "revalidate_update_time", "revalidate_apply_time",
-		"status", "status_id", "tcp_port", "server_type", "server_type_id", "upd_pending", "config_update_time",
-		"config_apply_time", "xmpp_id", "xmpp_passwd", "status_last_updated"}
+		"phys_location_id", "profile_name", "rack", "revalidate_update_time", "revalidate_apply_time",
+		"revalidate_update_failed", "status", "status_id", "tcp_port", "server_type", "server_type_id",
+		"config_update_time", "config_apply_time", "config_update_failed", "xmpp_id", "xmpp_passwd",
+		"status_last_updated"}
 	interfaceCols := []string{"max_bandwidth", "monitor", "mtu", "name", "server", "router_host_name", "router_port_name"}
 	rows := sqlmock.NewRows(cols)
 	interfaceRows := sqlmock.NewRows(interfaceCols)
@@ -311,40 +312,40 @@ func TestGetMidServers(t *testing.T) {
 	for _, srv := range testServers {
 		ts := srv.Server
 		rows = rows.AddRow(
-			*ts.Cachegroup,
-			*ts.CachegroupID,
-			*ts.CDNID,
-			*ts.CDNName,
-			*ts.DomainName,
+			ts.CacheGroup,
+			ts.CacheGroupID,
+			ts.CDNID,
+			ts.CDN,
+			ts.DomainName,
 			*ts.GUID,
-			*ts.HostName,
+			ts.HostName,
 			*ts.HTTPSPort,
-			*ts.ID,
+			ts.ID,
 			*ts.ILOIPAddress,
 			*ts.ILOIPGateway,
 			*ts.ILOIPNetmask,
 			*ts.ILOPassword,
 			*ts.ILOUsername,
-			*ts.LastUpdated,
+			ts.LastUpdated,
 			*ts.MgmtIPAddress,
 			*ts.MgmtIPGateway,
 			*ts.MgmtIPNetmask,
 			*ts.OfflineReason,
-			*ts.PhysLocation,
-			*ts.PhysLocationID,
-			fmt.Sprintf("{%s}", strings.Join(ts.ProfileNames, ",")),
+			ts.PhysicalLocation,
+			ts.PhysicalLocationID,
+			fmt.Sprintf("{%s}", strings.Join(ts.Profiles, ",")),
 			*ts.Rack,
-			*ts.RevalPending,
 			*ts.RevalUpdateTime,
 			*ts.RevalApplyTime,
-			*ts.Status,
-			*ts.StatusID,
+			ts.RevalUpdateFailed,
+			ts.Status,
+			ts.StatusID,
 			*ts.TCPPort,
 			ts.Type,
-			*ts.TypeID,
-			*ts.UpdPending,
+			ts.TypeID,
 			*ts.ConfigUpdateTime,
 			*ts.ConfigApplyTime,
+			ts.ConfigUpdateFailed,
 			*ts.XMPPID,
 			*ts.XMPPPasswd,
 			*ts.StatusLastUpdated,
@@ -354,7 +355,7 @@ func TestGetMidServers(t *testing.T) {
 			srv.Interface.Monitor,
 			srv.Interface.MTU,
 			srv.Interface.Name,
-			*ts.ID,
+			ts.ID,
 			srv.Interface.RouterHostName,
 			srv.Interface.RouterPortName,
 		)
@@ -364,7 +365,7 @@ func TestGetMidServers(t *testing.T) {
 				ip.Address,
 				ip.Gateway,
 				ip.ServiceAddress,
-				*ts.ID,
+				ts.ID,
 				srv.Interface.Name,
 			)
 		}
@@ -380,7 +381,7 @@ func TestGetMidServers(t *testing.T) {
 
 	user := auth.CurrentUser{}
 	version := api.Version{Major: 4, Minor: 0}
-	servers, _, userErr, sysErr, errCode, _ := getServers(nil, v, db.MustBegin(), &user, false, version)
+	servers, _, userErr, sysErr, errCode, _ := getServers(nil, v, db.MustBegin(), &user, false, version, false)
 
 	if userErr != nil || sysErr != nil {
 		t.Errorf("getServers expected: no errors, actual: %v %v with status: %s", userErr, sysErr, http.StatusText(errCode))
@@ -389,9 +390,10 @@ func TestGetMidServers(t *testing.T) {
 	cols2 := []string{"cachegroup", "cachegroup_id", "cdn_id", "cdn_name", "domain_name", "guid", "host_name",
 		"https_port", "id", "ilo_ip_address", "ilo_ip_gateway", "ilo_ip_netmask", "ilo_password", "ilo_username",
 		"last_updated", "mgmt_ip_address", "mgmt_ip_gateway", "mgmt_ip_netmask", "offline_reason", "phys_location",
-		"phys_location_id", "profile_name", "rack", "reval_pending", "revalidate_update_time", "revalidate_apply_time",
-		"status", "status_id", "tcp_port", "server_type", "server_type_id", "upd_pending", "config_update_time",
-		"config_apply_time", "xmpp_id", "xmpp_passwd", "status_last_updated"}
+		"phys_location_id", "profile_name", "rack", "revalidate_update_time", "revalidate_apply_time",
+		"revalidate_update_failed", "status", "status_id", "tcp_port", "server_type", "server_type_id",
+		"config_update_time", "config_apply_time", "config_update_failed", "xmpp_id", "xmpp_passwd",
+		"status_last_updated"}
 	rows2 := sqlmock.NewRows(cols2)
 
 	cgs := []tc.CacheGroup{}
@@ -437,59 +439,56 @@ func TestGetMidServers(t *testing.T) {
 	}
 	cgs = append(cgs, testCG2)
 
-	serverMap := make(map[int]tc.ServerV40, len(servers))
+	serverMap := make(map[int]tc.ServerV5, len(servers))
 	serverIDs := make([]int, 0, len(servers))
 	for _, server := range servers {
-		if server.ID == nil {
-			t.Fatal("Found server with nil ID")
-		}
-		serverIDs = append(serverIDs, *server.ID)
-		serverMap[*server.ID] = server
+		serverIDs = append(serverIDs, server.ID)
+		serverMap[server.ID] = server
 	}
 
-	var ts tc.ServerV40
+	var ts tc.ServerV5
 	for _, s := range servers {
-		if s.HostName != nil && *s.HostName == "server2" {
+		if s.HostName == "server2" {
 			ts = s
 			break
 		}
 	}
-	*ts.ID = *ts.ID + 1
+	ts.ID++
 	rows2 = rows2.AddRow(
-		*ts.Cachegroup,
-		*ts.CachegroupID,
-		*ts.CDNID,
-		*ts.CDNName,
-		*ts.DomainName,
+		ts.CacheGroup,
+		ts.CacheGroupID,
+		ts.CDNID,
+		ts.CDN,
+		ts.DomainName,
 		*ts.GUID,
-		*ts.HostName,
+		ts.HostName,
 		*ts.HTTPSPort,
-		*ts.ID,
+		ts.ID,
 		*ts.ILOIPAddress,
 		*ts.ILOIPGateway,
 		*ts.ILOIPNetmask,
 		*ts.ILOPassword,
 		*ts.ILOUsername,
-		*ts.LastUpdated,
+		ts.LastUpdated,
 		*ts.MgmtIPAddress,
 		*ts.MgmtIPGateway,
 		*ts.MgmtIPNetmask,
 		*ts.OfflineReason,
-		*ts.PhysLocation,
-		*ts.PhysLocationID,
-		fmt.Sprintf("{%s}", strings.Join(ts.ProfileNames, ",")),
+		ts.PhysicalLocation,
+		ts.PhysicalLocationID,
+		fmt.Sprintf("{%s}", strings.Join(ts.Profiles, ",")),
 		*ts.Rack,
-		*ts.RevalPending,
 		*ts.RevalUpdateTime,
 		*ts.RevalApplyTime,
-		*ts.Status,
-		*ts.StatusID,
+		ts.RevalUpdateFailed,
+		ts.Status,
+		ts.StatusID,
 		*ts.TCPPort,
 		ts.Type,
-		*ts.TypeID,
-		*ts.UpdPending,
+		ts.TypeID,
 		*ts.ConfigUpdateTime,
 		*ts.ConfigApplyTime,
+		ts.ConfigUpdateFailed,
 		*ts.XMPPID,
 		*ts.XMPPPasswd,
 		*ts.StatusLastUpdated,
@@ -509,16 +508,12 @@ func TestGetMidServers(t *testing.T) {
 		t.Errorf("getMidServers expected: Type == MID, actual: %v", serverMap[mid[0]].Type)
 	}
 
-	if serverMap[mid[0]].Cachegroup == nil {
-		t.Error("getMidServers expected: Cachegroup == parentCacheGroup, actual: nil")
-	} else if *(serverMap[mid[0]].Cachegroup) != "parentCacheGroup" {
-		t.Errorf("getMidServers expected: Cachegroup == parentCacheGroup, actual: %v", *(serverMap[mid[0]].Cachegroup))
+	if actual := serverMap[mid[0]].CacheGroup; actual != "parentCacheGroup" {
+		t.Errorf("getMidServers expected: Cachegroup == parentCacheGroup, actual: %s", actual)
 	}
 
-	if serverMap[mid[0]].CachegroupID == nil {
-		t.Error("getMidServers expected: CachegroupID == 2, actual: nil")
-	} else if *(serverMap[mid[0]].CachegroupID) != 2 {
-		t.Errorf("getMidServers expected: CachegroupID == 2, actual: %v", *(serverMap[mid[0]].CachegroupID))
+	if actual := serverMap[mid[0]].CacheGroupID; actual != 2 {
+		t.Errorf("getMidServers expected: CachegroupID == 2, actual: %d", actual)
 	}
 }
 
@@ -723,3 +718,387 @@ func (s SortableServers) Swap(i, j int) {
 func (s SortableServers) Less(i, j int) bool {
 	return s[i].HostName < s[j].HostName
 }
+
+func TestUpdateStatusLastUpdatedTime(t *testing.T) {
+	mockDB, mock, err := sqlmock.New()
+	if err != nil {
+		t.Fatalf("an error '%v' was not expected when opening a stub database connection", err)
+	}
+	defer mockDB.Close()
+
+	db := sqlx.NewDb(mockDB, "sqlmock")
+	defer db.Close()
+
+	lastUpdated := time.Now()
+	mock.ExpectBegin()
+	mock.ExpectExec("UPDATE").WithArgs(lastUpdated, 1).WillReturnResult(sqlmock.NewResult(1, 1))
+	mock.ExpectCommit()
+
+	sysErr, _, code := updateStatusLastUpdatedTime(1, &lastUpdated, db.MustBegin().Tx)
+	if sysErr != nil {
+		t.Errorf("unable to update time, system error: %v", sysErr)
+	}
+	if code != http.StatusOK {
+		t.Errorf("updated time failed with status code:%d", code)
+	}
+}
+
+func TestCreateInterfaces(t *testing.T) {
+	mockDB, mock, err := sqlmock.New()
+	if err != nil {
+		t.Fatalf("an error '%v' was not expected when opening a stub database connection", err)
+	}
+	defer mockDB.Close()
+
+	db := sqlx.NewDb(mockDB, "sqlmock")
+	defer db.Close()
+
+	testInterface := getTestServers()[0].Interface
+	var iface []tc.ServerInterfaceInfoV40
+	iface = append(iface, testInterface)
+
+	mock.ExpectBegin()
+	mock.ExpectExec("INSERT INTO interface").
+		WithArgs(iface[0].MaxBandwidth, iface[0].Monitor, iface[0].MTU, iface[0].Name, 1, iface[0].RouterHostName, iface[0].RouterPortName).
+		WillReturnResult(sqlmock.NewResult(1, 1))
+
+	mock.ExpectExec("INSERT INTO ip_address").
+		WithArgs(iface[0].IPAddresses[0].Address, iface[0].IPAddresses[0].Gateway, iface[0].Name, 1, iface[0].IPAddresses[0].ServiceAddress).
+		WillReturnResult(sqlmock.NewResult(1, 1))
+	mock.ExpectCommit()
+
+	usrErr, sysErr, code := createInterfaces(1, iface, db.MustBegin().Tx)
+	if usrErr != nil {
+		t.Errorf("unable to create interface, user error: %v", usrErr)
+	}
+	if sysErr != nil {
+		t.Errorf("unable to create interface, system error: %v", sysErr)
+	}
+	if code != http.StatusOK {
+		t.Errorf("unable to create interface, failed with status code:%d", code)
+	}
+}
+
+func TestDeleteInterfaces(t *testing.T) {
+	mockDB, mock, err := sqlmock.New()
+	if err != nil {
+		t.Fatalf("an error '%v' was not expected when opening a stub database connection", err)
+	}
+	defer mockDB.Close()
+
+	db := sqlx.NewDb(mockDB, "sqlmock")
+	defer db.Close()
+
+	mock.ExpectBegin()
+	mock.ExpectExec("DELETE FROM ip_address").WithArgs(1).WillReturnResult(sqlmock.NewResult(1, 1))
+
+	mock.ExpectExec("DELETE FROM interface").WithArgs(1).WillReturnResult(sqlmock.NewResult(1, 1))
+	mock.ExpectCommit()
+
+	usrErr, sysErr, code := deleteInterfaces(1, db.MustBegin().Tx)
+	if usrErr != nil {
+		t.Errorf("unable to delete interface, user error: %v", usrErr)
+	}
+	if sysErr != nil {
+		t.Errorf("unable to delete interface, system error: %v", sysErr)
+	}
+	if code != http.StatusOK {
+		t.Errorf("unable to delete interface, failed with status code:%d", code)
+	}
+}
+
+func TestInsertServerProfile(t *testing.T) {
+	mockDB, mock, err := sqlmock.New()
+	if err != nil {
+		t.Fatalf("an error '%v' was not expected when opening a stub database connection", err)
+	}
+	defer mockDB.Close()
+
+	db := sqlx.NewDb(mockDB, "sqlmock")
+	defer db.Close()
+
+	profileName := []string{"traffic_ops", "global"}
+	priority := []int{0, 1}
+	mock.ExpectBegin()
+	mock.ExpectExec("INSERT INTO").WithArgs(1, pq.Array(profileName), pq.Array(priority)).WillReturnResult(sqlmock.NewResult(2, 2))
+	mock.ExpectCommit()
+
+	usrErr, sysErr, code := insertServerProfile(1, profileName, db.MustBegin().Tx)
+	if usrErr != nil {
+		t.Errorf("unable to insert profile, user error: %v", usrErr)
+	}
+	if sysErr != nil {
+		t.Errorf("unable to insert profile, system error: %v", sysErr)
+	}
+	if code != http.StatusOK {
+		t.Errorf("unable to insert profile, failed with status code:%d", code)
+	}
+}
+
+func TestCreateServerV4(t *testing.T) {
+	mockDB, mock, err := sqlmock.New()
+	if err != nil {
+		t.Fatalf("an error '%v' was not expected when opening a stub database connection", err)
+	}
+	defer mockDB.Close()
+
+	db := sqlx.NewDb(mockDB, "sqlmock")
+	defer db.Close()
+
+	s4 := getTestServers()[0].Server.Downgrade()
+
+	mock.ExpectBegin()
+	rows0 := sqlmock.NewRows([]string{"id"})
+	rows0.AddRow(1)
+	mock.ExpectQuery("SELECT").WithArgs(s4.ProfileNames[0]).WillReturnRows(rows0)
+
+	rows := sqlmock.NewRows([]string{
+		"cachegroup",
+		"cachegroup_id",
+		"cdn_id",
+		"cdn_name",
+		"domain_name",
+		"guid",
+		"host_name",
+		"https_port",
+		"id",
+		"ilo_ip_address",
+		"ilo_ip_gateway",
+		"ilo_ip_netmask",
+		"ilo_password",
+		"ilo_username",
+		"last_updated",
+		"mgmt_ip_address",
+		"mgmt_ip_gateway",
+		"mgmt_ip_netmask",
+		"offline_reason",
+		"phys_location",
+		"phys_location_id",
+		"profile_name",
+		"rack",
+		"status",
+		"status_id",
+		"tcp_port",
+		"server_type",
+		"server_type_id",
+	})
+	rows.AddRow(
+		*s4.Cachegroup,
+		*s4.CachegroupID,
+		*s4.CDNID,
+		*s4.CDNName,
+		*s4.DomainName,
+		*s4.GUID,
+		*s4.HostName,
+		*s4.HTTPSPort,
+		*s4.ID,
+		*s4.ILOIPAddress,
+		*s4.ILOIPGateway,
+		*s4.ILOIPNetmask,
+		*s4.ILOPassword,
+		*s4.ILOUsername,
+		*s4.LastUpdated,
+		*s4.MgmtIPAddress,
+		*s4.MgmtIPGateway,
+		*s4.MgmtIPNetmask,
+		*s4.OfflineReason,
+		*s4.PhysLocation,
+		*s4.PhysLocationID,
+		fmt.Sprintf("{%s}", strings.Join(s4.ProfileNames, ",")),
+		*s4.Rack,
+		*s4.Status,
+		*s4.StatusID,
+		*s4.TCPPort,
+		s4.Type,
+		*s4.TypeID,
+	)
+	mock.ExpectQuery("INSERT INTO server").
+		WithArgs(*s4.CachegroupID, *s4.CDNID, *s4.DomainName, *s4.HostName, *s4.HTTPSPort, *s4.ILOIPAddress,
+			*s4.ILOIPNetmask, *s4.ILOIPGateway, *s4.ILOUsername, *s4.ILOPassword, *s4.MgmtIPAddress,
+			*s4.MgmtIPNetmask, *s4.MgmtIPGateway, *s4.OfflineReason, *s4.PhysLocationID, 1, *s4.Rack,
+			*s4.StatusID, *s4.TCPPort, *s4.TypeID, *s4.XMPPID, *s4.XMPPPasswd).
+		WillReturnRows(rows)
+	mock.ExpectCommit()
+
+	sid, err := createServerV4(db.MustBegin(), s4)
+	if err != nil {
+		t.Errorf("unable to create v4 server, error: %v", err)
+	}
+	if sid != int64(*s4.ID) {
+		t.Errorf("mismatched server ID, expected: %d, got: %d", *s4.ID, sid)
+	}
+}
+
+func TestCreateServerV3(t *testing.T) {
+	mockDB, mock, err := sqlmock.New()
+	if err != nil {
+		t.Fatalf("an error '%v' was not expected when opening a stub database connection", err)
+	}
+	defer mockDB.Close()
+
+	db := sqlx.NewDb(mockDB, "sqlmock")
+	defer db.Close()
+
+	s3 := tc.ServerV30{
+		CommonServerProperties: tc.CommonServerProperties{
+			Cachegroup:       util.Ptr("mb"),
+			CachegroupID:     util.Ptr(1),
+			CDNID:            util.Ptr(1),
+			CDNName:          util.Ptr("ALL"),
+			DeliveryServices: nil,
+			DomainName:       util.Ptr(""),
+			FQDN:             nil,
+			FqdnTime:         time.Time{},
+			GUID:             util.Ptr(""),
+			HostName:         util.Ptr("test"),
+			HTTPSPort:        util.Ptr(8443),
+			ID:               util.Ptr(1),
+			ILOIPAddress:     util.Ptr(""),
+			ILOIPGateway:     util.Ptr(""),
+			ILOIPNetmask:     util.Ptr(""),
+			ILOPassword:      util.Ptr(""),
+			ILOUsername:      util.Ptr(""),
+			LastUpdated:      util.Ptr(tc.TimeNoMod{}),
+			MgmtIPAddress:    util.Ptr(""),
+			MgmtIPGateway:    util.Ptr(""),
+			MgmtIPNetmask:    util.Ptr(""),
+			OfflineReason:    util.Ptr(""),
+			PhysLocation:     util.Ptr("boulder"),
+			PhysLocationID:   util.Ptr(1),
+			Profile:          util.Ptr("GLOBAL"),
+			ProfileDesc:      nil,
+			ProfileID:        util.Ptr(1),
+			Rack:             util.Ptr(""),
+			RevalPending:     util.Ptr(false),
+			Status:           util.Ptr("ACTIVE"),
+			StatusID:         util.Ptr(1),
+			TCPPort:          util.Ptr(8080),
+			Type:             "EDGE",
+			TypeID:           util.Ptr(2),
+			UpdPending:       util.Ptr(false),
+			XMPPID:           util.Ptr(""),
+			XMPPPasswd:       util.Ptr(""),
+		},
+		RouterHostName:    util.Ptr(""),
+		RouterPortName:    util.Ptr(""),
+		Interfaces:        []tc.ServerInterfaceInfo{},
+		StatusLastUpdated: util.Ptr(time.Time{}),
+	}
+
+	mock.ExpectBegin()
+	rows := sqlmock.NewRows([]string{"id"})
+	rows.AddRow(1)
+	mock.ExpectQuery("INSERT INTO server").
+		WithArgs(*s3.CachegroupID, *s3.CDNID, *s3.DomainName, *s3.HostName, *s3.HTTPSPort, *s3.ILOIPAddress,
+			*s3.ILOIPNetmask, *s3.ILOIPGateway, *s3.ILOUsername, *s3.ILOPassword, *s3.MgmtIPAddress,
+			*s3.MgmtIPNetmask, *s3.MgmtIPGateway, *s3.OfflineReason, *s3.PhysLocationID, *s3.ProfileID,
+			*s3.Rack, *s3.StatusID, *s3.TCPPort, *s3.TypeID, *s3.XMPPID, *s3.XMPPPasswd, *s3.StatusLastUpdated).
+		WillReturnRows(rows)
+	mock.ExpectCommit()
+
+	sid, err := createServerV3(db.MustBegin(), s3)
+	if err != nil {
+		t.Errorf("unable to create v3 server, error: %v", err)
+	}
+	if sid != int64(*s3.ID) {
+		t.Errorf("mismatched server ID, expected: %d, got: %d", *s3.ID, sid)
+	}
+}
+
+func TestUpdateServer(t *testing.T) {
+	mockDB, mock, err := sqlmock.New()
+	if err != nil {
+		t.Fatalf("an error '%v' was not expected when opening a stub database connection", err)
+	}
+	defer mockDB.Close()
+
+	db := sqlx.NewDb(mockDB, "sqlmock")
+	defer db.Close()
+
+	server := getTestServers()[0].Server
+
+	mock.ExpectBegin()
+	rows := sqlmock.NewRows([]string{
+		"cachegroup",
+		"cachegroup_id",
+		"cdn_id",
+		"cdn_name",
+		"domain_name",
+		"guid",
+		"host_name",
+		"https_port",
+		"id",
+		"ilo_ip_address",
+		"ilo_ip_gateway",
+		"ilo_ip_netmask",
+		"ilo_password",
+		"ilo_username",
+		"last_updated",
+		"mgmt_ip_address",
+		"mgmt_ip_gateway",
+		"mgmt_ip_netmask",
+		"offline_reason",
+		"phys_location",
+		"phys_location_id",
+		"profile_name",
+		"rack",
+		"status",
+		"status_id",
+		"tcp_port",
+		"server_type",
+		"server_type_id",
+		"status_last_updated",
+	})
+	rows.AddRow(
+		server.CacheGroup,
+		server.CacheGroupID,
+		server.CDNID,
+		server.CDN,
+		server.DomainName,
+		*server.GUID,
+		server.HostName,
+		*server.HTTPSPort,
+		server.ID,
+		*server.ILOIPAddress,
+		*server.ILOIPGateway,
+		*server.ILOIPNetmask,
+		*server.ILOPassword,
+		*server.ILOUsername,
+		server.LastUpdated,
+		*server.MgmtIPAddress,
+		*server.MgmtIPGateway,
+		*server.MgmtIPNetmask,
+		*server.OfflineReason,
+		server.PhysicalLocation,
+		server.PhysicalLocationID,
+		fmt.Sprintf("{%s}", strings.Join(server.Profiles, ",")),
+		*server.Rack,
+		server.Status,
+		server.StatusID,
+		*server.TCPPort,
+		server.Type,
+		server.TypeID,
+		*server.StatusLastUpdated,
+	)
+	mock.ExpectQuery("UPDATE server SET").
+		WithArgs(server.CacheGroupID, server.CDNID, server.DomainName, server.HostName, *server.HTTPSPort, *server.ILOIPAddress,
+			*server.ILOIPNetmask, *server.ILOIPGateway, *server.ILOUsername, *server.ILOPassword, *server.MgmtIPAddress,
+			*server.MgmtIPNetmask, *server.MgmtIPGateway, *server.OfflineReason, server.PhysicalLocationID, 1, *server.Rack,
+			server.StatusID, *server.TCPPort, server.TypeID, *server.XMPPPasswd, *server.StatusLastUpdated, server.ID).
+		WillReturnRows(rows)
+	mock.ExpectCommit()
+
+	sid, code, usrErr, sysErr := updateServer(db.MustBegin(), server)
+	if usrErr != nil {
+		t.Errorf("unable to update v4 server, user error: %v", usrErr)
+	}
+	if sysErr != nil {
+		t.Errorf("unable to update v4 server, system error: %v", sysErr)
+	}
+	if sid != int64(server.ID) {
+		t.Errorf("updated incorrect server, expected: %d, got: %d", server.ID, sid)
+	}
+	if code != http.StatusOK {
+		t.Errorf("failed to update server with id: %d, expected: %d, got: %d", server.ID, http.StatusOK, code)
+	}
+}
