@@ -64,7 +64,6 @@ var trafficPortal = angular.module('trafficPortal', [
         require('./modules/private/cacheGroups/new').name,
         require('./modules/private/cacheGroups/asns').name,
         require('./modules/private/cacheGroups/servers').name,
-        require('./modules/private/cacheGroups/staticDnsEntries').name,
         require('./modules/private/cacheChecks').name,
         require('./modules/private/cacheStats').name,
         require('./modules/private/capabilities').name,
@@ -73,6 +72,7 @@ var trafficPortal = angular.module('trafficPortal', [
         require('./modules/private/capabilities/users').name,
         require('./modules/private/cdns').name,
         require('./modules/private/cdns/config').name,
+        require("./modules/private/ssl").name,
         require('./modules/private/cdns/deliveryServices').name,
         require('./modules/private/cdns/dnssecKeys').name,
         require('./modules/private/cdns/dnssecKeys/generate').name,
@@ -103,7 +103,6 @@ var trafficPortal = angular.module('trafficPortal', [
         require('./modules/private/deliveryServiceRequests/edit').name,
         require('./modules/private/deliveryServiceRequests/list').name,
         require('./modules/private/deliveryServices').name,
-        require('./modules/private/deliveryServices/capabilities').name,
         require('./modules/private/deliveryServices/clone').name,
         require('./modules/private/deliveryServices/charts').name,
         require('./modules/private/deliveryServices/charts/view').name,
@@ -129,7 +128,6 @@ var trafficPortal = angular.module('trafficPortal', [
         require('./modules/private/deliveryServices/targets/edit').name,
         require('./modules/private/deliveryServices/targets/list').name,
         require('./modules/private/deliveryServices/targets/new').name,
-        require('./modules/private/deliveryServices/users').name,
         require('./modules/private/deliveryServices/urlSigKeys').name,
         require('./modules/private/deliveryServices/uriSigningKeys').name,
         require('./modules/private/deliveryServices/sslKeys').name,
@@ -140,8 +138,6 @@ var trafficPortal = angular.module('trafficPortal', [
         require('./modules/private/divisions/list').name,
         require('./modules/private/divisions/new').name,
         require('./modules/private/divisions/regions').name,
-        require('./modules/private/endpoints').name,
-        require('./modules/private/endpoints/list').name,
         require('./modules/private/iso').name,
         require('./modules/private/jobs').name,
         require('./modules/private/jobs/list').name,
@@ -265,6 +261,7 @@ var trafficPortal = angular.module('trafficPortal', [
         require('./common/modules/locks').name,
         require('./common/modules/message').name,
         require('./common/modules/navigation').name,
+        require("./common/modules/ssl").name,
         require('./common/modules/notifications').name,
         require('./common/modules/release').name,
 
@@ -339,6 +336,7 @@ var trafficPortal = angular.module('trafficPortal', [
         require('./common/modules/form/serviceCategory').name,
         require('./common/modules/form/serviceCategory/edit').name,
         require('./common/modules/form/serviceCategory/new').name,
+        require("./common/modules/form/ssl").name,
         require('./common/modules/form/status').name,
         require('./common/modules/form/status/edit').name,
         require('./common/modules/form/status/new').name,
@@ -362,7 +360,6 @@ var trafficPortal = angular.module('trafficPortal', [
         require('./common/modules/table/cacheGroups').name,
         require('./common/modules/table/cacheGroupAsns').name,
         require('./common/modules/table/cacheGroupServers').name,
-        require('./common/modules/table/cacheGroupStaticDnsEntries').name,
         require('./common/modules/table/capabilities').name,
         require('./common/modules/table/capabilityEndpoints').name,
         require('./common/modules/table/capabilityUsers').name,
@@ -378,7 +375,6 @@ var trafficPortal = angular.module('trafficPortal', [
         require('./common/modules/table/cdniConfigRequests').name,
         require('./common/modules/table/coordinates').name,
         require('./common/modules/table/deliveryServices').name,
-        require('./common/modules/table/deliveryServiceCapabilities').name,
         require('./common/modules/table/deliveryServiceJobs').name,
         require('./common/modules/table/deliveryServiceOrigins').name,
         require('./common/modules/table/deliveryServiceRegexes').name,
@@ -387,10 +383,8 @@ var trafficPortal = angular.module('trafficPortal', [
         require('./common/modules/table/deliveryServiceServers').name,
         require('./common/modules/table/deliveryServiceStaticDnsEntries').name,
         require('./common/modules/table/deliveryServiceTargets').name,
-        require('./common/modules/table/deliveryServiceUsers').name,
         require('./common/modules/table/divisions').name,
         require('./common/modules/table/divisionRegions').name,
-        require('./common/modules/table/endpoints').name,
         require('./common/modules/table/federationResolvers').name,
         require('./common/modules/table/jobs').name,
         require('./common/modules/table/notifications').name,
