@@ -17,7 +17,25 @@
  * under the License.
  */
 
-var TableDeliveryServicesRequestsController = function (tableName, dsRequests, $scope, $state, $uibModal, $anchorScroll, $q, $location, $document, dateUtils, locationUtils, typeService, deliveryServiceService, deliveryServiceRequestService, messageModel, propertiesModel, userModel) {
+/**
+ * The controller for the table that lists Delivery Service Requests.
+ *
+ * @param {string} tableName
+ * @param {import("../../../api/DeliveryServiceRequestService").DeliveryServiceRequest[]} dsRequests
+ * @param {*} $scope
+ * @param {*} $state
+ * @param {import("../../../service/utils/angular.ui.bootstrap").IModalService} $uibModal
+ * @param {import("angular").IAnchorScrollService} $anchorScroll
+ * @param {import("angular").IDocumentService} $document
+ * @param {import("../../../service/utils/DateUtils")} dateUtils
+ * @param {import("../../../service/utils/LocationUtils")} locationUtils
+ * @param {import("../../../api/TypeService")} typeService
+ * @param {import("../../../api/DeliveryServiceRequestService")} deliveryServiceRequestService
+ * @param {import("../../../models/MessageModel")} messageModel
+ * @param {import("../../../models/PropertiesModel")} propertiesModel
+ * @param {import("../../../models/UserModel")} userModel
+ */
+var TableDeliveryServicesRequestsController = function (tableName, dsRequests, $scope, $state, $uibModal, $anchorScroll, $document, dateUtils, locationUtils, typeService, deliveryServiceRequestService, messageModel, propertiesModel, userModel) {
 
 	/**
 	 * Gets value to display a default tooltip.
@@ -33,7 +51,7 @@ var TableDeliveryServicesRequestsController = function (tableName, dsRequests, $
 		return params.value ? dateUtils.getRelativeTime(params.value) : params.value;
 	}
 
-	columns = [
+	const columns = [
 		{
 			headerName: "Delivery Service",
 			valueGetter: function(params) {
@@ -124,7 +142,11 @@ var TableDeliveryServicesRequestsController = function (tableName, dsRequests, $
 	/** All of the ds requests - createdAt and lastUpdated fields converted to actual Date */
 	$scope.dsRequests = dsRequests.map(
 		function(x) {
+			// The right way to do these is with an interceptor, but nobody wants
+			// to put in that kind of effort for a legacy product.
+			// @ts-ignore
 			x.createdAt = x.createdAt ? new Date(x.createdAt.replace("+00", "Z")) : x.createdAt;
+			// @ts-ignore
 			x.lastUpdated = x.lastUpdated ? new Date(x.lastUpdated.replace("+00", "Z")) : x.lastUpdated;
 		});
 
@@ -137,10 +159,10 @@ var TableDeliveryServicesRequestsController = function (tableName, dsRequests, $
 	/** Options, configuration, data and callbacks for the ag-grid table. */
 	$scope.gridOptions = {
 		onCellMouseDown: function() {
-			$scope.mouseDownSelectionText = window.getSelection().toString();
+			$scope.mouseDownSelectionText = String(window.getSelection());
 		},
 		onRowClicked: function(params) {
-			const selection = window.getSelection().toString();
+			const selection = String(window.getSelection());
 			if(selection === "" || selection === $scope.mouseDownSelectionText) {
 				const typeId = (params.data.requested) ? params.data.requested.typeId : params.data.original.typeId;
 
@@ -200,7 +222,11 @@ var TableDeliveryServicesRequestsController = function (tableName, dsRequests, $
 			$scope.menuStyle.bottom = "unset";
 			$scope.menuStyle.right = "unset";
 			$scope.$apply();
-			const boundingRect = document.getElementById("context-menu").getBoundingClientRect();
+			const ctxMenu = document.getElementById("context-menu");
+			if (!ctxMenu) {
+				throw new Error("context-menu ID not found in the DOM")
+			}
+			const boundingRect = ctxMenu.getBoundingClientRect();
 
 			if (boundingRect.bottom > window.innerHeight){
 				$scope.menuStyle.bottom = String(window.innerHeight - params.event.clientY) + "px";
@@ -229,7 +255,7 @@ var TableDeliveryServicesRequestsController = function (tableName, dsRequests, $
 		},
         onFirstDataRendered: function() {
 			try {
-				const filterState = JSON.parse(localStorage.getItem(tableName + "_table_filters")) || {};
+				const filterState = JSON.parse(localStorage.getItem(tableName + "_table_filters") ?? "null") || {};
 				$scope.gridOptions.api.setFilterModel(filterState);
 			} catch (e) {
 				console.error("Failure to load stored filter state:", e);
@@ -242,7 +268,7 @@ var TableDeliveryServicesRequestsController = function (tableName, dsRequests, $
 		onGridReady: function() {
 			try {
 				// need to create the show/hide column checkboxes and bind to the current visibility
-				const colstates = JSON.parse(localStorage.getItem(tableName + "_table_columns"));
+				const colstates = JSON.parse(localStorage.getItem(tableName + "_table_columns") ?? "null");
 				if (colstates) {
 					if (!$scope.gridOptions.columnApi.setColumnState(colstates)) {
 						console.error("Failed to load stored column state: one or more columns not found");
@@ -255,7 +281,7 @@ var TableDeliveryServicesRequestsController = function (tableName, dsRequests, $
 			}
 
 			try {
-				const sortState = JSON.parse(localStorage.getItem(tableName + "_table_sort"));
+				const sortState = JSON.parse(localStorage.getItem(tableName + "_table_sort") ?? "null");
 				$scope.gridOptions.api.setSortModel(sortState);
 			} catch (e) {
 				console.error("Failure to load stored sort state:", e);
@@ -269,17 +295,17 @@ var TableDeliveryServicesRequestsController = function (tableName, dsRequests, $
 			}
 
 			try {
-				const ps = localStorage.getItem(tableName + "_page_size");
+				const ps = Number(localStorage.getItem(tableName + "_page_size"));
 				if (ps && ps > 0) {
-					$scope.pageSize = Number(ps);
+					$scope.pageSize = ps;
 					$scope.gridOptions.api.paginationSetPageSize($scope.pageSize);
 				}
 			} catch (e) {
 				console.error("Failure to load stored page size:", e);
 			}
-			
+
 			try {
-				const page = parseInt(localStorage.getItem(tableName + "_table_page"));
+				const page = parseInt(localStorage.getItem(tableName + "_table_page") ?? "");
 				const totalPages = $scope.gridOptions.api.paginationGetTotalPages();
 				if (page !== undefined && page > 0 && page <= totalPages-1) {
 					$scope.gridOptions.api.paginationGoToPage(page);
@@ -287,7 +313,7 @@ var TableDeliveryServicesRequestsController = function (tableName, dsRequests, $
 			} catch (e) {
 				console.error("Failed to load stored page number:", e);
 			}
-			
+
 			$scope.gridOptions.api.addEventListener("paginationChanged", function() {
 				localStorage.setItem(tableName + "_table_page", $scope.gridOptions.api.paginationGetCurrentPage());
 			});
@@ -342,7 +368,7 @@ var TableDeliveryServicesRequestsController = function (tableName, dsRequests, $
 	$scope.onPageSizeChanged = function() {
 		const value = Number($scope.pageSize);
 		$scope.gridOptions.api.paginationSetPageSize(value);
-		localStorage.setItem(tableName + "_page_size", value);
+		localStorage.setItem(tableName + "_page_size", String(value));
 	};
 
 	$scope.clearTableFilters = function() {
@@ -372,13 +398,13 @@ var TableDeliveryServicesRequestsController = function (tableName, dsRequests, $
 	$scope.assignRequest = function (request, assign, $event) {
 		$event.stopPropagation(); // this kills the click event so it doesn't trigger anything else
 		var params = {
-			title: 'Assign Delivery Service Request',
-			message: (assign) ? 'Are you sure you want to assign this delivery service request to yourself?' : 'Are you sure you want to unassign this delivery service request?'
+			title: "Assign Delivery Service Request",
+			message: (assign) ? "Are you sure you want to assign this delivery service request to yourself?" : "Are you sure you want to unassign this delivery service request?"
 		};
 		var modalInstance = $uibModal.open({
-			templateUrl: 'common/modules/dialog/confirm/dialog.confirm.tpl.html',
-			controller: 'DialogConfirmController',
-			size: 'md',
+			templateUrl: "common/modules/dialog/confirm/dialog.confirm.tpl.html",
+			controller: "DialogConfirmController",
+			size: "md",
 			resolve: {
 				params: function () {
 					return params;
@@ -390,9 +416,9 @@ var TableDeliveryServicesRequestsController = function (tableName, dsRequests, $
 			deliveryServiceRequestService.assignDeliveryServiceRequest(request.id, assignee).then(function () {
 				$scope.refresh();
 				if (assign) {
-					messageModel.setMessages([ { level: 'success', text: 'Delivery service request was assigned' } ], false);
+					messageModel.setMessages([ { level: "success", text: "Delivery service request was assigned" } ], false);
 				} else {
-					messageModel.setMessages([ { level: 'success', text: 'Delivery service request was unassigned' } ], false);
+					messageModel.setMessages([ { level: "success", text: "Delivery service request was unassigned" } ], false);
 				}
 			});
 		}, function () {
@@ -400,37 +426,37 @@ var TableDeliveryServicesRequestsController = function (tableName, dsRequests, $
 		});
 	};
 
-	$scope.editStatus = function (request, $event) {
+	/**
+	 * @param {import("../../../api/DeliveryServiceRequestService").DeliveryServiceRequest & {id: number}} request
+	 * @param {Event} $event
+	 */
+	$scope.editStatus = async function (request, $event) {
 		$event.stopPropagation(); // this kills the click event so it doesn't trigger anything else
-		var params = {
+		const params = {
 			title: "Edit Delivery Service Request Status",
-			message: 'Please select the appropriate status for this request.'
+			message: "Please select the appropriate status for this request."
 		};
-		var modalInstance = $uibModal.open({
-			templateUrl: 'common/modules/dialog/select/dialog.select.tpl.html',
-			controller: 'DialogSelectController',
-			size: 'md',
+		const modalInstance = $uibModal.open({
+			templateUrl: "common/modules/dialog/select/dialog.select.tpl.html",
+			controller: "DialogSelectController",
+			size: "md",
 			resolve: {
-				params: function () {
-					return params;
-				},
-				collection: function () {
-					return [
-						{id: $scope.DRAFT, name: 'Save as Draft'},
-						{id: $scope.SUBMITTED, name: 'Submit for Review / Deployment'}
-					];
-				}
+				params,
+				collection: () => [
+					{id: $scope.DRAFT, name: "Save as Draft"},
+					{id: $scope.SUBMITTED, name: "Submit for Review / Deployment"}
+				]
 			}
 		});
-		modalInstance.result.then(function (action) {
-			var status = (action.id == $scope.DRAFT) ? 'draft' : 'submitted';
-			deliveryServiceRequestService.updateDeliveryServiceRequestStatus(request.id, status).then(function () {
-				$scope.refresh();
-				messageModel.setMessages([ { level: 'success', text: 'Delivery service request status was updated' } ], false);
-			});
-		}, function () {
+		try {
+			const action = await modalInstance.result;
+			const status = (action.id == $scope.DRAFT) ? "draft" : "submitted";
+			await deliveryServiceRequestService.updateDeliveryServiceRequestStatus(request.id, status);
+			$scope.refresh();
+			messageModel.setMessages([ { level: "success", text: "Delivery service request status was updated" } ], false);
+		} catch {
 			// do nothing
-		});
+		}
 	};
 
 	$scope.rejectRequest = function (request, $event) {
@@ -567,5 +593,5 @@ var TableDeliveryServicesRequestsController = function (tableName, dsRequests, $
 
 };
 
-TableDeliveryServicesRequestsController.$inject = ['tableName', 'dsRequests', '$scope', '$state', '$uibModal', '$anchorScroll', '$q', '$location', '$document', 'dateUtils', 'locationUtils', 'typeService', 'deliveryServiceService', 'deliveryServiceRequestService', 'messageModel', 'propertiesModel', 'userModel'];
+TableDeliveryServicesRequestsController.$inject = ['tableName', 'dsRequests', '$scope', '$state', '$uibModal', '$anchorScroll', '$document', 'dateUtils', 'locationUtils', 'typeService', 'deliveryServiceRequestService', 'messageModel', 'propertiesModel', 'userModel'];
 module.exports = TableDeliveryServicesRequestsController;
