@@ -17,7 +17,22 @@
  * under the License.
  */
 
-function TableDeliveryServicesController(tableName, deliveryServices, $anchorScroll, $scope, $state, $location, $uibModal, deliveryServiceService, deliveryServiceRequestService, deliveryServiceUtils, locationUtils, messageModel, propertiesModel, userModel) {
+/**
+ * This is the controller for (almost) all tables of Delivery Services.
+ *
+ * @param {string} tableName
+ * @param {import("../../../api/DeliveryServiceService").DeliveryService[]} deliveryServices
+ * @param {{deliveryService: string; targets: {deliveryService: string}[]}[]} steeringTargets
+ * @param {import("angular").IAnchorScrollService} $anchorScroll
+ * @param {*} $scope
+ * @param {*} $state
+ * @param {import("angular").ILocationService} $location
+ * @param {import("../../../service/utils/DeliveryServiceUtils")} deliveryServiceUtils
+ * @param {import("../../../models/MessageModel")} messageModel
+ * @param {import("../../../models/PropertiesModel")} propertiesModel
+ * @param {import("../../../models/UserModel")} userModel
+ */
+function TableDeliveryServicesController(tableName, deliveryServices, steeringTargets, $anchorScroll, $scope, $state, $location, $uibModal, deliveryServiceService, deliveryServiceRequestService, deliveryServiceUtils, locationUtils, messageModel, propertiesModel, userModel) {
 	$scope.tableName = tableName;
 
 	/** The columns of the ag-grid table */
@@ -25,7 +40,13 @@ function TableDeliveryServicesController(tableName, deliveryServices, $anchorScr
 		{
 			headerName: "Active",
 			field: "active",
-			hide: false
+			hide: false,
+			valueGetter: ({data}) => {
+				if (propertiesModel.properties.deliveryServices?.exposeInactive || data.active === "ACTIVE") {
+					return data.active;
+				}
+				return "INACTIVE";
+			}
 		},
 		{
 			headerName: "Anonymous Blocking",
@@ -122,7 +143,7 @@ function TableDeliveryServicesController(tableName, deliveryServices, $anchorScr
 			headerName: "Geo Limit",
 			field: "geoLimit",
 			hide: true,
-			valueFormatter: params => deliveryServiceUtils.geoLimits[params.data.geoLimit],
+			valueGetter: params => deliveryServiceUtils.geoLimits[params.data.geoLimit],
 			tooltipValueGetter: params => deliveryServiceUtils.geoLimits[params.data.geoLimit]
 		},
 		{
@@ -139,7 +160,7 @@ function TableDeliveryServicesController(tableName, deliveryServices, $anchorScr
 			headerName: "Geolocation Provider",
 			field: "geoProvider",
 			hide: true,
-			valueFormatter: params => deliveryServiceUtils.geoProviders[params.data.geoProvider],
+			valueGetter: params => deliveryServiceUtils.geoProviders[params.data.geoProvider],
 			tooltipValueGetter: params => deliveryServiceUtils.geoProviders[params.data.geoProvider]
 		},
 		{
@@ -261,21 +282,21 @@ function TableDeliveryServicesController(tableName, deliveryServices, $anchorScr
 			headerName: "Protocol",
 			field: "protocol",
 			hide: false,
-			valueFormatter: params => deliveryServiceUtils.protocols[params.data.protocol],
+			valueGetter: params => deliveryServiceUtils.protocols[params.data.protocol],
 			tooltipValueGetter: params => deliveryServiceUtils.protocols[params.data.protocol]
 		},
 		{
 			headerName: "Qstring Handling",
 			field: "qstringIgnore",
 			hide: true,
-			valueFormatter: params => deliveryServiceUtils.qstrings[params.data.qstringIgnore],
+			valueGetter: params => deliveryServiceUtils.qstrings[params.data.qstringIgnore],
 			tooltipValueGetter: params => deliveryServiceUtils.qstrings[params.data.qstringIgnore]
 		},
 		{
 			headerName: "Range Request Handling",
 			field: "rangeRequestHandling",
 			hide: true,
-			valueFormatter: params => deliveryServiceUtils.rrhs[params.data.rangeRequestHandling],
+			valueGetter: params => deliveryServiceUtils.rrhs[params.data.rangeRequestHandling],
 			tooltipValueGetter: params => deliveryServiceUtils.rrhs[params.data.rangeRequestHandling]
 		},
 		{
@@ -283,6 +304,11 @@ function TableDeliveryServicesController(tableName, deliveryServices, $anchorScr
 			field: "regexRemap",
 			hide: true
 		},
+		{
+			headerName: "Regional Max Origin Connections",
+			field: "regional",
+			hide: true
+		},
 		{
 			headerName: "Regional Geoblocking",
 			field: "regionalGeoBlocking",
@@ -293,6 +319,11 @@ function TableDeliveryServicesController(tableName, deliveryServices, $anchorScr
 			field: "remapText",
 			hide: true
 		},
+		{
+			headerName: "Required Capability(ies)",
+			field: "requiredCapabilities",
+			hide: true
+		},
 		{
 			headerName: "Routing Name",
 			field: "routingName",
@@ -319,6 +350,13 @@ function TableDeliveryServicesController(tableName, deliveryServices, $anchorScr
 			hide: true,
 			filter: "agNumberColumnFilter"
 		},
+		{
+			headerName: "Target For",
+			field: "isTargetFor",
+			hide: true,
+			valueGetter: params => params.data.isTargetsFor,
+			tooltipValueGetter: params => `Steering targets for: ${params.data.isTargetsFor.join(", ")}`
+		},
 		{
 			headerName: "Tenant",
 			field: "tenant",
@@ -351,16 +389,16 @@ function TableDeliveryServicesController(tableName, deliveryServices, $anchorScr
 		}
 	];
 
-	let dsRequestsEnabled = propertiesModel.properties.dsRequests.enabled;
+	let dsRequestsEnabled = propertiesModel.properties?.dsRequests?.enabled;
 
-	let showCustomCharts = propertiesModel.properties.deliveryServices.charts.customLink.show;
+	let showCustomCharts = propertiesModel.properties.deliveryServices?.charts.customLink.show;
 
 	/**
 	 * @param {string} typeName
 	 */
 	function createDeliveryService(typeName) {
 		locationUtils.navigateToPath(`/delivery-services/new?dsType=${typeName}`);
-	};
+	}
 
 	/**
 	 * Opens a dialog that the user uses to clone the given Delivery Service.
@@ -370,7 +408,7 @@ function TableDeliveryServicesController(tableName, deliveryServices, $anchorScr
 	async function clone(ds) {
 		const params = {
 			title: `Clone Delivery Service: ${ds.xmlId}`,
-			message: "Please select a content routing category for the clone"
+			message: "Please select a <a href='https://traffic-control-cdn.readthedocs.io/en/latest/overview/delivery_services.html#ds-types' target='_blank'>content routing category</a> for the clone"
 		};
 
 		const modalInstance = $uibModal.open({
@@ -392,13 +430,13 @@ function TableDeliveryServicesController(tableName, deliveryServices, $anchorScr
 
 		const {name} = await modalInstance.result;
 		locationUtils.navigateToPath(`/delivery-services/${ds.id}/clone?dsType=${name}`);
-	};
+	}
 
 	/**
 	 * Opens a dialog asking the user for confirmation before deleting the given
 	 * Delivery Service.
 	 *
-	 * @param {{readonly xmlId: string;}} ds
+	 * @param {import("../../../api/DeliveryServiceService").DeliveryService} ds
 	 */
 	async function confirmDelete(ds) {
 		const params = {
@@ -426,12 +464,12 @@ function TableDeliveryServicesController(tableName, deliveryServices, $anchorScr
 				messageModel.setMessages(fault.data.alerts, false);
 			}
 		} catch {}
-	};
+	}
 
 	/**
 	 * Creates a new DSR to delete the given Delivery Service.
 	 *
-	 * @param {{readonly xmlId: string}} ds
+	 * @param {import("../../../api/DeliveryServiceService").DeliveryService} ds
 	 */
 	async function createDeliveryServiceDeleteRequest(ds) {
 		const params = {
@@ -449,7 +487,7 @@ function TableDeliveryServicesController(tableName, deliveryServices, $anchorScr
 						{ id: $scope.DRAFT, name: "Save Request as Draft" },
 						{ id: $scope.SUBMITTED, name: "Submit Request for Review and Deployment" }
 					];
-					if (userModel.user.role == propertiesModel.properties.dsRequests.overrideRole) {
+					if (userModel.user.role === propertiesModel.properties?.dsRequests?.overrideRole) {
 						statuses.push({ id: $scope.COMPLETE, name: "Fulfill Request Immediately" });
 					}
 					return statuses;
@@ -460,7 +498,7 @@ function TableDeliveryServicesController(tableName, deliveryServices, $anchorScr
 		let status = 'draft';
 		if (options.status.id == $scope.SUBMITTED || options.status.id == $scope.COMPLETE) {
 			status = 'submitted';
-		};
+		}
 
 		const dsRequest = {
 			changeType: 'delete',
@@ -506,10 +544,6 @@ function TableDeliveryServicesController(tableName, deliveryServices, $anchorScr
 		}
 	}
 
-	/** All of the delivery services - lastUpdated fields converted to actual Dates */
-	$scope.deliveryServices = deliveryServices.map(
-		x => ({...x, lastUpdated: x.lastUpdated ? new Date(x.lastUpdated.replace("+00", "Z").replace(" ", "T")) : x.lastUpdated})
-	);
 
 	$scope.DRAFT = 0;
 	$scope.SUBMITTED = 1;
@@ -518,7 +552,8 @@ function TableDeliveryServicesController(tableName, deliveryServices, $anchorScr
 	$scope.COMPLETE = 4;
 
 	/**
-	 * @param {{readonly id: number; type: string}} ds
+	 * @deprecated This should instead just be an ng-href.
+	 * @param {{readonly id: number; type: string; xmlId: string}} ds
 	 */
 	function viewCharts(ds) {
 		if (showCustomCharts) {
@@ -526,7 +561,7 @@ function TableDeliveryServicesController(tableName, deliveryServices, $anchorScr
 		} else {
 			locationUtils.navigateToPath(`/delivery-services/${ds.id}/charts?dsType=${ds.type}`);
 		}
-	};
+	}
 
 	$scope.refresh = function() {
 		$state.reload(); // reloads all the resolves for the view
@@ -535,7 +570,7 @@ function TableDeliveryServicesController(tableName, deliveryServices, $anchorScr
 	async function selectDSType() {
 		const params = {
 			title: "Create Delivery Service",
-			message: "Please select a content routing category"
+			message: "Please select a <a href='https://traffic-control-cdn.readthedocs.io/en/latest/overview/delivery_services.html#ds-types' target='_blank'>content routing category</a>"
 		};
 		const modalInstance = $uibModal.open({
 			templateUrl: 'common/modules/dialog/select/dialog.select.tpl.html',
@@ -559,7 +594,19 @@ function TableDeliveryServicesController(tableName, deliveryServices, $anchorScr
 		} catch {
 			// do nothing
 		}
-	};
+	}
+
+	this.$onInit = function() {
+		const xmlIds = [];
+		for(const ds of deliveryServices) {
+			xmlIds.push(ds.xmlId);
+		}
+		const dsTargets = deliveryServiceUtils.getSteeringTargetsForDS(xmlIds, steeringTargets);
+		/** All the delivery services - lastUpdated fields converted to actual Dates */
+		$scope.deliveryServices = deliveryServices.map(
+			x => ({...x, isTargetsFor: Array.from(dsTargets[x.xmlId]), lastUpdated: x.lastUpdated ? new Date(x.lastUpdated.replace("+00", "Z").replace(" ", "T")) : x.lastUpdated})
+		);
+	}
 
 	async function compareDSs() {
 		const params = {
@@ -582,7 +629,7 @@ function TableDeliveryServicesController(tableName, deliveryServices, $anchorScr
 		} catch {
 			// do nothing
 		}
-	};
+	}
 
 	$scope.options = {
 		onRowClick: params => {
@@ -610,14 +657,14 @@ function TableDeliveryServicesController(tableName, deliveryServices, $anchorScr
 
 	$scope.contextMenuOptions = [
 		{
-			getHref: ds => `/delivery-services/${ds.id}?dsType=${ds.type}`,
+			getHref: ds => `#!/delivery-services/${ds.id}?dsType=${ds.type}`,
 			getText: ds => `Open ${ds.xmlId} in a new tab`,
 			newTab: true,
 			type: 2
 		},
 		{type: 0},
 		{
-			getHref: ds => `/delivery-services/${ds.id}?dsType=${ds.type}`,
+			getHref: ds => `#!/delivery-services/${ds.id}?dsType=${ds.type}`,
 			text: "Edit",
 			type: 2
 		},
@@ -669,11 +716,6 @@ function TableDeliveryServicesController(tableName, deliveryServices, $anchorScr
 			text: "Manage Regexes",
 			type: 2
 		},
-		{
-			getHref: ds => `#!/delivery-services/${ds.id}/required-server-capabilities?dsType=${ds.type}`,
-			text: "Manage Required Server Capabilities",
-			type: 2
-		},
 		{
 			getHref: ds => `#!/delivery-services/${ds.id}/servers?dsType=${ds.type}`,
 			text: "Manage Servers",
@@ -691,7 +733,7 @@ function TableDeliveryServicesController(tableName, deliveryServices, $anchorScr
 		},
 
 	];
-};
+}
 
-TableDeliveryServicesController.$inject = ["tableName", "deliveryServices", "$anchorScroll", "$scope", "$state", "$location", "$uibModal", "deliveryServiceService", "deliveryServiceRequestService", "deliveryServiceUtils", "locationUtils", "messageModel", "propertiesModel", "userModel"];
+TableDeliveryServicesController.$inject = ["tableName", "deliveryServices", "steeringTargets", "$anchorScroll", "$scope", "$state", "$location", "$uibModal", "deliveryServiceService", "deliveryServiceRequestService", "deliveryServiceUtils", "locationUtils", "messageModel", "propertiesModel", "userModel"];
 module.exports = TableDeliveryServicesController;
