@@ -46,6 +46,7 @@ import java.util.concurrent.ScheduledExecutorService;
 import java.util.concurrent.TimeUnit;
 import java.util.stream.Collectors;
 
+import com.google.common.collect.Iterators;
 import org.apache.traffic_control.traffic_router.core.edge.Node.IPVersions;
 import org.apache.traffic_control.traffic_router.core.util.JsonUtils;
 import org.apache.traffic_control.traffic_router.core.util.JsonUtilsException;
@@ -103,7 +104,6 @@ public class ZoneManager extends Resolver {
 	private final TrafficRouter trafficRouter;
 	private static LoadingCache<ZoneKey, Zone> dynamicZoneCache = null;
 	private static LoadingCache<ZoneKey, Zone> zoneCache = null;
-	private static ConcurrentMap<String, ZoneKey> domainsToZoneKeys = new ConcurrentHashMap<>();
 	private static ScheduledExecutorService zoneMaintenanceExecutor = null;
 	private static ExecutorService zoneExecutor = null;
 	private static final int DEFAULT_PRIMER_LIMIT = 500;
@@ -177,7 +177,7 @@ public class ZoneManager extends Resolver {
 
 			final ConcurrentMap<String, ZoneKey> newDomainsToZoneKeys = new ConcurrentHashMap<>();
 
-			if (tr.isDnssecZoneDiffingEnabled()) {
+			if (tr.isDnssecEnabled()) {
 				if (ZoneManager.dynamicZoneCache == null || ZoneManager.zoneCache == null) {
 					initZoneDirectory();
 				} else {
@@ -219,8 +219,6 @@ public class ZoneManager extends Resolver {
 				LOGGER.info("old static zone cache size: " + oldZCSize + ", new static zone cache size: " + zc.size() +
 						", old dynamic zone cache size: " + oldDCZSize + ", new dynamic zone cache size: " + dzc.size());
 
-				ZoneManager.domainsToZoneKeys = newDomainsToZoneKeys;
-
 				if (tze != null) {
 					tze.shutdownNow();
 				}
@@ -441,7 +439,7 @@ public class ZoneManager extends Resolver {
 
 	private static List<Record> fillZones(final Map<String, List<Record>> zoneMap, final Map<String, DeliveryService> dsMap, final TrafficRouter tr, final List<Record> superRecords, final LoadingCache<ZoneKey, Zone> zc, final LoadingCache<ZoneKey, Zone> dzc, final List<Runnable> generationTasks, final BlockingQueue<Runnable> primingTasks, final ConcurrentMap<String, ZoneKey> newDomainsToZoneKeys)
 			throws IOException {
-		final String hostname = InetAddress.getLocalHost().getHostName().replaceAll("\\..*", "");
+		final String hostname = getTRLocalHostname(tr);
 
 		final List<Record> records = new ArrayList<Record>();
 
@@ -456,6 +454,17 @@ public class ZoneManager extends Resolver {
 		return records;
 	}
 
+	protected static String getTRLocalHostname(final TrafficRouter tr) throws UnknownHostException {
+		// if there is only one TR in the CRConfig, just use that TR's hostname
+		// instead of checking for the local server's hostname
+		final boolean singleTR = Iterators.size(tr.getCacheRegister().getTrafficRouters().fieldNames()) == 1;
+		if (singleTR) {
+			return tr.getCacheRegister().getTrafficRouters().fieldNames().next();
+		} else {
+			return InetAddress.getLocalHost().getHostName().replaceAll("\\..*", "");
+		}
+	}
+
 	@SuppressWarnings({"PMD.ExcessiveParameterList"})
 	private static List<Record> createZone(final String domain, final Map<String, List<Record>> zoneMap, final Map<String, DeliveryService> dsMap,
 			final TrafficRouter tr, final LoadingCache<ZoneKey, Zone> zc, final LoadingCache<ZoneKey, Zone> dzc, final List<Runnable> generationTasks,
@@ -511,23 +520,8 @@ public class ZoneManager extends Resolver {
 		generationTasks.add(() -> {
 			try {
 				final ZoneKey newZoneKey = signatureManager.generateZoneKey(name, list);
-				if (tr.isDnssecZoneDiffingEnabled() && domainsToZoneKeys.containsKey(domain)) {
-					final ZoneKey oldZoneKey = domainsToZoneKeys.get(domain);
-					if (zonesAreEqual(newZoneKey.getRecords(), oldZoneKey.getRecords())) {
-						final Zone oldZone = ZoneManager.zoneCache.getIfPresent(oldZoneKey);
-						if (oldZone != null) {
-							LOGGER.info("found matching ZoneKey for " + domain + " - copying from current Zone cache into new Zone cache - no re-signing necessary");
-							zc.put(oldZoneKey, oldZone);
-							newDomainsToZoneKeys.put(domain, oldZoneKey);
-							return;
-						}
-						LOGGER.warn("found matching ZoneKey for " + domain + " but the Zone was not found in the Zone cache");
-					} else {
-						LOGGER.info("new zone for " + domain + " is not equal to the old zone - re-signing necessary");
-					}
-				}
 				final Zone zone = zc.get(newZoneKey); // cause the zone to be loaded into the new cache
-				if (tr.isDnssecZoneDiffingEnabled()) {
+				if (tr.isDnssecEnabled()) {
 					newDomainsToZoneKeys.put(domain, newZoneKey);
 				}
 				final CacheRegister data = tr.getCacheRegister();
