@@ -37,10 +37,10 @@ import (
 	"syscall"
 	"time"
 
-	"github.com/apache/trafficcontrol/lib/go-log"
-	"github.com/apache/trafficcontrol/lib/go-tc"
-	"github.com/apache/trafficcontrol/lib/go-util"
-	client "github.com/apache/trafficcontrol/traffic_ops/v3-client"
+	"github.com/apache/trafficcontrol/v8/lib/go-log"
+	"github.com/apache/trafficcontrol/v8/lib/go-tc"
+	"github.com/apache/trafficcontrol/v8/lib/go-util"
+	client "github.com/apache/trafficcontrol/v8/traffic_ops/v5-client"
 
 	"github.com/Shopify/sarama"
 	"github.com/cihub/seelog"
@@ -48,9 +48,9 @@ import (
 )
 
 const UserAgent = "traffic-stats"
-const TrafficOpsRequestTimeout = time.Second * time.Duration(10)
 
 const (
+	defaultTrafficOpsRequestTimeout    = 10
 	defaultPollingInterval             = 10
 	defaultDailySummaryPollingInterval = 60
 	defaultConfigInterval              = 300
@@ -95,6 +95,7 @@ type StartupConfig struct {
 	ToUser                      string   `json:"toUser"`
 	ToPasswd                    string   `json:"toPasswd"`
 	ToURL                       string   `json:"toUrl"`
+	ToRequestTimeoutSeconds     int      `json:"toRequestTimeout"`
 	DisableInflux               bool     `json:"disableInflux"`
 	InfluxUser                  string   `json:"influxUser"`
 	InfluxPassword              string   `json:"influxPassword"`
@@ -162,17 +163,17 @@ var useSeelog bool = true
 // about caches, cachegroups, and health urls
 type RunningConfig struct {
 	HealthUrls      map[string]map[string][]string // the 1st map key is CDN_name, the second is DsStats or CacheStats
-	CacheMap        map[string]tc.Server           // map hostName to cache
+	CacheMap        map[string]tc.ServerV5         // map hostName to cache
 	LastSummaryTime time.Time
 }
 
-//InfluxDBProps contains URL and connection information for InfluxDB servers
+// InfluxDBProps contains URL and connection information for InfluxDB servers
 type InfluxDBProps struct {
 	URL          string
 	InfluxClient influx.Client
 }
 
-//Timers struct contains all the timers
+// Timers struct contains all the timers
 type Timers struct {
 	Poll         <-chan time.Time
 	DailySummary <-chan time.Time
@@ -516,6 +517,10 @@ func loadStartupConfig(configFile string, oldConfig StartupConfig) (StartupConfi
 		config.MaxPublishSize = defaultMaxPublishSize
 	}
 
+	if config.ToRequestTimeoutSeconds <= 0 {
+		config.ToRequestTimeoutSeconds = defaultTrafficOpsRequestTimeout
+	}
+
 	if config.LogConfig != nil {
 		if err = log.InitCfg(config.LogConfig); err != nil {
 			return config, fmt.Errorf("initializing logging configuration: %w", err)
@@ -617,7 +622,7 @@ func calcDailyMaxGbps(client influx.Client, bp influx.BatchPoints, startTime tim
 					value = value / kilobitsToGigabits
 					statTime, _ := time.Parse(time.RFC3339, t)
 					infof("max gbps for cdn %v = %v", cdn, value)
-					var statsSummary tc.StatsSummary
+					var statsSummary tc.StatsSummaryV5
 					statsSummary.CDNName = util.StrPtr(cdn)
 					statsSummary.DeliveryService = util.StrPtr("all")
 					statsSummary.StatName = util.StrPtr("daily_maxgbps")
@@ -677,7 +682,7 @@ func calcDailyBytesServed(client influx.Client, bp influx.BatchPoints, startTime
 			bytesServedTB := bytesServed / bytesToTerabytes
 			infof("TBytes served for cdn %v = %v", cdn, bytesServedTB)
 			//write to Traffic Ops
-			var statsSummary tc.StatsSummary
+			var statsSummary tc.StatsSummaryV5
 			statsSummary.CDNName = util.StrPtr(cdn)
 			statsSummary.DeliveryService = util.StrPtr("all")
 			statsSummary.StatName = util.StrPtr("daily_bytesserved")
@@ -720,14 +725,14 @@ func queryDB(con influx.Client, cmd string, database string) (res []influx.Resul
 	return
 }
 
-func writeSummaryStats(config StartupConfig, statsSummary tc.StatsSummary) {
-	to, _, err := client.LoginWithAgent(config.ToURL, config.ToUser, config.ToPasswd, true, UserAgent, false, TrafficOpsRequestTimeout)
+func writeSummaryStats(config StartupConfig, statsSummary tc.StatsSummaryV5) {
+	to, _, err := client.LoginWithAgent(config.ToURL, config.ToUser, config.ToPasswd, true, UserAgent, false, time.Duration(config.ToRequestTimeoutSeconds)*time.Second)
 	if err != nil {
 		newErr := fmt.Errorf("Could not store summary stats! Error logging in to %v: %v", config.ToURL, err)
 		errorln(newErr)
 		return
 	}
-	_, _, err = to.CreateSummaryStats(statsSummary)
+	_, _, err = to.CreateSummaryStats(statsSummary, client.RequestOptions{})
 	if err != nil {
 		errorf("could not create summary stats: %v", err)
 	}
@@ -735,7 +740,7 @@ func writeSummaryStats(config StartupConfig, statsSummary tc.StatsSummary) {
 
 func getToData(config StartupConfig, init bool, configChan chan RunningConfig) {
 	var runningConfig RunningConfig
-	to, _, err := client.LoginWithAgent(config.ToURL, config.ToUser, config.ToPasswd, true, UserAgent, false, TrafficOpsRequestTimeout)
+	to, _, err := client.LoginWithAgent(config.ToURL, config.ToUser, config.ToPasswd, true, UserAgent, false, time.Duration(config.ToRequestTimeoutSeconds)*time.Second)
 	if err != nil {
 		msg := fmt.Sprintf("Error logging in to %v: %v", config.ToURL, err)
 		if init {
@@ -745,7 +750,7 @@ func getToData(config StartupConfig, init bool, configChan chan RunningConfig) {
 		return
 	}
 
-	servers, _, err := to.GetServers(nil)
+	servers, _, err := to.GetServers(client.RequestOptions{})
 	if err != nil {
 		msg := fmt.Sprintf("Error getting server list from %v: %v ", config.ToURL, err)
 		if init {
@@ -755,14 +760,14 @@ func getToData(config StartupConfig, init bool, configChan chan RunningConfig) {
 		return
 	}
 
-	runningConfig.CacheMap = make(map[string]tc.Server)
-	for _, server := range servers {
+	runningConfig.CacheMap = make(map[string]tc.ServerV5)
+	for _, server := range servers.Response {
 		runningConfig.CacheMap[server.HostName] = server
 	}
 
 	cacheStatPath := "/publish/CacheStats?hc=1&wildcard=1&stats="
 	dsStatPath := "/publish/DsStats?hc=1&wildcard=1&stats="
-	parameters, _, err := to.GetParametersByProfileName("TRAFFIC_STATS")
+	parameters, _, err := to.GetParametersByProfileName("TRAFFIC_STATS", client.RequestOptions{})
 	if err != nil {
 		msg := fmt.Sprintf("Error getting parameter list from %v: %v", config.ToURL, err)
 		if init {
@@ -772,7 +777,7 @@ func getToData(config StartupConfig, init bool, configChan chan RunningConfig) {
 		return
 	}
 
-	for _, param := range parameters {
+	for _, param := range parameters.Response {
 		if param.Name == "DsStats" {
 			statName := param.Value
 			dsStatPath += "," + statName
@@ -785,7 +790,13 @@ func getToData(config StartupConfig, init bool, configChan chan RunningConfig) {
 
 	setHealthURLs(config, &runningConfig, cacheStatPath, dsStatPath)
 
-	lastSummaryTimeResponse, _, err := to.GetSummaryStatsLastUpdated(util.StrPtr("daily_maxgbps"))
+	opts := client.RequestOptions{
+		QueryParameters: url.Values{
+			"lastSummaryDate": {"true"},
+			"statName":        {"daily_maxgbps"},
+		},
+	}
+	lastSummaryTimeResponse, _, err := to.GetSummaryStatsLastUpdated(opts)
 	if err != nil {
 		errorf("unable to get summary stats last updated: %v", err)
 	} else if lastSummaryTimeResponse.Response.SummaryTime == nil {
@@ -806,7 +817,7 @@ func setHealthURLs(config StartupConfig, runningConfig *RunningConfig, cacheStat
 		}
 
 		if server.Type == tc.MonitorTypeName && server.Status == config.StatusToMon {
-			cdnName := server.CDNName
+			cdnName := server.CDN
 			if cdnName == "" {
 				errorln("Unable to find CDN name for " + server.HostName + ".. skipping")
 				continue
@@ -815,15 +826,16 @@ func setHealthURLs(config StartupConfig, runningConfig *RunningConfig, cacheStat
 			if runningConfig.HealthUrls[cdnName] == nil {
 				runningConfig.HealthUrls[cdnName] = make(map[string][]string)
 			}
-			healthURL := "http://" + server.HostName + "." + server.DomainName + ":" + strconv.Itoa(server.TCPPort) + cacheStatPath
+			tcpPort := util.CoalesceToDefault(server.TCPPort)
+			healthURL := "http://" + server.HostName + "." + server.DomainName + ":" + strconv.Itoa(tcpPort) + cacheStatPath
 			runningConfig.HealthUrls[cdnName]["CacheStats"] = append(runningConfig.HealthUrls[cdnName]["CacheStats"], healthURL)
-			healthURL = "http://" + server.HostName + "." + server.DomainName + ":" + strconv.Itoa(server.TCPPort) + dsStatPath
+			healthURL = "http://" + server.HostName + "." + server.DomainName + ":" + strconv.Itoa(tcpPort) + dsStatPath
 			runningConfig.HealthUrls[cdnName]["DsStats"] = append(runningConfig.HealthUrls[cdnName]["DsStats"], healthURL)
 		}
 	}
 }
 
-func calcMetrics(cdnName string, urls []string, cacheMap map[string]tc.Server, config StartupConfig) {
+func calcMetrics(cdnName string, urls []string, cacheMap map[string]tc.ServerV5, config StartupConfig) {
 	sampleTime := time.Now().Unix()
 	// get the data from trafficMonitor
 	var trafMonData []byte
@@ -947,7 +959,7 @@ func calcDsValues(tmData []byte, cdnName string, sampleTime int64, config Startu
 	return nil
 }
 
-func calcCacheValues(trafmonData []byte, cdnName string, sampleTime int64, cacheMap map[string]tc.Server, config StartupConfig) error {
+func calcCacheValues(trafmonData []byte, cdnName string, sampleTime int64, cacheMap map[string]tc.ServerV5, config StartupConfig) error {
 	var jData tc.LegacyStats
 	err := json.Unmarshal(trafmonData, &jData)
 	if err != nil {
@@ -993,7 +1005,7 @@ func calcCacheValues(trafmonData []byte, cdnName string, sampleTime int64, cache
 			}
 
 			tags := map[string]string{
-				"cachegroup": cache.Cachegroup,
+				"cachegroup": cache.CacheGroup,
 				"hostname":   string(cacheName),
 				"cdn":        cdnName,
 				"type":       cache.Type,
@@ -1041,6 +1053,18 @@ func influxConnect(config StartupConfig) (influx.Client, error) {
 		host := hosts[n]
 		hosts = append(hosts[:n], hosts[n+1:]...)
 		parsedURL, _ := url.Parse(host.URL)
+		if host.InfluxClient != nil && parsedURL.Scheme == "http" {
+			// NOTE: closing an http client just closes idle connections -- the client can still make new requests
+			if err := host.InfluxClient.Close(); err != nil {
+				errorf("closing http influx client: %s", err)
+			}
+			_, _, err := host.InfluxClient.Ping(10)
+			if err != nil {
+				warnf("pinging InfluxDB: %v", err)
+				continue
+			}
+			return host.InfluxClient, nil
+		}
 		if parsedURL.Scheme == "udp" {
 			conf := influx.UDPConfig{
 				Addr: parsedURL.Host,
@@ -1063,12 +1087,6 @@ func influxConnect(config StartupConfig) (influx.Client, error) {
 			errorf("An error occurred creating InfluxDB HTTP client: %v", err)
 			continue
 		}
-		//Close old connections explicitly
-		if host.InfluxClient != nil {
-			if err := host.InfluxClient.Close(); err != nil {
-				errorf("closing influx client: %s", err)
-			}
-		}
 		host.InfluxClient = con
 		_, _, err = con.Ping(10)
 		if err != nil {
@@ -1077,7 +1095,7 @@ func influxConnect(config StartupConfig) (influx.Client, error) {
 		}
 		return con, nil
 	}
-	err := errors.New("Could not connect to any of the InfluxDb servers defined in the influxUrls config.")
+	err := errors.New("could not connect to any of the InfluxDb servers defined in the influxUrls config")
 	return nil, err
 }
 
